@@ -28,6 +28,7 @@
 
 #include <simgear/compiler.h>
 
+#include <boost/foreach.hpp>
 #include <string>
 
 #include <osg/ref_ptr>
@@ -43,6 +44,7 @@
 #include <Main/globals.hxx>
 #include <Scenery/scenery.hxx>
 #include <Scripting/NasalSys.hxx>
+#include <Scripting/NasalModelData.hxx>
 #include <Sound/fg_fx.hxx>
 
 #include "AIBase.hxx"
@@ -52,8 +54,54 @@ const char *default_model = "Models/Geometry/glider.ac";
 const double FGAIBase::e = 2.71828183;
 const double FGAIBase::lbs_to_slugs = 0.031080950172;   //conversion factor
 
+using std::string;
 using namespace simgear;
 
+class FGAIModelData : public simgear::SGModelData {
+public:
+    FGAIModelData(SGPropertyNode *root = NULL)
+        : _nasal( new FGNasalModelDataProxy(root) ),
+        _ready(false),
+        _initialized(false)
+    {
+    }
+
+    
+    ~FGAIModelData()
+    {
+    }
+    
+    virtual FGAIModelData* clone() const { return new FGAIModelData(); }
+
+    /** osg callback, thread-safe */
+    void modelLoaded(const std::string& path, SGPropertyNode *prop, osg::Node *n)
+    {
+        // WARNING: Called in a separate OSG thread! Only use thread-safe stuff here...
+        if (_ready)
+            return;
+        
+        _fxpath = prop->getStringValue("sound/path");
+        _nasal->modelLoaded(path, prop, n);
+        
+        _ready = true;
+
+    }
+    
+    /** init hook to be called after model is loaded.
+     * Not thread-safe. Call from main thread only. */
+    void init(void) { _initialized = true; }
+    
+    bool needInitilization(void) { return _ready && !_initialized;}
+    bool isInitialized(void) { return _initialized;}
+    inline std::string& get_sound_path() { return _fxpath;}
+    
+private:
+    std::auto_ptr<FGNasalModelDataProxy> _nasal;
+    std::string _fxpath;
+    bool _ready;
+    bool _initialized;
+};
+
 FGAIBase::FGAIBase(object_type ot, bool enableHot) :
     _max_speed(300),
     _name(""),
@@ -99,8 +147,6 @@ FGAIBase::FGAIBase(object_type ot, bool enableHot) :
     _roll_offset = 0;
     _yaw_offset = 0;
 
-    userpos = SGGeod::fromDeg(0, 0);
-
     pos = SGGeod::fromDeg(0, 0);
     speed = 0;
     altitude_ft = 0;
@@ -163,8 +209,10 @@ FGAIBase::removeModel()
         osg::ref_ptr<osg::Object> temp = _model.get();
         pSceneryManager->get_scene_graph()->removeChild(aip.getSceneGraph());
         // withdraw from SGModelPlacement and drop own reference (unref)
-        aip.init( 0 );
+        aip.clear();
+        _modeldata = 0;
         _model = 0;
+        
         // pass it on to the pager, to be be deleted in the pager thread
         pSceneryManager->getPager()->queueDeleteRequest(temp);
     }
@@ -298,21 +346,17 @@ bool FGAIBase::init(bool search_in_AI_path)
     string f;
     if(search_in_AI_path)
     {
-    // setup a modified Options structure, with only the $fg-root/AI defined;
-    // we'll check that first, then give the normal search logic a chance.
-    // this ensures that models in AI/ are preferred to normal models, where
-    // both exist.
-        osg::ref_ptr<osgDB::ReaderWriter::Options> 
-          opt(osg::clone(osgDB::Registry::instance()->getOptions(), osg::CopyOp::SHALLOW_COPY));
-
-        SGPath ai_path(globals->get_fg_root(), "AI");
-        opt->setDatabasePath(ai_path.str());
-        
-        f = osgDB::findDataFile(model_path, opt.get());
-    }
+        BOOST_FOREACH(SGPath p, globals->get_data_paths("AI")) {
+            p.append(model_path);
+            if (p.exists()) {
+                f = p.str();
+                break;
+            }
+        } // of AI data paths iteration
+    } // of search in AI path
 
     if (f.empty()) {
-      f = simgear::SGModelLib::findDataFile(model_path);
+        f = simgear::SGModelLib::findDataFile(model_path);
     }
     
     if(f.empty())
@@ -478,106 +522,68 @@ void FGAIBase::removeSoundFx() {
     }
 }
 
-double FGAIBase::UpdateRadar(FGAIManager* manager) {
+double FGAIBase::UpdateRadar(FGAIManager* manager)
+{
     bool control = fgGetBool("/sim/controls/radar", true);
 
     if(!control) return 0;
 
-    double radar_range_ft2 = fgGetDouble("/instrumentation/radar/range");
+    double radar_range_m = fgGetDouble("/instrumentation/radar/range");
     bool force_on = fgGetBool("/instrumentation/radar/debug-mode", false);
-    radar_range_ft2 *= SG_NM_TO_METER * SG_METER_TO_FEET * 1.1; // + 10%
-    radar_range_ft2 *= radar_range_ft2;
-
-    double user_latitude  = manager->get_user_latitude();
-    double user_longitude = manager->get_user_longitude();
-    double lat_range = fabs(pos.getLatitudeDeg() - user_latitude) * ft_per_deg_lat;
-    double lon_range = fabs(pos.getLongitudeDeg() - user_longitude) * ft_per_deg_lon;
-    double range_ft2 = lat_range*lat_range + lon_range*lon_range;
-
-    //
-    // Test whether the target is within radar range.
-    //
-    in_range = (range_ft2 && (range_ft2 <= radar_range_ft2));
-
-    if ( in_range || force_on ) {
-        props->setBoolValue("radar/in-range", true);
-
-        // copy values from the AIManager
-        double user_altitude  = manager->get_user_altitude();
-        double user_heading   = manager->get_user_heading();
-        double user_pitch     = manager->get_user_pitch();
-        //double user_yaw       = manager->get_user_yaw();
-        //double user_speed     = manager->get_user_speed();
-
-        // calculate range to target in feet and nautical miles
-        double range_ft = sqrt( range_ft2 );
-        range = range_ft / 6076.11549;
-
-        // calculate bearing to target
-        if (pos.getLatitudeDeg() >= user_latitude) {
-            bearing = atan2(lat_range, lon_range) * SG_RADIANS_TO_DEGREES;
-            if (pos.getLongitudeDeg() >= user_longitude) {
-                bearing = 90.0 - bearing;
-            } else {
-                bearing = 270.0 + bearing;
-            }
-        } else {
-            bearing = atan2(lon_range, lat_range) * SG_RADIANS_TO_DEGREES;
-            if (pos.getLongitudeDeg() >= user_longitude) {
-                bearing = 180.0 - bearing;
-            } else {
-                bearing = 180.0 + bearing;
-            }
-        }
-
-        // This is an alternate way to compute bearing and distance which
-        // agrees with the original scheme within about 0.1 degrees.
-        //
-        // Point3D start( user_longitude * SGD_DEGREES_TO_RADIANS,
-        //                user_latitude * SGD_DEGREES_TO_RADIANS, 0 );
-        // Point3D dest( pos.getLongitudeRad(), pos.getLatitudeRad(), 0 );
-        // double gc_bearing, gc_range;
-        // calc_gc_course_dist( start, dest, &gc_bearing, &gc_range );
-        // gc_range *= SG_METER_TO_NM;
-        // gc_bearing *= SGD_RADIANS_TO_DEGREES;
-        // printf("orig b = %.3f %.2f  gc b= %.3f, %.2f\n",
-        //        bearing, range, gc_bearing, gc_range);
-
-        // calculate look left/right to target, without yaw correction
-        horiz_offset = bearing - user_heading;
-        if (horiz_offset > 180.0) horiz_offset -= 360.0;
-        if (horiz_offset < -180.0) horiz_offset += 360.0;
-
-        // calculate elevation to target
-        elevation = atan2( altitude_ft - user_altitude, range_ft ) * SG_RADIANS_TO_DEGREES;
-
-        // calculate look up/down to target
-        vert_offset = elevation - user_pitch;
-
-        /* this calculation needs to be fixed, but it isn't important anyway
-        // calculate range rate
-        double recip_bearing = bearing + 180.0;
-        if (recip_bearing > 360.0) recip_bearing -= 360.0;
-        double my_horiz_offset = recip_bearing - hdg;
-        if (my_horiz_offset > 180.0) my_horiz_offset -= 360.0;
-        if (my_horiz_offset < -180.0) my_horiz_offset += 360.0;
-        rdot = (-user_speed * cos( horiz_offset * SG_DEGREES_TO_RADIANS ))
-        +(-speed * 1.686 * cos( my_horiz_offset * SG_DEGREES_TO_RADIANS ));
-        */
-
-        // now correct look left/right for yaw
-        // horiz_offset += user_yaw; // FIXME: WHY WOULD WE WANT TO ADD IN SIDE-SLIP HERE?
-
-        // calculate values for radar display
-        y_shift = range * cos( horiz_offset * SG_DEGREES_TO_RADIANS);
-        x_shift = range * sin( horiz_offset * SG_DEGREES_TO_RADIANS);
-        rotation = hdg - user_heading;
-        if (rotation < 0.0) rotation += 360.0;
-        ht_diff = altitude_ft - user_altitude;
-
+    radar_range_m *= SG_NM_TO_METER  * 1.1; // + 10%
+    radar_range_m *= radar_range_m; // squared
+    
+    double d2 = distSqr(SGVec3d::fromGeod(pos), globals->get_aircraft_position_cart());
+    double range_ft = sqrt(d2) * SG_METER_TO_FEET;
+    
+    if (!force_on && (d2 > radar_range_m)) {
+        return range_ft * range_ft;
     }
+    
+    props->setBoolValue("radar/in-range", true);
 
-    return range_ft2;
+    // copy values from the AIManager
+    double user_heading   = manager->get_user_heading();
+    double user_pitch     = manager->get_user_pitch();
+  
+    range = range_ft * SG_FEET_TO_METER * SG_METER_TO_NM;
+
+    // calculate bearing to target
+    bearing = SGGeodesy::courseDeg(globals->get_aircraft_position(), pos);
+
+    // calculate look left/right to target, without yaw correction
+    horiz_offset = bearing - user_heading;
+    SG_NORMALIZE_RANGE(horiz_offset, -180.0, 180.0);
+   
+    // calculate elevation to target
+    ht_diff = altitude_ft - globals->get_aircraft_position().getElevationFt();
+    elevation = atan2( ht_diff, range_ft ) * SG_RADIANS_TO_DEGREES;
+
+    // calculate look up/down to target
+    vert_offset = elevation - user_pitch;
+
+    /* this calculation needs to be fixed, but it isn't important anyway
+    // calculate range rate
+    double recip_bearing = bearing + 180.0;
+    if (recip_bearing > 360.0) recip_bearing -= 360.0;
+    double my_horiz_offset = recip_bearing - hdg;
+    if (my_horiz_offset > 180.0) my_horiz_offset -= 360.0;
+    if (my_horiz_offset < -180.0) my_horiz_offset += 360.0;
+    rdot = (-user_speed * cos( horiz_offset * SG_DEGREES_TO_RADIANS ))
+    +(-speed * 1.686 * cos( my_horiz_offset * SG_DEGREES_TO_RADIANS ));
+    */
+
+    // now correct look left/right for yaw
+    // horiz_offset += user_yaw; // FIXME: WHY WOULD WE WANT TO ADD IN SIDE-SLIP HERE?
+
+    // calculate values for radar display
+    y_shift = range * cos( horiz_offset * SG_DEGREES_TO_RADIANS);
+    x_shift = range * sin( horiz_offset * SG_DEGREES_TO_RADIANS);
+    
+    rotation = hdg - user_heading;
+    SG_NORMALIZE_RANGE(rotation, 0.0, 360.0);
+
+    return range_ft * range_ft;
 }
 
 /*
@@ -636,12 +642,6 @@ void FGAIBase::_setLatitude ( double latitude )  {
     pos.setLatitudeDeg(latitude);
 }
 
-void FGAIBase::_setUserPos(){
-    userpos.setLatitudeDeg(manager->get_user_latitude());
-    userpos.setLongitudeDeg(manager->get_user_longitude());
-    userpos.setElevationM(manager->get_user_altitude() * SG_FEET_TO_METER);
-}
-
 void FGAIBase::_setSubID( int s ) {
     _subID = s;
 }
@@ -888,27 +888,4 @@ int FGAIBase::_newAIModelID() {
 }
 
 
-FGAIModelData::FGAIModelData(SGPropertyNode *root)
-  : _nasal( new FGNasalModelDataProxy(root) ),
-    _ready(false),
-    _initialized(false)
-{
-}
 
-FGAIModelData::~FGAIModelData()
-{
-    delete _nasal;
-    _nasal = NULL;
-}
-
-void FGAIModelData::modelLoaded(const string& path, SGPropertyNode *prop, osg::Node *n)
-{
-    // WARNING: Called in a separate OSG thread! Only use thread-safe stuff here...
-    if (_ready)
-        return;
-
-    _fxpath = prop->getStringValue("sound/path");
-    _nasal->modelLoaded(path, prop, n);
-
-    _ready = true;
-}
