@@ -17,6 +17,8 @@
 #  include "config.h"
 #endif
 
+#include <cstdio>
+
 #include <simgear/misc/sg_path.hxx>
 #include <simgear/xml/easyxml.hxx>
 #include <simgear/misc/strutils.hxx>
@@ -30,7 +32,7 @@
 #include "runwayprefloader.hxx"
 
 #include "dynamics.hxx"
-#include "simple.hxx"
+#include "airport.hxx"
 #include "runwayprefs.hxx"
 
 #include <Navaids/NavDataCache.hxx>
@@ -48,7 +50,7 @@ void XMLLoader::load(FGAirportDynamics* d)
   }
 
   flightgear::NavDataCache* cache = flightgear::NavDataCache::instance();
-  if (!cache->isCachedFileModified(path)) {
+  if (!cache->isCachedFileModified(path) || cache->isReadOnly()) {
     return;
   }
   
