@@ -90,9 +90,9 @@ public:
 
   /** Get the altitude above sea level dependent on the location. */
   virtual double GetAltitude(const FGLocation& l) const {
-    double pt[3] = { SG_FEET_TO_METER*l(eX),
-                     SG_FEET_TO_METER*l(eY),
-                     SG_FEET_TO_METER*l(eZ) };
+    double pt[3] = { SG_FEET_TO_METER*l(FGJSBBase::eX),
+                     SG_FEET_TO_METER*l(FGJSBBase::eY),
+                     SG_FEET_TO_METER*l(FGJSBBase::eZ) };
     double lat, lon, alt;
     sgCartToGeod( pt, &lat, &lon, &alt);
     return alt * SG_METER_TO_FEET;
@@ -102,7 +102,7 @@ public:
   virtual double GetAGLevel(double t, const FGLocation& l,
                             FGLocation& cont, FGColumnVector3& n,
                             FGColumnVector3& v, FGColumnVector3& w) const {
-    double loc_cart[3] = { l(eX), l(eY), l(eZ) };
+    double loc_cart[3] = { l(FGJSBBase::eX), l(FGJSBBase::eY), l(FGJSBBase::eZ) };
     double contact[3], normal[3], vel[3], angularVel[3], agl = 0;
     mInterface->get_agl_ft(t, loc_cart, SG_METER_TO_FEET*2, contact, normal,
                            vel, angularVel, &agl);
@@ -114,7 +114,7 @@ public:
   }
 
   virtual double GetTerrainGeoCentRadius(double t, const FGLocation& l) const {
-    double loc_cart[3] = { l(eX), l(eY), l(eZ) };
+    double loc_cart[3] = { l(FGJSBBase::eX), l(FGJSBBase::eY), l(FGJSBBase::eZ) };
     double contact[3], normal[3], vel[3], angularVel[3], agl = 0;
     mInterface->get_agl_ft(t, loc_cart, SG_METER_TO_FEET*2, contact, normal,
                            vel, angularVel, &agl);
@@ -167,24 +167,22 @@ FGJSBsim::FGJSBsim( double dt )
         TURBULENCE_TYPE_NAMES["ttTustin"]   = FGWinds::ttTustin;
     }
 
-                                // Set up the debugging level
-                                // FIXME: this will not respond to
-                                // runtime changes
+    // Set up the debugging level
+    // FIXME: this will not respond to
+    // runtime changes
 
                                 // if flight is excluded, don't bother
-    if ((logbuf::get_log_classes() & SG_FLIGHT) != 0) {
+    if ((sglog().get_log_classes() & SG_FLIGHT) != 0) {
 
                                 // do a rough-and-ready mapping to
                                 // the levels documented in FGFDMExec.h
-        switch (logbuf::get_log_priority()) {
+        switch (sglog().get_log_priority()) {
         case SG_BULK:
             FGJSBBase::debug_lvl = 0x1f;
             break;
         case SG_DEBUG:
-            FGJSBBase::debug_lvl = 0x0f;
+            FGJSBBase::debug_lvl = 0x1f;
         case SG_INFO:
-            FGJSBBase::debug_lvl = 0x01;
-            break;
         case SG_WARN:
         case SG_ALERT:
             FGJSBBase::debug_lvl = 0x00;
@@ -192,7 +190,8 @@ FGJSBsim::FGJSBsim( double dt )
         }
     }
 
-    fdmex = new FGFDMExec( (FGPropertyManager*)globals->get_props() );
+    PropertyManager = new FGPropertyManager( (FGPropertyNode*)globals->get_props() );
+    fdmex = new FGFDMExec( PropertyManager );
 
     // Register ground callback.
     fdmex->SetGroundCallback( new FGFSGroundCallback(this) );
@@ -363,6 +362,7 @@ FGJSBsim::FGJSBsim( double dt )
 FGJSBsim::~FGJSBsim(void)
 {
   delete fdmex;
+  delete PropertyManager;
 }
 
 /******************************************************************************/
@@ -758,9 +758,9 @@ bool FGJSBsim::copy_from_JSBsim()
                            Propagate->GetVel(FGJSBBase::eEast),
                            Propagate->GetVel(FGJSBBase::eDown) );
 
-    _set_Velocities_Wind_Body( Propagate->GetUVW(1),
-                               Propagate->GetUVW(2),
-                               Propagate->GetUVW(3) );
+    _set_Velocities_Body( Propagate->GetUVW(1),
+                          Propagate->GetUVW(2),
+                          Propagate->GetUVW(3) );
 
     // Make the HUD work ...
     _set_Velocities_Ground( Propagate->GetVel(FGJSBBase::eNorth),
@@ -1122,9 +1122,9 @@ void FGJSBsim::set_Velocities_Local( double north, double east, double down )
   FGInterface::set_Velocities_Local(north, east, down);
 }
 
-void FGJSBsim::set_Velocities_Wind_Body( double u, double v, double w)
+void FGJSBsim::set_Velocities_Body( double u, double v, double w)
 {
-  SG_LOG(SG_FLIGHT,SG_INFO, "FGJSBsim::set_Velocities_Wind_Body: "
+  SG_LOG(SG_FLIGHT,SG_INFO, "FGJSBsim::set_Velocities_Body: "
      << u << ", " <<  v << ", " <<  w );
 
   if (needTrim) {
@@ -1138,7 +1138,7 @@ void FGJSBsim::set_Velocities_Wind_Body( double u, double v, double w)
     Propagate->SetUVW(3, w);
   }
 
-  FGInterface::set_Velocities_Wind_Body(u, v, w);
+  FGInterface::set_Velocities_Body(u, v, w);
 }
 
 //Euler angles
@@ -1215,9 +1215,9 @@ void FGJSBsim::init_gear(void )
     for (int i=0;i<Ngear;i++) {
       FGLGear *gear = gr->GetGearUnit(i);
       SGPropertyNode * node = fgGetNode("gear/gear", i, true);
-      node->setDoubleValue("xoffset-in", gear->GetBodyLocation()(1));
-      node->setDoubleValue("yoffset-in", gear->GetBodyLocation()(2));
-      node->setDoubleValue("zoffset-in", gear->GetBodyLocation()(3));
+      node->setDoubleValue("xoffset-in", gear->GetBodyLocation()(1) * 12);
+      node->setDoubleValue("yoffset-in", gear->GetBodyLocation()(2) * 12);
+      node->setDoubleValue("zoffset-in", gear->GetBodyLocation()(3) * 12);
       node->setBoolValue("wow", gear->GetWOW());
       node->setDoubleValue("rollspeed-ms", gear->GetWheelRollVel()*0.3043);
       node->setBoolValue("has-brake", gear->GetBrakeGroup() > 0);
