@@ -58,12 +58,14 @@ INCLUDES
 #include "models/FGFCS.h"
 #include "input_output/FGPropertyManager.h"
 #include "input_output/string_utilities.h"
+#include "input_output/FGXMLFileRead.h"
+#include "input_output/FGXMLElement.h"
 
 using namespace std;
 
 namespace JSBSim {
 
-static const char *IdSrc = "$Id: FGInitialCondition.cpp,v 1.81 2012/04/08 15:22:56 jberndt Exp $";
+static const char *IdSrc = "$Id: FGInitialCondition.cpp,v 1.89 2013/11/24 11:40:55 bcoconni Exp $";
 static const char *IdHdr = ID_INITIALCONDITION;
 
 //******************************************************************************
@@ -75,9 +77,7 @@ FGInitialCondition::FGInitialCondition(FGFDMExec *FDMExec) : fdmex(FDMExec)
   if(FDMExec != NULL ) {
     PropertyManager=fdmex->GetPropertyManager();
     Atmosphere=fdmex->GetAtmosphere();
-    Constructing = true;
     bind();
-    Constructing = false;
   } else {
     cout << "FGInitialCondition: This class requires a pointer to a valid FGFDMExec object" << endl;
   }
@@ -111,7 +111,7 @@ void FGInitialCondition::ResetIC(double u0, double v0, double w0,
 
   position.SetLongitude(lonRad0);
   position.SetLatitude(latRad0);
-  position.SetAltitudeAGL(altAGLFt0, fdmex->GetSimTime());
+  position.SetAltitudeAGL(altAGLFt0, 0.0);
 
   orientation = FGQuaternion(phi0, theta0, psi0);
   const FGMatrix33& Tb2l = orientation.GetTInv();
@@ -151,41 +151,7 @@ void FGInitialCondition::InitializeIC(void)
 
   lastSpeedSet = setvt;
   lastAltitudeSet = setasl;
-}
-
-//******************************************************************************
-
-void FGInitialCondition::WriteStateFile(int num)
-{
-  if (Constructing) return;
-
-  string filename = fdmex->GetFullAircraftPath();
-
-  if (filename.empty())
-    filename = "initfile.xml";
-  else
-    filename.append("/initfile.xml");
-
-  ofstream outfile(filename.c_str());
-  FGPropagate* Propagate = fdmex->GetPropagate();
-
-  if (outfile.is_open()) {
-    outfile << "<?xml version=\"1.0\"?>" << endl;
-    outfile << "<initialize name=\"reset00\">" << endl;
-    outfile << "  <ubody unit=\"FT/SEC\"> " << Propagate->GetUVW(eU) << " </ubody> " << endl;
-    outfile << "  <vbody unit=\"FT/SEC\"> " << Propagate->GetUVW(eV) << " </vbody> " << endl;
-    outfile << "  <wbody unit=\"FT/SEC\"> " << Propagate->GetUVW(eW) << " </wbody> " << endl;
-    outfile << "  <phi unit=\"DEG\"> " << Propagate->GetEuler(ePhi)*radtodeg << " </phi>" << endl;
-    outfile << "  <theta unit=\"DEG\"> " << Propagate->GetEuler(eTht)*radtodeg << " </theta>" << endl;
-    outfile << "  <psi unit=\"DEG\"> " << Propagate->GetEuler(ePsi)*radtodeg << " </psi>" << endl;
-    outfile << "  <longitude unit=\"DEG\"> " << Propagate->GetLongitudeDeg() << " </longitude>" << endl;
-    outfile << "  <latitude unit=\"DEG\"> " << Propagate->GetLatitudeDeg() << " </latitude>" << endl;
-    outfile << "  <altitude unit=\"FT\"> " << Propagate->GetDistanceAGL() << " </altitude>" << endl;
-    outfile << "</initialize>" << endl;
-    outfile.close();
-  } else {
-    cerr << "Could not open and/or write the state to the initial conditions file: " << filename << endl;
-  }
+  enginesRunning.clear();
 }
 
 //******************************************************************************
@@ -701,22 +667,22 @@ void FGInitialCondition::SetTerrainElevationFtIC(double elev)
 
 double FGInitialCondition::GetAltitudeAGLFtIC(void) const
 {
-  return position.GetAltitudeAGL(fdmex->GetSimTime());
+  return position.GetAltitudeAGL(0.0);
 }
 
 //******************************************************************************
 
 double FGInitialCondition::GetTerrainElevationFtIC(void) const
 {
-  return position.GetTerrainRadius(fdmex->GetSimTime())
-       - position.GetSeaLevelRadius();
+  return position.GetTerrainRadius(0.0) - position.GetSeaLevelRadius();
 }
 
 //******************************************************************************
 
 void FGInitialCondition::SetAltitudeAGLFtIC(double agl)
 {
-  double terrainElevation = position.GetTerrainRadius(fdmex->GetSimTime()) - position.GetSeaLevelRadius();
+  double terrainElevation = position.GetTerrainRadius(0.0)
+    - position.GetSeaLevelRadius();
   SetAltitudeASLFtIC(agl + terrainElevation);
   lastAltitudeSet = setagl;
 }
@@ -899,14 +865,15 @@ double FGInitialCondition::GetBodyVelFpsIC(int idx) const
 
 bool FGInitialCondition::Load(string rstfile, bool useStoredPath)
 {
-  string sep = "/";
+  string init_file_name;
   if( useStoredPath ) {
-    init_file_name = fdmex->GetFullAircraftPath() + sep + rstfile + ".xml";
+    init_file_name = fdmex->GetFullAircraftPath() + "/" + rstfile + ".xml";
   } else {
     init_file_name = rstfile;
   }
 
-  document = LoadXMLDocument(init_file_name);
+  FGXMLFileRead XMLFileRead;
+  Element* document = XMLFileRead.LoadXMLDocument(init_file_name);
 
   // Make sure that the document is valid
   if (!document) {
@@ -923,29 +890,20 @@ bool FGInitialCondition::Load(string rstfile, bool useStoredPath)
   bool result = false;
 
   if (version == HUGE_VAL) {
-    result = Load_v1(); // Default to the old version
+    result = Load_v1(document); // Default to the old version
   } else if (version >= 3.0) {
     cerr << "Only initialization file formats 1 and 2 are currently supported" << endl;
     exit (-1);
   } else if (version >= 2.0) {
-    result = Load_v2();
+    result = Load_v2(document);
   } else if (version >= 1.0) {
-    result = Load_v1();
+    result = Load_v1(document);
   }
 
-  fdmex->RunIC();
-
   // Check to see if any engines are specified to be initialized in a running state
-  FGPropulsion* propulsion = fdmex->GetPropulsion();
   Element* running_elements = document->FindElement("running");
   while (running_elements) {
-    int n = int(running_elements->GetDataAsNumber());
-    try {
-      propulsion->InitRunning(n);
-    } catch (string str) {
-      cerr << str << endl;
-      result = false;
-    }
+    enginesRunning.push_back(int(running_elements->GetDataAsNumber()));
     running_elements = document->FindNextElement("running");
   }
 
@@ -954,7 +912,7 @@ bool FGInitialCondition::Load(string rstfile, bool useStoredPath)
 
 //******************************************************************************
 
-bool FGInitialCondition::Load_v1(void)
+bool FGInitialCondition::Load_v1(Element* document)
 {
   bool result = true;
 
@@ -1040,7 +998,7 @@ bool FGInitialCondition::Load_v1(void)
 
 //******************************************************************************
 
-bool FGInitialCondition::Load_v2(void)
+bool FGInitialCondition::Load_v2(Element* document)
 {
   FGColumnVector3 vOrient;
   bool result = true;
@@ -1067,14 +1025,11 @@ bool FGInitialCondition::Load_v2(void)
         if (position_el->FindElement("longitude"))
           position.SetLongitude(position_el->FindElementValueAsNumberConvertTo("longitude", "RAD"));
 
-        if (position_el->FindElement("latitude"))
-          position.SetLatitude(position_el->FindElementValueAsNumberConvertTo("latitude", "RAD"));
-
         if (position_el->FindElement("radius")) {
           position.SetRadius(position_el->FindElementValueAsNumberConvertTo("radius", "FT"));
         } else if (position_el->FindElement("altitudeAGL")) {
           position.SetAltitudeAGL(position_el->FindElementValueAsNumberConvertTo("altitudeAGL", "FT"),
-                                  fdmex->GetSimTime());
+                                  0.0);
         } else if (position_el->FindElement("altitudeMSL")) {
           position.SetAltitudeASL(position_el->FindElementValueAsNumberConvertTo("altitudeMSL", "FT"));
         } else {
@@ -1082,6 +1037,19 @@ bool FGInitialCondition::Load_v2(void)
           result = false;
         }
 
+        Element* latitude_el = position_el->FindElement("latitude");
+        if (latitude_el) {
+          string lat_type = latitude_el->GetAttributeValue("type");
+          double latitude = position_el->FindElementValueAsNumberConvertTo("latitude", "RAD");
+          if (lat_type == "geod" || lat_type == "geodetic") {
+              double longitude = position.GetLongitude();
+              double altitude = position.GetAltitudeASL();                 // SetPositionGeodetic() assumes altitude 
+              position.SetPositionGeodetic(longitude, latitude, altitude); // is geodetic, but it's close enough for now.
+              position.SetAltitudeAGL(altitude, 0.0);
+          } else {
+            position.SetLatitude(latitude);
+          }
+        }
       } else {
         position = position_el->FindElementTripletConvertTo("FT");
       }
@@ -1445,13 +1413,6 @@ void FGInitialCondition::bind(void)
                        &FGInitialCondition::GetRRadpsIC,
                        &FGInitialCondition::SetRRadpsIC,
                        true);
-
-  typedef int (FGInitialCondition::*iPMF)(void) const;
-  PropertyManager->Tie("simulation/write-state-file",
-                       this,
-                       (iPMF)0,
-                       &FGInitialCondition::WriteStateFile);
-
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
