@@ -47,7 +47,6 @@ SENTRY
 INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#include "input_output/FGXMLFileRead.h"
 #include "math/FGLocation.h"
 #include "math/FGQuaternion.h"
 
@@ -55,7 +54,7 @@ INCLUDES
 DEFINITIONS
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#define ID_INITIALCONDITION "$Id: FGInitialCondition.h,v 1.33 2011/12/10 15:49:21 bcoconni Exp $"
+#define ID_INITIALCONDITION "$Id: FGInitialCondition.h,v 1.39 2013/11/24 11:40:55 bcoconni Exp $"
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 FORWARD DECLARATIONS
@@ -67,6 +66,8 @@ class FGFDMExec;
 class FGMatrix33;
 class FGColumnVector3;
 class FGAtmosphere;
+class FGPropertyManager;
+class Element;
 
 typedef enum { setvt, setvc, setve, setmach, setuvw, setned, setvg } speedset;
 typedef enum { setasl, setagl} altitudeset;
@@ -216,14 +217,14 @@ CLASS DOCUMENTATION
    @property ic/r-rad_sec (read/write) Yaw rate initial condition in radians/second
 
    @author Tony Peden
-   @version "$Id: FGInitialCondition.h,v 1.33 2011/12/10 15:49:21 bcoconni Exp $"
+   @version "$Id: FGInitialCondition.h,v 1.39 2013/11/24 11:40:55 bcoconni Exp $"
 */
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 CLASS DECLARATION
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-class FGInitialCondition : public FGJSBBase, public FGXMLFileRead
+class FGInitialCondition : public FGJSBBase
 {
 public:
   /// Constructor
@@ -650,15 +651,17 @@ public:
       @param rstname The name of an initial conditions file
       @param useStoredPath true if the stored path to the IC file should be used
       @return true if successful */
-  bool Load(string rstname, bool useStoredPath = true );
+  bool Load(std::string rstname, bool useStoredPath = true );
 
-  /** Get init-file name
-  */
-  string GetInitFile(void) const {return init_file_name;}
-  /** Set init-file name
-  */
-  void SetInitFile(string f) { init_file_name = f;}
-  void WriteStateFile(int num);
+  /** Get the number of engines running
+   */
+  unsigned int GetNumEnginesRunning(void) const
+  { return (unsigned int)enginesRunning.size(); }
+
+  /** Gets the running engine identification
+      @param engine index of running engine instance
+      @return the identification of running engine instance requested */
+  int GetEngineRunning(unsigned int engine) const { return enginesRunning[engine]; }
 
 private:
   FGColumnVector3 vUVW_NED;
@@ -674,15 +677,14 @@ private:
 
   speedset lastSpeedSet;
   altitudeset lastAltitudeSet;
+  std::vector<int> enginesRunning;
 
   FGFDMExec *fdmex;
   FGPropertyManager *PropertyManager;
   FGAtmosphere* Atmosphere;
 
-  bool Load_v1(void);
-  bool Load_v2(void);
-
-  bool Constructing;
+  bool Load_v1(Element* document);
+  bool Load_v2(Element* document);
 
   void InitializeIC(void);
   void SetEulerAngleRadIC(int idx, double angle);
@@ -695,8 +697,6 @@ private:
   void calcThetaBeta(double alfa, const FGColumnVector3& _vt_NED);
   void bind(void);
   void Debug(int from);
-
-  string init_file_name;
 };
 }
 #endif
