@@ -47,7 +47,7 @@ INCLUDES
 
 #include "FGScript.h"
 #include "input_output/FGXMLElement.h"
-#include "input_output/FGXMLParse.h"
+#include "input_output/FGXMLFileRead.h"
 #include "initialization/FGTrim.h"
 #include "models/FGInput.h"
 
@@ -55,7 +55,7 @@ using namespace std;
 
 namespace JSBSim {
 
-static const char *IdSrc = "$Id: FGScript.cpp,v 1.49 2011/11/10 12:06:14 jberndt Exp $";
+static const char *IdSrc = "$Id: FGScript.cpp,v 1.53 2013/11/24 11:40:55 bcoconni Exp $";
 static const char *IdHdr = ID_FGSCRIPT;
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -99,7 +99,7 @@ FGScript::~FGScript()
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-bool FGScript::LoadScript(string script, double deltaT)
+bool FGScript::LoadScript(string script, double deltaT, const string initfile)
 {
   string aircraft="", initialize="", comparison = "", prop_name="";
   string notifyPropertyName="";
@@ -114,7 +114,8 @@ bool FGScript::LoadScript(string script, double deltaT)
   struct event *newEvent;
   FGCondition *newCondition;
 
-  document = LoadXMLDocument(script);
+  FGXMLFileRead XMLFileRead;
+  Element* document = XMLFileRead.LoadXMLDocument(script);
 
   if (!document) {
     cerr << "File: " << script << " could not be loaded." << endl;
@@ -174,9 +175,15 @@ bool FGScript::LoadScript(string script, double deltaT)
     }
 
     initialize = element->GetAttributeValue("initialize");
+    if (initfile.empty()) {
     if (initialize.empty()) {
       cerr << "Initialization file must be specified in use element." << endl;
       return false;
+      }
+    } else {
+      cout << endl << "The initialization file specified in the script file (" << initialize
+                   << ") has been overridden with a specified file (" << initfile << ")." << endl;
+      initialize = initfile;
     }
 
   } else {
@@ -269,6 +276,9 @@ bool FGScript::LoadScript(string script, double deltaT)
 
     // Notify about when this event is triggered?
     if ((notify_element = event_element->FindElement("notify")) != 0) {
+      if (notify_element->HasAttribute("format")) {
+        if (notify_element->GetAttributeValue("format") == "kml") newEvent->NotifyKML = true;
+      }
       newEvent->Notify = true;
       // Check here for new <description> tag that gets echoed
       string notify_description = notify_element->FindElementValue("description");
@@ -278,23 +288,16 @@ bool FGScript::LoadScript(string script, double deltaT)
       notify_property_element = notify_element->FindElement("property");
       while (notify_property_element) {
         notifyPropertyName = notify_property_element->GetDataLine();
-        if (PropertyManager->GetNode(notifyPropertyName)) {
-          newEvent->NotifyProperties.push_back( PropertyManager->GetNode(notifyPropertyName) );
+
+        newEvent->NotifyPropertyNames.push_back(notifyPropertyName);
+        newEvent->NotifyProperties.push_back(0);
           string caption_attribute = notify_property_element->GetAttributeValue("caption");
           if (caption_attribute.empty()) {
             newEvent->DisplayString.push_back(notifyPropertyName);
           } else {
             newEvent->DisplayString.push_back(caption_attribute);
           }
-        } else {
-          cout << endl << fgred << "  Could not find the property named "
-               << notifyPropertyName << " in script" << endl << "  \""
-               << ScriptName << "\". Execution is aborted. Please recheck "
-               << "your input files and scripts." << reset << endl;
-          delete newEvent->Condition;
-          delete newEvent;
-          return false;
-        }
+
         notify_property_element = notify_element->FindNextElement("property");
       }
     }
@@ -303,7 +306,13 @@ bool FGScript::LoadScript(string script, double deltaT)
     set_element = event_element->FindElement("set");
     while (set_element) {
       prop_name = set_element->GetAttributeValue("name");
+      if (PropertyManager->HasNode(prop_name)) {
       newEvent->SetParam.push_back( PropertyManager->GetNode(prop_name) );
+      } else {
+        newEvent->SetParam.push_back( 0L );
+      }
+      newEvent->SetParamName.push_back( prop_name );
+
       //Todo - should probably do some safety checking here to make sure one or the other
       //of value or function is specified.
       if (!set_element->GetAttributeValue("value").empty()) {
@@ -368,104 +377,143 @@ bool FGScript::RunScript(void)
 
   // Iterate over all events.
   for (unsigned int ev_ctr=0; ev_ctr < Events.size(); ev_ctr++) {
+
+    struct event &thisEvent = Events[ev_ctr];
+
     // Determine whether the set of conditional tests for this condition equate
     // to true and should cause the event to execute. If the conditions evaluate 
     // to true, then the event is triggered. If the event is not persistent,
     // then this trigger will remain set true. If the event is persistent,
     // the trigger will reset to false when the condition evaluates to false.
-    if (Events[ev_ctr].Condition->Evaluate()) {
-      if (!Events[ev_ctr].Triggered) {
+    if (thisEvent.Condition->Evaluate()) {
+      if (!thisEvent.Triggered) {
 
         // The conditions are true, do the setting of the desired Event parameters
-        for (i=0; i<Events[ev_ctr].SetValue.size(); i++) {
-          Events[ev_ctr].OriginalValue[i] = Events[ev_ctr].SetParam[i]->getDoubleValue();
-          if (Events[ev_ctr].Functions[i] != 0) { // Parameter should be set to a function value
+        for (i=0; i<thisEvent.SetValue.size(); i++) {
+          if (thisEvent.SetParam[i] == 0L) { // Late bind property if necessary
+            if (PropertyManager->HasNode(thisEvent.SetParamName[i])) {
+              thisEvent.SetParam[i] = PropertyManager->GetNode(thisEvent.SetParamName[i]);
+            } else {
+              throw("No property, \""+thisEvent.SetParamName[i]+"\" is defined.");
+            }
+          }
+          thisEvent.OriginalValue[i] = thisEvent.SetParam[i]->getDoubleValue();
+          if (thisEvent.Functions[i] != 0) { // Parameter should be set to a function value
             try {
-              Events[ev_ctr].SetValue[i] = Events[ev_ctr].Functions[i]->GetValue();
+              thisEvent.SetValue[i] = thisEvent.Functions[i]->GetValue();
             } catch (string msg) {
               std::cerr << std::endl << "A problem occurred in the execution of the script. " << msg << endl;
               throw;
             }
           }
-          switch (Events[ev_ctr].Type[i]) {
+          switch (thisEvent.Type[i]) {
           case FG_VALUE:
           case FG_BOOL:
-            Events[ev_ctr].newValue[i] = Events[ev_ctr].SetValue[i];
+            thisEvent.newValue[i] = thisEvent.SetValue[i];
             break;
           case FG_DELTA:
-            Events[ev_ctr].newValue[i] = Events[ev_ctr].OriginalValue[i] + Events[ev_ctr].SetValue[i];
+            thisEvent.newValue[i] = thisEvent.OriginalValue[i] + thisEvent.SetValue[i];
             break;
           default:
             cerr << "Invalid Type specified" << endl;
             break;
           }
-          Events[ev_ctr].StartTime = currentTime + Events[ev_ctr].Delay;
-          Events[ev_ctr].ValueSpan[i] = Events[ev_ctr].newValue[i] - Events[ev_ctr].OriginalValue[i];
-          Events[ev_ctr].Transiting[i] = true;
+          thisEvent.StartTime = currentTime + thisEvent.Delay;
+          thisEvent.ValueSpan[i] = thisEvent.newValue[i] - thisEvent.OriginalValue[i];
+          thisEvent.Transiting[i] = true;
         }
       }
-      Events[ev_ctr].Triggered = true;
+      thisEvent.Triggered = true;
 
-    } else if (Events[ev_ctr].Persistent) { // If the event is persistent, reset the trigger.
-      Events[ev_ctr].Triggered = false; // Reset the trigger for persistent events
-      Events[ev_ctr].Notified = false;  // Also reset the notification flag
-    } else if (Events[ev_ctr].Continuous) { // If the event is continuous, reset the trigger.
-      Events[ev_ctr].Triggered = false; // Reset the trigger for persistent events
-      Events[ev_ctr].Notified = false;  // Also reset the notification flag
+    } else if (thisEvent.Persistent) { // If the event is persistent, reset the trigger.
+      thisEvent.Triggered = false; // Reset the trigger for persistent events
+      thisEvent.Notified = false;  // Also reset the notification flag
+    } else if (thisEvent.Continuous) { // If the event is continuous, reset the trigger.
+      thisEvent.Triggered = false; // Reset the trigger for persistent events
+      thisEvent.Notified = false;  // Also reset the notification flag
     }
 
-    if ((currentTime >= Events[ev_ctr].StartTime) && Events[ev_ctr].Triggered) {
+    if ((currentTime >= thisEvent.StartTime) && thisEvent.Triggered) {
 
-      for (i=0; i<Events[ev_ctr].SetValue.size(); i++) {
-        if (Events[ev_ctr].Transiting[i]) {
-          Events[ev_ctr].TimeSpan = currentTime - Events[ev_ctr].StartTime;
-          switch (Events[ev_ctr].Action[i]) {
+      for (i=0; i<thisEvent.SetValue.size(); i++) {
+        if (thisEvent.Transiting[i]) {
+          thisEvent.TimeSpan = currentTime - thisEvent.StartTime;
+          switch (thisEvent.Action[i]) {
           case FG_RAMP:
-            if (Events[ev_ctr].TimeSpan <= Events[ev_ctr].TC[i]) {
-              newSetValue = Events[ev_ctr].TimeSpan/Events[ev_ctr].TC[i] * Events[ev_ctr].ValueSpan[i] + Events[ev_ctr].OriginalValue[i];
+            if (thisEvent.TimeSpan <= thisEvent.TC[i]) {
+              newSetValue = thisEvent.TimeSpan/thisEvent.TC[i] * thisEvent.ValueSpan[i] + thisEvent.OriginalValue[i];
             } else {
-              newSetValue = Events[ev_ctr].newValue[i];
-              if (Events[ev_ctr].Continuous != true) Events[ev_ctr].Transiting[i] = false;
+              newSetValue = thisEvent.newValue[i];
+              if (thisEvent.Continuous != true) thisEvent.Transiting[i] = false;
             }
             break;
           case FG_STEP:
-            newSetValue = Events[ev_ctr].newValue[i];
+            newSetValue = thisEvent.newValue[i];
 
             // If this is not a continuous event, reset the transiting flag.
             // Otherwise, it is known that the event is a continuous event.
             // Furthermore, if the event is to be determined by a function,
             // then the function will be continuously calculated.
-            if (Events[ev_ctr].Continuous != true)
-              Events[ev_ctr].Transiting[i] = false;
-            else if (Events[ev_ctr].Functions[i] != 0)
-              newSetValue = Events[ev_ctr].Functions[i]->GetValue();
+            if (thisEvent.Continuous != true)
+              thisEvent.Transiting[i] = false;
+            else if (thisEvent.Functions[i] != 0)
+              newSetValue = thisEvent.Functions[i]->GetValue();
 
             break;
           case FG_EXP:
-            newSetValue = (1 - exp( -Events[ev_ctr].TimeSpan/Events[ev_ctr].TC[i] )) * Events[ev_ctr].ValueSpan[i] + Events[ev_ctr].OriginalValue[i];
+            newSetValue = (1 - exp( -thisEvent.TimeSpan/thisEvent.TC[i] )) * thisEvent.ValueSpan[i] + thisEvent.OriginalValue[i];
             break;
           default:
             cerr << "Invalid Action specified" << endl;
             break;
           }
-          Events[ev_ctr].SetParam[i]->setDoubleValue(newSetValue);
+          thisEvent.SetParam[i]->setDoubleValue(newSetValue);
         }
       }
 
       // Print notification values after setting them
-      if (Events[ev_ctr].Notify && !Events[ev_ctr].Notified) {
-        cout << endl << "  Event " << event_ctr << " (" << Events[ev_ctr].Name << ")"
-             << " executed at time: " << currentTime << endl;
-        if (!Events[ev_ctr].Description.empty()) {
-          cout << "    " << Events[ev_ctr].Description << endl;
+      if (thisEvent.Notify && !thisEvent.Notified) {
+        if (thisEvent.NotifyKML) {
+          cout << endl << "<Placemark>" << endl;
+          cout << "  <name> " << currentTime << " seconds" << " </name>" << endl;
+          cout << "  <description>" << endl;
+          cout << "  <![CDATA[" << endl;
+          cout << "  <b>" << thisEvent.Name << " (Event " << event_ctr << ")" << " executed at time: " << currentTime << "</b><br/>" << endl;
+        } else  {
+          cout << endl << underon
+               << highint << thisEvent.Name << normint << underoff
+               << " (Event " << event_ctr << ")" 
+               << " executed at time: " << highint << currentTime << normint << endl;
         }
-        for (j=0; j<Events[ev_ctr].NotifyProperties.size();j++) {
-//          cout << "    " << Events[ev_ctr].NotifyProperties[j]->GetRelativeName()
-          cout << "    " << Events[ev_ctr].DisplayString[j]
-               << " = " << Events[ev_ctr].NotifyProperties[j]->getDoubleValue() << endl;
+        if (!thisEvent.Description.empty()) {
+          cout << "    " << thisEvent.Description << endl;
+        }
+        for (j=0; j<thisEvent.NotifyProperties.size();j++) {
+          if (thisEvent.NotifyProperties[j] == 0) {
+            if (PropertyManager->HasNode(thisEvent.NotifyPropertyNames[j])) {
+              thisEvent.NotifyProperties[j] = PropertyManager->GetNode(thisEvent.NotifyPropertyNames[j]);
+            } else {
+              throw("Could not find property named "+thisEvent.NotifyPropertyNames[j]+" in script.");
+            }
+          }
+          cout << "    " << thisEvent.DisplayString[j] << " = " << thisEvent.NotifyProperties[j]->getDoubleValue();
+          if (thisEvent.NotifyKML) cout << " <br/>";
+          cout << endl;
+        }
+        if (thisEvent.NotifyKML) {
+          cout << "  ]]>" << endl;
+          cout << "  </description>" << endl;
+          cout << "  <Point>" << endl;
+          cout << "    <altitudeMode> absolute </altitudeMode>" << endl;
+          cout << "    <extrude> 1 </extrude>" << endl;
+          cout << "    <coordinates>" << FDMExec->GetPropagate()->GetLongitudeDeg()
+            << "," << FDMExec->GetPropagate()->GetGeodLatitudeDeg()
+            << "," << FDMExec->GetPropagate()->GetAltitudeASLmeters() << "</coordinates>" << endl;
+          cout << "  </Point>" << endl;
+          cout << "</Placemark>" << endl;
         }
         cout << endl;
-        Events[ev_ctr].Notified = true;
+        thisEvent.Notified = true;
       }
 
     }
@@ -538,25 +586,37 @@ void FGScript::Debug(int from)
         for (unsigned j=0; j<Events[i].SetValue.size(); j++) {
           if (Events[i].SetValue[j] == 0.0 && Events[i].Functions[j] != 0L) {
             if (Events[i].SetParam[j] == 0) {
+              if (Events[i].SetParamName[j].size() == 0) {
               cerr << fgred << highint << endl
                    << "  An attempt has been made to access a non-existent property" << endl
                    << "  in this event. Please check the property names used, spelling, etc."
                    << reset << endl;
               exit(-1);
+              } else {
+                cout << endl << "      set " << Events[i].SetParamName[j]
+                     << " to function value (Late Bound)";
             }
+            } else {
             cout << endl << "      set " << Events[i].SetParam[j]->GetRelativeName("/fdm/jsbsim/")
                  << " to function value";
+            }
           } else {
             if (Events[i].SetParam[j] == 0) {
+              if (Events[i].SetParamName[j].size() == 0) {
               cerr << fgred << highint << endl
                    << "  An attempt has been made to access a non-existent property" << endl
                    << "  in this event. Please check the property names used, spelling, etc."
                    << reset << endl;
               exit(-1);
+              } else {
+                cout << endl << "      set " << Events[i].SetParamName[j]
+                     << " to function value (Late Bound)";
             }
+            } else {
             cout << endl << "      set " << Events[i].SetParam[j]->GetRelativeName("/fdm/jsbsim/")
                  << " to " << Events[i].SetValue[j];
           }
+          }
 
           switch (Events[i].Type[j]) {
           case FG_VALUE:
@@ -592,10 +652,14 @@ void FGScript::Debug(int from)
         // Print notifications
         if (Events[i].Notify) {
           if (Events[i].NotifyProperties.size() > 0) {
-            cout << "  Notifications" << ":" << endl << "    {" << endl;
-            for (unsigned j=0; j<Events[i].NotifyProperties.size();j++) {
+            if (Events[i].NotifyKML) {
+              cout << "  Notifications (KML Format):" << endl << "    {" << endl;
+            } else {
+              cout << "  Notifications:" << endl << "    {" << endl;
+            }
+            for (unsigned j=0; j<Events[i].NotifyPropertyNames.size();j++) {
               cout << "      "
-                   << Events[i].NotifyProperties[j]->GetRelativeName("/fdm/jsbsim/")
+                   << Events[i].NotifyPropertyNames[j]
                    << endl;
             }
             cout << "    }" << endl;
