@@ -43,7 +43,7 @@ using namespace std;
 
 namespace JSBSim {
 
-static const char *IdSrc = "$Id: FGFunction.cpp,v 1.43 2012/02/05 11:15:54 bcoconni Exp $";
+static const char *IdSrc = "$Id: FGFunction.cpp,v 1.53 2013/09/27 19:42:08 jberndt Exp $";
 static const char *IdHdr = ID_FUNCTION;
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -64,6 +64,9 @@ const std::string FGFunction::difference_string = "difference";
 const std::string FGFunction::product_string = "product";
 const std::string FGFunction::quotient_string = "quotient";
 const std::string FGFunction::pow_string = "pow";
+const std::string FGFunction::sqrt_string = "sqrt";
+const std::string FGFunction::toradians_string = "toradians";
+const std::string FGFunction::todegrees_string = "todegrees";
 const std::string FGFunction::exp_string = "exp";
 const std::string FGFunction::log2_string = "log2";
 const std::string FGFunction::ln_string = "ln";
@@ -83,6 +86,8 @@ const std::string FGFunction::avg_string = "avg";
 const std::string FGFunction::fraction_string = "fraction";
 const std::string FGFunction::mod_string = "mod";
 const std::string FGFunction::random_string = "random";
+const std::string FGFunction::urandom_string = "urandom";
+const std::string FGFunction::pi_string = "pi";
 const std::string FGFunction::integer_string = "integer";
 const std::string FGFunction::rotation_alpha_local_string = "rotation_alpha_local";
 const std::string FGFunction::rotation_beta_local_string = "rotation_beta_local";
@@ -101,6 +106,7 @@ const std::string FGFunction::or_string = "or";
 const std::string FGFunction::not_string = "not";
 const std::string FGFunction::ifthen_string = "ifthen";
 const std::string FGFunction::switch_string = "switch";
+const std::string FGFunction::interpolate1d_string = "interpolate1d";
 
 FGFunction::FGFunction(FGPropertyManager* propMan, Element* el, const string& prefix)
                                       : PropertyManager(propMan), Prefix(prefix)
@@ -110,11 +116,23 @@ FGFunction::FGFunction(FGPropertyManager* propMan, Element* el, const string& pr
   cached = false;
   cachedValue = -HUGE_VAL;
   invlog2val = 1.0/log10(2.0);
+  pCopyTo = 0L;
 
   Name = el->GetAttributeValue("name");
   operation = el->GetName();
 
   if (operation == function_string) {
+    sCopyTo = el->GetAttributeValue("copyto");
+    if (!sCopyTo.empty()) {
+
+      if (sCopyTo.find("#") != string::npos) {
+        if (is_number(Prefix)) sCopyTo = replace(sCopyTo,"#",Prefix);
+      }
+
+      pCopyTo = PropertyManager->GetNode(sCopyTo);
+      if (pCopyTo == 0L) cerr << "Property \"" << sCopyTo << "\" must be previously defined in function "
+                              << Name << endl;
+    }
     Type = eTopLevel;
   } else if (operation == product_string) {
     Type = eProduct;
@@ -126,6 +144,12 @@ FGFunction::FGFunction(FGPropertyManager* propMan, Element* el, const string& pr
     Type = eQuotient;
   } else if (operation == pow_string) {
     Type = ePow;
+  } else if (operation == sqrt_string) {
+    Type = eSqrt;
+  } else if (operation == toradians_string) {
+    Type = eToRadians;
+  } else if (operation == todegrees_string) {
+    Type = eToDegrees;
   } else if (operation == log2_string) {
     Type = eLog2;
   } else if (operation == ln_string) {
@@ -166,6 +190,10 @@ FGFunction::FGFunction(FGPropertyManager* propMan, Element* el, const string& pr
     Type = eMod;
   } else if (operation == random_string) {
     Type = eRandom;
+  } else if (operation == urandom_string) {
+    Type = eUrandom;
+  } else if (operation == pi_string) {
+    Type = ePi;
   } else if (operation == rotation_alpha_local_string) {
     Type = eRotation_alpha_local;
   } else if (operation == rotation_beta_local_string) {
@@ -198,12 +226,14 @@ FGFunction::FGFunction(FGPropertyManager* propMan, Element* el, const string& pr
     Type = eIfThen;
   } else if (operation == switch_string) {
     Type = eSwitch;
+  } else if (operation == interpolate1d_string) {
+    Type = eInterpolate1D;
   } else if (operation != description_string) {
     cerr << "Bad operation " << operation << " detected in configuration file" << endl;
   }
 
   element = el->GetElement();
-  if (!element) {
+  if (!element && Type != eRandom && Type != eUrandom && Type != ePi) {
     cerr << fgred << highint << endl;
     cerr << "  No element was specified as an argument to the \"" << operation << "\" operation" << endl;
     cerr << "  This can happen when, for instance, a cos operation is specified and a " << endl;
@@ -224,13 +254,13 @@ FGFunction::FGFunction(FGPropertyManager* propMan, Element* el, const string& pr
           property_name = replace(property_name,"#",Prefix);
         }
       }
-      FGPropertyManager* newNode = 0L;
+      FGPropertyNode* newNode = 0L;
       if (PropertyManager->HasNode(property_name)) {
         newNode = PropertyManager->GetNode(property_name);
         Parameters.push_back(new FGPropertyValue( newNode ));
       } else {
-        cerr << fgcyan << "Warning: The property " + property_name + " is initially undefined."
-             << reset << endl;
+        // cerr << fgcyan << "Warning: The property " + property_name + " is initially undefined."
+        //      << reset << endl;
         Parameters.push_back(new FGPropertyValue( property_name,
                                                   PropertyManager ));
       }
@@ -244,6 +274,9 @@ FGFunction::FGFunction(FGPropertyManager* propMan, Element* el, const string& pr
                operation == sum_string ||
                operation == quotient_string ||
                operation == pow_string ||
+               operation == sqrt_string ||
+               operation == toradians_string ||
+               operation == todegrees_string ||
                operation == exp_string ||
                operation == log2_string ||
                operation == ln_string ||
@@ -263,6 +296,8 @@ FGFunction::FGFunction(FGPropertyManager* propMan, Element* el, const string& pr
                operation == integer_string ||
                operation == mod_string ||
                operation == random_string ||
+               operation == urandom_string ||
+               operation == pi_string ||
                operation == avg_string ||
                operation == rotation_alpha_local_string||
                operation == rotation_beta_local_string||
@@ -279,7 +314,8 @@ FGFunction::FGFunction(FGPropertyManager* propMan, Element* el, const string& pr
                operation == or_string ||
                operation == not_string ||
                operation == ifthen_string ||
-               operation == switch_string)
+               operation == switch_string ||
+               operation == interpolate1d_string)
     {
       Parameters.push_back(new FGFunction(PropertyManager, element, Prefix));
     } else if (operation != description_string) {
@@ -334,10 +370,13 @@ double FGFunction::GetValue(void) const
 
   if (cached) return cachedValue;
 
-  temp = Parameters[0]->GetValue();
+  if (   Type != eRandom
+      && Type != eUrandom
+      && Type != ePi      ) temp = Parameters[0]->GetValue();
   
   switch (Type) {
   case eTopLevel:
+    if (pCopyTo) pCopyTo->setDoubleValue(temp);
     break;
   case eProduct:
     for (i=1;i<Parameters.size();i++) {
@@ -363,6 +402,15 @@ double FGFunction::GetValue(void) const
   case ePow:
     temp = pow(temp,Parameters[1]->GetValue());
     break;
+  case eSqrt:
+    temp = sqrt(temp);
+    break;
+  case eToRadians:
+    temp *= M_PI/180.0;
+    break;
+  case eToDegrees:
+    temp *= 180.0/M_PI;
+    break;
   case eExp:
     temp = exp(temp);
     break;
@@ -434,6 +482,12 @@ double FGFunction::GetValue(void) const
   case eRandom:
     temp = GaussianRandomNumber();
     break;
+  case eUrandom:
+    temp = -1.0 + (((double)rand()/double(RAND_MAX))*2.0);
+    break;
+  case ePi:
+    temp = M_PI;
+    break;
   case eLT:
     temp = (temp < Parameters[1]->GetValue())?1:0;
     break;
@@ -499,6 +553,27 @@ double FGFunction::GetValue(void) const
       }
     }
     break;
+  case eInterpolate1D:
+    {
+      unsigned int sz = Parameters.size();
+      if (temp <= Parameters[1]->GetValue()) {
+        temp = Parameters[2]->GetValue();
+      } else if (temp >= Parameters[sz-2]->GetValue()) {
+        temp = Parameters[sz-1]->GetValue();
+      } else {
+        for (unsigned int i=1; i<=sz-4; i+=2) {
+          if (temp < Parameters[i+2]->GetValue()) {
+            double factor = (temp - Parameters[i]->GetValue()) /
+                            (Parameters[i+2]->GetValue() - Parameters[i]->GetValue());
+            double span = Parameters[i+3]->GetValue() - Parameters[i+1]->GetValue();
+            double val = factor*span;
+            temp = Parameters[i+1]->GetValue() + val;
+            break;
+          }
+        }
+      }
+    }
+    break;
   case eRotation_alpha_local:
     if (Parameters.size()==6) // calculates local angle of attack for skydiver body component
         //Euler angles from the intermediate body frame to the local body frame must be from a z-y-x axis rotation order
@@ -731,6 +806,13 @@ void FGFunction::bind(void)
       }
     }
 
+    if (PropertyManager->HasNode(tmp)) {
+      FGPropertyNode* _property = PropertyManager->GetNode(tmp);
+      if (_property->isTied()) {
+      cout << "Property " << tmp << " has already been successfully bound (late)." << endl;
+        throw("Failed to bind the property to an existing already tied node.");
+      }
+    }
     PropertyManager->Tie( tmp, this, &FGFunction::GetValue);
   }
 }
