@@ -40,6 +40,7 @@ SENTRY
   INCLUDES
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
+#include <string>
 #include "FGJSBBase.h"
 #include "FGColumnVector3.h"
 
@@ -47,7 +48,7 @@ SENTRY
   DEFINITIONS
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
-#define ID_QUATERNION "$Id: FGQuaternion.h,v 1.24 2011/12/10 15:49:21 bcoconni Exp $"
+#define ID_QUATERNION "$Id: FGQuaternion.h,v 1.26 2013/11/24 16:53:15 bcoconni Exp $"
 
 namespace JSBSim {
 
@@ -148,6 +149,27 @@ public:
     }
   }
 
+  /** Initializer by a rotation axis and an angle.
+      Initialize the quaternion to represent the rotation around a given
+      angle and an arbitrary axis.
+      @param angle The angle in radians
+      @param axis  The rotation axis
+   */
+  FGQuaternion(double angle, const FGColumnVector3& axis)
+    : mCacheValid(false) {
+
+    double angle2 = 0.5 * angle;
+
+    double length = axis.Magnitude();
+    double Sangle2 = sin(angle2) / length;
+    double Cangle2 = cos(angle2);
+
+    data[0] = Cangle2;
+    data[1] = Sangle2 * axis(1);
+    data[2] = Sangle2 * axis(2);
+    data[3] = Sangle2 * axis(3);
+  }
+
   /** Initializer by matrix.
       Initialize the quaternion with the matrix representing a transform from one frame
       to another using the standard aerospace sequence, Yaw-Pitch-Roll (3-2-1).
@@ -463,6 +485,8 @@ public:
       Useful for initialization of increments */
   static FGQuaternion zero(void) { return FGQuaternion( 0.0, 0.0, 0.0, 0.0 ); }
 
+  std::string Dump(const std::string& delimiter) const;
+
   friend FGQuaternion QExp(const FGColumnVector3& omega);
 
 private:
