@@ -43,12 +43,14 @@ INCLUDES
 #include "FGFDMExec.h"
 #include "FGAerodynamics.h"
 #include "input_output/FGPropertyManager.h"
+#include "input_output/FGXMLFileRead.h"
+#include "input_output/FGXMLElement.h"
 
 using namespace std;
 
 namespace JSBSim {
 
-static const char *IdSrc = "$Id: FGAerodynamics.cpp,v 1.45 2012/04/13 13:25:52 jberndt Exp $";
+static const char *IdSrc = "$Id: FGAerodynamics.cpp,v 1.49 2013/11/24 11:40:55 bcoconni Exp $";
 static const char *IdHdr = ID_AERODYNAMICS;
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -77,6 +79,7 @@ FGAerodynamics::FGAerodynamics(FGFDMExec* FDMExec) : FGModel(FDMExec)
   axisType = atNone;
 
   AeroFunctions = new AeroFunctionArray[6];
+  AeroFunctionsAtCG = new AeroFunctionArray[6];
 
   impending_stall = stall_hyst = 0.0;
   alphaclmin = alphaclmax = 0.0;
@@ -101,8 +104,12 @@ FGAerodynamics::~FGAerodynamics()
   for (i=0; i<6; i++)
     for (j=0; j<AeroFunctions[i].size(); j++)
       delete AeroFunctions[i][j];
+  for (i=0; i<6; i++)
+    for (j=0; j<AeroFunctionsAtCG[i].size(); j++)
+      delete AeroFunctionsAtCG[i][j];
 
   delete[] AeroFunctions;
+  delete[] AeroFunctionsAtCG;
 
   delete AeroRPShift;
 
@@ -165,7 +172,9 @@ bool FGAerodynamics::Run(bool Holding)
   }
 
   vFw.InitMatrix();
+  vFwAtCG.InitMatrix();
   vFnative.InitMatrix();
+  vFnativeAtCG.InitMatrix();
 
   for (axis_ctr = 0; axis_ctr < 3; axis_ctr++) {
     for (ctr=0; ctr < AeroFunctions[axis_ctr].size(); ctr++) {
@@ -173,24 +182,54 @@ bool FGAerodynamics::Run(bool Holding)
     }
   }
 
+  for (axis_ctr = 0; axis_ctr < 3; axis_ctr++) {
+    for (ctr=0; ctr < AeroFunctionsAtCG[axis_ctr].size(); ctr++) {
+      vFnativeAtCG(axis_ctr+1) += AeroFunctionsAtCG[axis_ctr][ctr]->GetValue();
+    }
+  }
+
   // Note that we still need to convert to wind axes here, because it is
   // used in the L/D calculation, and we still may want to look at Lift
   // and Drag.
 
+  // JSB 4/27/12 - After use, convert wind axes to produce normal lift
+  // and drag values - not negative ones!
+
+  // As a clarification, JSBSim assumes that drag and lift values are defined 
+  // in wind axes - BUT with a 180 rotation about the Y axis. That is, lift and
+  // drag will be positive up and aft, respectively, so that they are reported
+  // as positive numbers. However, the wind axes themselves assume that the X
+  // and Z forces are positive forward and down.
+
   switch (axisType) {
     case atBodyXYZ:       // Forces already in body axes; no manipulation needed
       vFw = in.Tb2w*vFnative;
       vForces = vFnative;
+      vFw(eDrag)*=-1; vFw(eLift)*=-1;
+
+      vFwAtCG = in.Tb2w*vFnativeAtCG;
+      vForcesAtCG = vFnativeAtCG;
+      vFwAtCG(eDrag)*=-1; vFwAtCG(eLift)*=-1;
       break;
     case atLiftDrag:      // Copy forces into wind axes
       vFw = vFnative;
       vFw(eDrag)*=-1; vFw(eLift)*=-1;
       vForces = in.Tw2b*vFw;
+      vFw(eDrag)*=-1; vFw(eLift)*=-1;
+
+      vFwAtCG = vFnativeAtCG;
+      vFwAtCG(eDrag)*=-1; vFwAtCG(eLift)*=-1;
+      vForcesAtCG = in.Tw2b*vFwAtCG;
+      vFwAtCG(eDrag)*=-1; vFwAtCG(eLift)*=-1;
       break;
     case atAxialNormal:   // Convert native forces into Axial|Normal|Side system
       vFw = in.Tb2w*vFnative;
       vFnative(eX)*=-1; vFnative(eZ)*=-1;
       vForces = vFnative;
+
+      vFwAtCG = in.Tb2w*vFnativeAtCG;
+      vFnativeAtCG(eX)*=-1; vFnativeAtCG(eZ)*=-1;
+      vForcesAtCG = vFnativeAtCG;
       break;
     default:
       cerr << endl << "  A proper axis type has NOT been selected. Check "
@@ -200,12 +239,13 @@ bool FGAerodynamics::Run(bool Holding)
 
   // Calculate lift coefficient squared
   if ( in.Qbar > 0) {
-    clsq = vFw(eLift) / (in.Wingarea*in.Qbar);
+    clsq = (vFw(eLift) + vFwAtCG(eLift))/ (in.Wingarea*in.Qbar);
     clsq *= clsq;
   }
 
   // Calculate lift Lift over Drag
-  if ( fabs(vFw(eDrag)) > 0.0) lod = fabs( vFw(eLift) / vFw(eDrag) );
+  if ( fabs(vFw(eDrag) + vFwAtCG(eDrag)) > 0.0)
+    lod = fabs( (vFw(eLift) + vFwAtCG(eLift))/ (vFw(eDrag) + vFwAtCG(eDrag)));
 
   // Calculate aerodynamic reference point shift, if any. The shift
   // takes place in the structual axis. That is, if the shift is positive,
@@ -218,13 +258,18 @@ bool FGAerodynamics::Run(bool Holding)
   vDXYZcg(eY) = in.RPBody(eY) + vDeltaRP(eY);
   vDXYZcg(eZ) = in.RPBody(eZ) - vDeltaRP(eZ);
 
-  vMoments = vDXYZcg*vForces; // M = r X F
+  vMomentsMRC.InitMatrix();
 
   for (axis_ctr = 0; axis_ctr < 3; axis_ctr++) {
     for (ctr = 0; ctr < AeroFunctions[axis_ctr+3].size(); ctr++) {
-      vMoments(axis_ctr+1) += AeroFunctions[axis_ctr+3][ctr]->GetValue();
+      vMomentsMRC(axis_ctr+1) += AeroFunctions[axis_ctr+3][ctr]->GetValue();
     }
   }
+  vMoments = vMomentsMRC + vDXYZcg*vForces; // M = r X F
+  // Now add the "at CG" values to base forces - after the moments have been transferred
+  vForces += vForcesAtCG;
+  vFnative += vFnativeAtCG;
+  vFw += vFwAtCG;
 
   RunPostFunctions();
 
@@ -241,11 +286,13 @@ bool FGAerodynamics::Load(Element *element)
   Element *temp_element, *axis_element, *function_element;
 
   string separator = "/";
+  FGXMLFileRead XMLFileRead;
+  Element* document;
 
   fname = element->GetAttributeValue("file");
   if (!fname.empty()) {
     file = FDMExec->GetFullAircraftPath() + separator + fname;
-    document = LoadXMLDocument(file);
+    document = XMLFileRead.LoadXMLDocument(file);
     if (document == 0L) return false;
   } else {
     document = element;
@@ -253,7 +300,7 @@ bool FGAerodynamics::Load(Element *element)
 
   FGModel::Load(document); // Perform base class Pre-Load
 
-  DetermineAxisSystem(); // Detemine if Lift/Side/Drag, etc. is used.
+  DetermineAxisSystem(document); // Detemine if Lift/Side/Drag, etc. is used.
 
   Debug(2);
 
@@ -279,10 +326,16 @@ bool FGAerodynamics::Load(Element *element)
   axis_element = document->FindElement("axis");
   while (axis_element) {
     AeroFunctionArray ca;
+    AeroFunctionArray ca_atCG;
     axis = axis_element->GetAttributeValue("name");
     function_element = axis_element->FindElement("function");
     while (function_element) {
       string current_func_name = function_element->GetAttributeValue("name");
+      bool apply_at_cg = false;
+      if (function_element->HasAttribute("apply_at_cg")) {
+        if (function_element->GetAttributeValue("apply_at_cg") == "true") apply_at_cg = true;
+      }
+      if (!apply_at_cg) {
       try {
         ca.push_back( new FGFunction(PropertyManager, function_element) );
       } catch (string const str) {
@@ -290,9 +343,19 @@ bool FGAerodynamics::Load(Element *element)
              << current_func_name << ":" << str << " Aborting." << reset << endl;
         return false;
       }
+      } else {
+        try {
+          ca_atCG.push_back( new FGFunction(PropertyManager, function_element) );
+        } catch (string const str) {
+          cerr << endl << fgred << "Error loading aerodynamic function in " 
+               << current_func_name << ":" << str << " Aborting." << reset << endl;
+          return false;
+        }
+      }
       function_element = axis_element->FindNextElement("function");
     }
     AeroFunctions[AxisIdx[axis]] = ca;
+    AeroFunctionsAtCG[AxisIdx[axis]] = ca_atCG;
     axis_element = document->FindNextElement("axis");
   }
 
@@ -311,7 +374,7 @@ bool FGAerodynamics::Load(Element *element)
 // This is OK, and the warning is due to the SIDE specifier used for both
 // the Lift/Drag and Axial/Normal axis systems.
 
-void FGAerodynamics::DetermineAxisSystem()
+void FGAerodynamics::DetermineAxisSystem(Element* document)
 {
   Element* axis_element = document->FindElement("axis");
   string axis;
