@@ -50,7 +50,7 @@ using namespace std;
 
 namespace JSBSim {
 
-static const char *IdSrc = "$Id: FGAuxiliary.cpp,v 1.56 2011/12/11 17:03:05 bcoconni Exp $";
+static const char *IdSrc = "$Id: FGAuxiliary.cpp,v 1.62 2013/09/11 12:43:20 jberndt Exp $";
 static const char *IdHdr = ID_AUXILIARY;
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -76,7 +76,7 @@ FGAuxiliary::FGAuxiliary(FGFDMExec* fdmex) : FGModel(fdmex)
   seconds_in_day = 0.0;
   hoverbmac = hoverbcg = 0.0;
   Re = 0.0;
-  Nz = 0.0;
+  Nz = Ny = 0.0;
   lon_relative_position = lat_relative_position = relative_position = 0.0;
 
   vPilotAccel.InitMatrix();
@@ -111,7 +111,7 @@ bool FGAuxiliary::InitModel(void)
   seconds_in_day = 0.0;
   hoverbmac = hoverbcg = 0.0;
   Re = 0.0;
-  Nz = 0.0;
+  Nz = Ny = 0.0;
   lon_relative_position = lat_relative_position = relative_position = 0.0;
 
   vPilotAccel.InitMatrix();
@@ -159,6 +159,7 @@ bool FGAuxiliary::Run(bool Holding)
   double AeroV2 = vAeroUVW(eV)*vAeroUVW(eV);
   double AeroW2 = vAeroUVW(eW)*vAeroUVW(eW);
   double mUW = AeroU2 + AeroW2;
+  double Vtdot = (vAeroUVW(eU)*in.vUVWdot(eU) + vAeroUVW(eV)*in.vUVWdot(eV) + vAeroUVW(eW)*in.vUVWdot(eW))/Vt;
 
   double Vt2 = Vt*Vt;
 
@@ -168,13 +169,14 @@ bool FGAuxiliary::Run(bool Holding)
     if (vAeroUVW(eV) != 0.0)
       beta  =    mUW > 0.0 ? atan2(vAeroUVW(eV), sqrt(mUW)) : 0.0;
 
-    double signU=1;
-    if (vAeroUVW(eU) < 0.0) signU=-1;
+    //double signU=1;
+    //if (vAeroUVW(eU) < 0.0) signU=-1;
 
     if ( mUW >= 1.0 ) {
       adot = (vAeroUVW(eU)*in.vUVWdot(eW) - vAeroUVW(eW)*in.vUVWdot(eU))/mUW;
-      bdot = (signU*mUW*in.vUVWdot(eV)
-             - vAeroUVW(eV)*(vAeroUVW(eU)*in.vUVWdot(eU) + vAeroUVW(eW)*in.vUVWdot(eW)))/(Vt2*sqrt(mUW));
+      // bdot = (signU*mUW*in.vUVWdot(eV)
+      //        - vAeroUVW(eV)*(vAeroUVW(eU)*in.vUVWdot(eU) + vAeroUVW(eW)*in.vUVWdot(eW)))/(Vt2*sqrt(mUW));
+      bdot = (in.vUVWdot(eV)*Vt - vAeroUVW(eV)*Vtdot)/(Vt*sqrt(mUW));
     }
   }
 
@@ -214,7 +216,7 @@ bool FGAuxiliary::Run(bool Holding)
   }
 
   A = pow(((pt-in.Pressure)/in.PressureSL + 1),0.28571);
-  if (MachU > 0.0) {
+  if (abs(MachU) > 0.0) {
     vcas = sqrt(7 * in.PressureSL / in.DensitySL * (A-1));
     veas = sqrt(2 * qbar / in.DensitySL);
     vtrue = 1116.43559 * MachU * sqrt(in.Temperature / 518.67);
@@ -224,21 +226,11 @@ bool FGAuxiliary::Run(bool Holding)
 
   vPilotAccel.InitMatrix();
   vNcg = in.vBodyAccel/in.SLGravity;
-  if ( Vt > 1.0 ) {
-    // Nz is Acceleration in "g's", along normal axis (-Z body axis)
-    Nz = -vNcg(eZ);
-    vPilotAccel = in.vBodyAccel + in.vPQRdot * in.ToEyePt;
-    vPilotAccel += in.vPQR * (in.vPQR * in.ToEyePt);
-  } else {
-    // The line below handles low velocity (and on-ground) cases, basically
-    // representing the opposite of the force that the landing gear would
-    // exert on the ground (which is just the total weight). This eliminates
-    // any jitter that could be introduced by the landing gear. Theoretically,
-    // this branch could be eliminated, with a penalty of having a short
-    // transient at startup (lasting only a fraction of a second).
-    vPilotAccel = in.Tl2b * FGColumnVector3( 0.0, 0.0, -in.SLGravity );
-    Nz = -vPilotAccel(eZ) / in.SLGravity;
-  }
+  // Nz is Acceleration in "g's", along normal axis (-Z body axis)
+  Nz = -vNcg(eZ);
+  Ny =  vNcg(eY);
+  vPilotAccel = in.vBodyAccel + in.vPQRdot * in.ToEyePt;
+  vPilotAccel += in.vPQR * (in.vPQR * in.ToEyePt);
 
   vNwcg = mTb2w * vNcg;
   vNwcg(eZ) = 1.0 - vNwcg(eZ);
@@ -342,8 +334,8 @@ double FGAuxiliary::GetNlf(void) const
 void FGAuxiliary::CalculateRelativePosition(void)  //ToDo: This belongs elsewhere - perhaps in FGPropagate or Exec
 {
   const double earth_radius_mt = in.ReferenceRadius*fttom;
-  lat_relative_position=(in.Latitude  - FDMExec->GetIC()->GetLatitudeDegIC() *degtorad)*earth_radius_mt;
-  lon_relative_position=(in.Longitude - FDMExec->GetIC()->GetLongitudeDegIC()*degtorad)*earth_radius_mt;
+  lat_relative_position=(in.vLocation.GetLatitude()  - FDMExec->GetIC()->GetLatitudeDegIC() *degtorad)*earth_radius_mt;
+  lon_relative_position=(in.vLocation.GetLongitude() - FDMExec->GetIC()->GetLongitudeDegIC()*degtorad)*earth_radius_mt;
   relative_position = sqrt(lat_relative_position*lat_relative_position + lon_relative_position*lon_relative_position);
 };
 
@@ -382,6 +374,7 @@ void FGAuxiliary::bind(void)
   PropertyManager->Tie("accelerations/n-pilot-y-norm", this, eY, (PMF)&FGAuxiliary::GetNpilot);
   PropertyManager->Tie("accelerations/n-pilot-z-norm", this, eZ, (PMF)&FGAuxiliary::GetNpilot);
   PropertyManager->Tie("accelerations/Nz", this, &FGAuxiliary::GetNz);
+  PropertyManager->Tie("accelerations/Ny", this, &FGAuxiliary::GetNy);
   PropertyManager->Tie("forces/load-factor", this, &FGAuxiliary::GetNlf);
   /* PropertyManager->Tie("atmosphere/headwind-fps", this, &FGAuxiliary::GetHeadWind, true);
   PropertyManager->Tie("atmosphere/crosswind-fps", this, &FGAuxiliary::GetCrossWind, true); */
