@@ -67,6 +67,7 @@ INCLUDES
 #include <cstdlib>
 #include <iostream>
 #include <iomanip>
+#include <fstream>
 
 #include "FGPropagate.h"
 #include "FGGroundReactions.h"
@@ -77,7 +78,7 @@ using namespace std;
 
 namespace JSBSim {
 
-static const char *IdSrc = "$Id: FGPropagate.cpp,v 1.105 2012/03/26 21:26:11 bcoconni Exp $";
+static const char *IdSrc = "$Id: FGPropagate.cpp,v 1.119 2013/11/24 11:40:56 bcoconni Exp $";
 static const char *IdHdr = ID_PROPAGATE;
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -91,8 +92,6 @@ FGPropagate::FGPropagate(FGFDMExec* fdmex)
   Debug(0);
   Name = "FGPropagate";
 
-  vInertialVelocity.InitMatrix();
-
   /// These define the indices use to select the various integrators.
   // eNone = 0, eRectEuler, eTrapezoidal, eAdamsBashforth2, eAdamsBashforth3, eAdamsBashforth4};
 
@@ -125,8 +124,6 @@ bool FGPropagate::InitModel(void)
   VState.vLocation.SetEllipse(in.SemiMajor, in.SemiMinor);
   VState.vLocation.SetAltitudeAGL(4.0, FDMExec->GetSimTime());
 
-  vInertialVelocity.InitMatrix();
-
   VState.dqPQRidot.resize(4, FGColumnVector3(0.0,0.0,0.0));
   VState.dqUVWidot.resize(4, FGColumnVector3(0.0,0.0,0.0));
   VState.dqInertialVelocity.resize(4, FGColumnVector3(0.0,0.0,0.0));
@@ -442,11 +439,14 @@ void FGPropagate::UpdateBodyMatrices(void)
   Tb2l  = Tl2b.Transposed();          // body to local frame transform
   Tec2b = Ti2b * Tec2i;               // ECEF to body frame transform
   Tb2ec = Tec2b.Transposed();         // body to ECEF frame tranform
+
+  Qec2b = Tec2b.GetQuaternion();
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-void FGPropagate::SetInertialOrientation(const FGQuaternion& Qi) {
+void FGPropagate::SetInertialOrientation(const FGQuaternion& Qi)
+{
   VState.qAttitudeECI = Qi;
   VState.qAttitudeECI.Normalize();
   UpdateBodyMatrices();
@@ -510,6 +510,13 @@ double FGPropagate::GetDistanceAGL(void) const
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
+double FGPropagate::GetDistanceAGLKm(void) const
+{
+  return VState.vLocation.GetAltitudeAGL(FDMExec->GetSimTime())*0.0003048;
+}
+
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
 void FGPropagate::SetDistanceAGL(double tt)
 {
   VState.vLocation.SetAltitudeAGL(tt, FDMExec->GetSimTime());
@@ -518,6 +525,14 @@ void FGPropagate::SetDistanceAGL(double tt)
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
+void FGPropagate::SetDistanceAGLKm(double tt)
+{
+  VState.vLocation.SetAltitudeAGL(tt*3280.8399, FDMExec->GetSimTime());
+  UpdateVehicleState();
+}
+
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
 void FGPropagate::SetVState(const VehicleState& vstate)
 {
   //ToDo: Shouldn't all of these be set from the vstate vector passed in?
@@ -560,6 +575,13 @@ void FGPropagate::SetLocation(const FGLocation& l)
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
+FGColumnVector3 FGPropagate::GetEulerDeg(void) const
+{
+  return VState.qAttitudeLocal.GetEuler() * radtodeg;
+}
+
+//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+
 void FGPropagate::DumpState(void)
 {
   cout << endl;
@@ -571,9 +593,9 @@ void FGPropagate::DumpState(void)
        <<   "Position" << underoff << endl;
   cout << "    ECI:   " << VState.vInertialPosition.Dump(", ") << " (x,y,z, in ft)" << endl;
   cout << "    ECEF:  " << VState.vLocation << " (x,y,z, in ft)"  << endl;
-  cout << "    Local: " << VState.vLocation.GetLatitudeDeg()
+  cout << "    Local: " << VState.vLocation.GetGeodLatitudeDeg()
                         << ", " << VState.vLocation.GetLongitudeDeg()
-                        << ", " << GetAltitudeASL() << " (lat, lon, alt in deg and ft)" << endl;
+                        << ", " << GetAltitudeASL() << " (geodetic lat, lon, alt ASL in deg and ft)" << endl;
 
   cout << endl << "  " << underon
        <<   "Orientation" << underoff << endl;
@@ -593,11 +615,91 @@ void FGPropagate::DumpState(void)
   cout << "    ECEF:  " << (VState.vPQR*radtodeg).Dump(", ") << " (p,q,r in deg/s)" << endl;
 }
 
+//******************************************************************************
+
+void FGPropagate::WriteStateFile(int num)
+{
+  ofstream outfile;
+
+  if (num == 0) return;
+
+  string filename = FDMExec->GetFullAircraftPath();
+
+  string sim_time = to_string((double)FDMExec->GetSimTime());
+  if (filename.empty()) filename = "initfile.";
+  else                  filename.append("/initfile.");
+
+  // Append sim time to the filename since there may be more than one created during a simulation run
+  filename += to_string((double)FDMExec->GetSimTime())+".xml";
+
+  switch(num) {
+  case 1:
+    outfile.open(filename.c_str());
+    if (outfile.is_open()) {
+      outfile << "<?xml version=\"1.0\"?>" << endl;
+      outfile << "<initialize name=\"reset00\">" << endl;
+      outfile << "  <ubody unit=\"FT/SEC\"> " << VState.vUVW(eU) << " </ubody> " << endl;
+      outfile << "  <vbody unit=\"FT/SEC\"> " << VState.vUVW(eV) << " </vbody> " << endl;
+      outfile << "  <wbody unit=\"FT/SEC\"> " << VState.vUVW(eW) << " </wbody> " << endl;
+      outfile << "  <phi unit=\"DEG\"> " << VState.qAttitudeLocal.GetEuler(ePhi)*radtodeg << " </phi>" << endl;
+      outfile << "  <theta unit=\"DEG\"> " << VState.qAttitudeLocal.GetEuler(eTht)*radtodeg << " </theta>" << endl;
+      outfile << "  <psi unit=\"DEG\"> " << VState.qAttitudeLocal.GetEuler(ePsi)*radtodeg << " </psi>" << endl;
+      outfile << "  <longitude unit=\"DEG\"> " << VState.vLocation.GetLongitudeDeg() << " </longitude>" << endl;
+      outfile << "  <latitude unit=\"DEG\"> " << VState.vLocation.GetLatitudeDeg() << " </latitude>" << endl;
+      outfile << "  <altitude unit=\"FT\"> " << GetDistanceAGL() << " </altitude>" << endl;
+      outfile << "</initialize>" << endl;
+      outfile.close();
+    } else {
+      cerr << "Could not open and/or write the state to the initial conditions file: " << filename << endl;
+    }
+    break;
+  case 2:
+    outfile.open(filename.c_str());
+    if (outfile.is_open()) {
+      outfile << "<?xml version=\"1.0\"?>" << endl;
+      outfile << "<initialize name=\"IC File\" version=\"2.0\">" << endl;
+      outfile << "" << endl;
+      outfile << "  <position frame=\"ECEF\">" << endl;
+      outfile << "    <latitude unit=\"DEG\" type=\"geodetic\"> " << VState.vLocation.GetGeodLatitudeDeg() << " </latitude>" << endl;
+      outfile << "    <longitude unit=\"DEG\"> " << VState.vLocation.GetLongitudeDeg() << " </longitude>" << endl;
+      outfile << "    <altitudeMSL unit=\"FT\"> " << GetAltitudeASL() << " </altitudeMSL>" << endl;
+      outfile << "  </position>" << endl;
+      outfile << "" << endl;
+      outfile << "  <orientation unit=\"DEG\" frame=\"LOCAL\">" << endl;
+      outfile << "    <yaw> " << VState.qAttitudeLocal.GetEulerDeg(eYaw) << " </yaw>" << endl;
+      outfile << "    <pitch> " << VState.qAttitudeLocal.GetEulerDeg(ePitch) << " </pitch>" << endl;
+      outfile << "    <roll> " << VState.qAttitudeLocal.GetEulerDeg(eRoll) << " </roll>" << endl;
+      outfile << "  </orientation>" << endl;
+      outfile << "" << endl;
+      outfile << "  <velocity unit=\"FT/SEC\" frame=\"LOCAL\">" << endl;
+      outfile << "    <x> " << GetVel(eNorth) << " </x>" << endl;
+      outfile << "    <y> " << GetVel(eEast) << " </y>" << endl;
+      outfile << "    <z> " << GetVel(eDown) << " </z>" << endl;
+      outfile << "  </velocity>" << endl;
+      outfile << "" << endl;
+      outfile << "  <attitude_rate unit=\"DEG/SEC\" frame=\"BODY\">" << endl;
+      outfile << "    <roll> " << (VState.vPQR*radtodeg)(eRoll) << " </roll>" << endl;
+      outfile << "    <pitch> " << (VState.vPQR*radtodeg)(ePitch) << " </pitch>" << endl;
+      outfile << "    <yaw> " << (VState.vPQR*radtodeg)(eYaw) << " </yaw>" << endl;
+      outfile << "  </attitude_rate>" << endl;
+      outfile << "" << endl;
+      outfile << "</initialize>" << endl;
+      outfile.close();
+    } else {
+      cerr << "Could not open and/or write the state to the initial conditions file: " << filename << endl;
+    }
+    break;
+  default:
+    cerr << "When writing a state file, the supplied value must be 1 or 2 for the version number of the resulting IC file" << endl;
+  }
+}
+
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
 void FGPropagate::bind(void)
 {
   typedef double (FGPropagate::*PMF)(int) const;
+  typedef int (FGPropagate::*iPMF)(void) const;
 
   PropertyManager->Tie("velocities/h-dot-fps", this, &FGPropagate::Gethdot);
 
@@ -618,6 +720,7 @@ void FGPropagate::bind(void)
   PropertyManager->Tie("velocities/ri-rad_sec", this, eR, (PMF)&FGPropagate::GetPQRi);
 
   PropertyManager->Tie("velocities/eci-velocity-mag-fps", this, &FGPropagate::GetInertialVelocityMagnitude);
+  PropertyManager->Tie("velocities/ned-velocity-mag-fps", this, &FGPropagate::GetNEDVelocityMagnitude);
 
   PropertyManager->Tie("position/h-sl-ft", this, &FGPropagate::GetAltitudeASL, &FGPropagate::SetAltitudeASL, true);
   PropertyManager->Tie("position/h-sl-meters", this, &FGPropagate::GetAltitudeASLmeters, &FGPropagate::SetAltitudeASLmeters, true);
@@ -629,6 +732,8 @@ void FGPropagate::bind(void)
   PropertyManager->Tie("position/lat-geod-deg", this, &FGPropagate::GetGeodLatitudeDeg);
   PropertyManager->Tie("position/geod-alt-ft", this, &FGPropagate::GetGeodeticAltitude);
   PropertyManager->Tie("position/h-agl-ft", this,  &FGPropagate::GetDistanceAGL, &FGPropagate::SetDistanceAGL);
+  PropertyManager->Tie("position/geod-alt-km", this, &FGPropagate::GetGeodeticAltitudeKm);
+  PropertyManager->Tie("position/h-agl-km", this,  &FGPropagate::GetDistanceAGLKm, &FGPropagate::SetDistanceAGLKm);
   PropertyManager->Tie("position/radius-to-vehicle-ft", this, &FGPropagate::GetRadius);
   PropertyManager->Tie("position/terrain-elevation-asl-ft", this,
                           &FGPropagate::GetTerrainElevation,
@@ -641,6 +746,10 @@ void FGPropagate::bind(void)
   PropertyManager->Tie("attitude/theta-rad", this, (int)eTht, (PMF)&FGPropagate::GetEuler);
   PropertyManager->Tie("attitude/psi-rad", this, (int)ePsi, (PMF)&FGPropagate::GetEuler);
 
+  PropertyManager->Tie("attitude/phi-deg", this, (int)ePhi, (PMF)&FGPropagate::GetEulerDeg);
+  PropertyManager->Tie("attitude/theta-deg", this, (int)eTht, (PMF)&FGPropagate::GetEulerDeg);
+  PropertyManager->Tie("attitude/psi-deg", this, (int)ePsi, (PMF)&FGPropagate::GetEulerDeg);
+
   PropertyManager->Tie("attitude/roll-rad", this, (int)ePhi, (PMF)&FGPropagate::GetEuler);
   PropertyManager->Tie("attitude/pitch-rad", this, (int)eTht, (PMF)&FGPropagate::GetEuler);
   PropertyManager->Tie("attitude/heading-true-rad", this, (int)ePsi, (PMF)&FGPropagate::GetEuler);
@@ -649,6 +758,8 @@ void FGPropagate::bind(void)
   PropertyManager->Tie("simulation/integrator/rate/translational", (int*)&integrator_translational_rate);
   PropertyManager->Tie("simulation/integrator/position/rotational", (int*)&integrator_rotational_position);
   PropertyManager->Tie("simulation/integrator/position/translational", (int*)&integrator_translational_position);
+
+  PropertyManager->Tie("simulation/write-state-file", this, (iPMF)0, &FGPropagate::WriteStateFile);
 }
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
