@@ -59,14 +59,15 @@ INCLUDES
 #include "models/propulsion/FGTurboProp.h"
 #include "models/propulsion/FGTank.h"
 #include "input_output/FGPropertyManager.h"
-#include "input_output/FGXMLParse.h"
+#include "input_output/FGXMLFileRead.h"
+#include "input_output/FGXMLElement.h"
 #include "math/FGColumnVector3.h"
 
 using namespace std;
 
 namespace JSBSim {
 
-static const char *IdSrc = "$Id: FGPropulsion.cpp,v 1.62 2012/07/19 03:50:19 jberndt Exp $";
+static const char *IdSrc = "$Id: FGPropulsion.cpp,v 1.71 2013/11/24 11:40:56 bcoconni Exp $";
 static const char *IdHdr = ID_PROPULSION;
 
 extern short debug_lvl;
@@ -371,12 +372,26 @@ void FGPropulsion::InitRunning(int n)
 
 //%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 
-bool FGPropulsion::Load(Element* el)
+bool FGPropulsion::Load(Element* elem)
 {
   string type, engine_filename;
+  string separator = "/";
+  Element *el=0;
+  FGXMLFileRead XMLFileRead;
+  FGXMLParse main_file_parser;
 
   Debug(2);
 
+  string fname="", file="";
+  fname = elem->GetAttributeValue("file");
+  if (!fname.empty()) {
+    file = FDMExec->GetFullAircraftPath() + separator + fname;
+    el = XMLFileRead.LoadXMLDocument(file, main_file_parser);
+    if (el == 0L) return false;
+  } else {
+    el = elem;
+  }
+
   FGModel::Load(el); // Perform base class Load.
 
   // Process tank definitions first to establish the number of fuel tanks
@@ -408,7 +423,7 @@ bool FGPropulsion::Load(Element* el)
       return false;
     }
 
-    document = LoadXMLDocument(engine_filename);
+    Element* document = XMLFileRead.LoadXMLDocument(engine_filename);
     document->SetParent(engine_element);
 
     type = document->GetName();
@@ -445,7 +460,7 @@ bool FGPropulsion::Load(Element* el)
     numEngines++;
 
     engine_element = el->FindNextElement("engine");
-    ResetParser();
+    XMLFileRead.ResetParser();
   }
 
   CalculateTankInertias();
@@ -572,6 +587,9 @@ string FGPropulsion::GetPropulsionTankReport()
 {
   string out="";
   stringstream outstream;
+
+  /*const FGMatrix33& mTkI =*/ CalculateTankInertias();
+
   for (unsigned int i=0; i<numTanks; i++)
   {
     FGTank* tank = Tanks[i];
@@ -588,8 +606,8 @@ string FGPropulsion::GetPropulsionTankReport()
     outstream << highint << left << setw(4) << i << setw(30) << tankname << normint
       << right << setw(10) << tank->GetContents() << setw(8) << tank->GetXYZ(eX)
          << setw(8) << tank->GetXYZ(eY) << setw(8) << tank->GetXYZ(eZ)
-         << setw(12) << "*" << setw(12) << "*"
-         << setw(12) << "*" << endl;
+         << setw(12) << tank->GetIxx() << setw(12) << tank->GetIyy()
+         << setw(12) << tank->GetIzz() << endl;
   }
   return outstream.str();
 }
@@ -628,10 +646,8 @@ const FGMatrix33& FGPropulsion::CalculateTankInertias(void)
   tankJ = FGMatrix33();
 
   for (unsigned int i=0; i<size; i++) {
-    FGColumnVector3 vTankStructVec = in.vXYZcg - Tanks[i]->GetXYZ();
-
     tankJ += FDMExec->GetMassBalance()->GetPointmassInertia( lbtoslug * Tanks[i]->GetContents(),
-                                                             vTankStructVec);
+                                                             Tanks[i]->GetXYZ());
     tankJ(1,1) += Tanks[i]->GetIxx();
     tankJ(2,2) += Tanks[i]->GetIyy();
     tankJ(3,3) += Tanks[i]->GetIzz();
@@ -679,18 +695,32 @@ void FGPropulsion::SetStarter(int setting)
 
 void FGPropulsion::SetCutoff(int setting)
 {
+  bool bsetting = setting == 0 ? false : true;
+
   if (ActiveEngine < 0) {
     for (unsigned i=0; i<Engines.size(); i++) {
-      if (setting == 0)
-        ((FGTurbine*)Engines[i])->SetCutoff(false);
-      else
-        ((FGTurbine*)Engines[i])->SetCutoff(true);
+      switch (Engines[i]->GetType()) { 
+        case FGEngine::etTurbine:
+          ((FGTurbine*)Engines[i])->SetCutoff(bsetting);
+          break;
+        case FGEngine::etTurboprop:
+          ((FGTurboProp*)Engines[i])->SetCutoff(bsetting);
+          break;
+        default:
+          break;
+      }
     }
   } else {
-    if (setting == 0)
-      ((FGTurbine*)Engines[ActiveEngine])->SetCutoff(false);
-    else
-      ((FGTurbine*)Engines[ActiveEngine])->SetCutoff(true);
+    switch (Engines[ActiveEngine]->GetType()) { 
+      case FGEngine::etTurbine:
+        ((FGTurbine*)Engines[ActiveEngine])->SetCutoff(bsetting);
+        break;
+      case FGEngine::etTurboprop:
+        ((FGTurboProp*)Engines[ActiveEngine])->SetCutoff(bsetting);
+        break;
+      default:
+        break;
+    }
   }
 }
 
@@ -785,7 +815,7 @@ void FGPropulsion::bind(void)
 
   IsBound = true;
   PropertyManager->Tie("propulsion/set-running", this, (iPMF)0, &FGPropulsion::InitRunning, false);
-  if (HaveTurbineEngine) {
+  if (HaveTurbineEngine || HaveTurboPropEngine) {
     PropertyManager->Tie("propulsion/starter_cmd", this, (iPMF)0, &FGPropulsion::SetStarter,  false);
     PropertyManager->Tie("propulsion/cutoff_cmd", this,  (iPMF)0, &FGPropulsion::SetCutoff,   false);
   }
