@@ -38,12 +38,13 @@ INCLUDES
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
 
 #include "FGActuator.h"
+#include "input_output/FGXMLElement.h"
 
 using namespace std;
 
 namespace JSBSim {
 
-static const char *IdSrc = "$Id: FGActuator.cpp,v 1.23 2012/04/08 15:04:41 jberndt Exp $";
+static const char *IdSrc = "$Id: FGActuator.cpp,v 1.29 2013/11/24 11:40:56 bcoconni Exp $";
 static const char *IdHdr = ID_ACTUATOR;
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -62,7 +63,9 @@ FGActuator::FGActuator(FGFCS* fcs, Element* element) : FGFCSComponent(fcs, eleme
   PreviousRateLimOutput = 0.0;
   PreviousLagInput = PreviousLagOutput = 0.0;
   bias = lag = hysteresis_width = deadband_width = 0.0;
-  rate_limit = 0.0; // no limit
+  rate_limited = false;
+  rate_limit = rate_limit_incr = rate_limit_decr = 0.0; // no limit
+  rate_limit_incr_prop = rate_limit_decr_prop = 0;
   fail_zero = fail_hardover = fail_stuck = false;
   ca = cb = 0.0;
   initialized = 0;
@@ -74,9 +77,42 @@ FGActuator::FGActuator(FGFCS* fcs, Element* element) : FGFCSComponent(fcs, eleme
   if ( element->FindElement("hysteresis_width") ) {
     hysteresis_width = element->FindElementValueAsNumber("hysteresis_width");
   }
-  if ( element->FindElement("rate_limit") ) {
-    rate_limit = fabs(element->FindElementValueAsNumber("rate_limit"));
+
+  // There can be a single rate limit specified, or increasing and 
+  // decreasing rate limits specified, and rate limits can be numeric, or
+  // a property.
+  Element* ratelim_el = element->FindElement("rate_limit");
+  while ( ratelim_el ) {
+    rate_limited = true;
+    FGPropertyNode* rate_limit_prop=0;
+
+    string rate_limit_str = ratelim_el->GetDataLine();
+    trim(rate_limit_str);
+    if (is_number(rate_limit_str)) {
+      rate_limit = fabs(element->FindElementValueAsNumber("rate_limit"));
+    } else {
+      if (rate_limit_str[0] == '-') rate_limit_str.erase(0,1);
+      rate_limit_prop = PropertyManager->GetNode(rate_limit_str, true);
+      if (rate_limit_prop == 0)
+        std::cerr << "No such property, " << rate_limit_str << " for rate limiting" << std::endl;
+    }
+
+    if (ratelim_el->HasAttribute("sense")) {
+      string sense = ratelim_el->GetAttributeValue("sense");
+      if (sense.substr(0,4) == "incr") {
+        if (rate_limit_prop != 0) rate_limit_incr_prop = rate_limit_prop;
+        else                      rate_limit_incr = rate_limit;
+      } else if (sense.substr(0,4) == "decr") {
+        if (rate_limit_prop != 0) rate_limit_decr_prop = rate_limit_prop;
+        else                      rate_limit_decr = -rate_limit;
+      }
+    } else {
+      rate_limit_incr =  rate_limit;
+      rate_limit_decr = -rate_limit;
+    }
+    ratelim_el = element->FindNextElement("rate_limit");
   }
+
   if ( element->FindElement("bias") ) {
     bias = element->FindElementValueAsNumber("bias");
   }
@@ -122,7 +158,8 @@ bool FGActuator::Run(void )
     Output = PreviousOutput;
   } else {
     if (lag != 0.0)              Lag();        // models actuator lag
-    if (rate_limit != 0)         RateLimit();  // limit the actuator rate
+    if (rate_limit != 0 || (rate_limit_incr_prop != 0
+        || rate_limit_decr_prop != 0)) RateLimit();  // limit the actuator rate
     if (deadband_width != 0.0)   Deadband();
     if (hysteresis_width != 0.0) Hysteresis();
     if (bias != 0.0)             Bias();       // models a finite bias
@@ -136,8 +173,8 @@ bool FGActuator::Run(void )
 
   if (clip) {
     saturated = false;
-    if (Output >= clipmax) saturated = true;
-    else if (Output <= clipmin) saturated = true;
+    if (Output >= clipmax && clipmax != 0) saturated = true;
+    else if (Output <= clipmin && clipmin != 0) saturated = true;
   }
 
   if (IsOutput) SetOutput();
@@ -196,9 +233,12 @@ void FGActuator::RateLimit(void)
   double input = Output;
   if ( initialized ) {
     double delta = input - PreviousRateLimOutput;
-    if (fabs(delta) > dt * rate_limit) {
-      double signed_rate_limit = delta > 0.0 ? rate_limit : -rate_limit;
-      Output = PreviousRateLimOutput + signed_rate_limit * dt;
+    if (rate_limit_incr_prop != 0) rate_limit_incr = rate_limit_incr_prop->getDoubleValue();
+    if (rate_limit_decr_prop != 0) rate_limit_decr = rate_limit_decr_prop->getDoubleValue();
+    if (delta > dt * rate_limit_incr) {
+      Output = PreviousRateLimOutput + rate_limit_incr * dt;
+    } else if (delta < dt * rate_limit_decr) {
+      Output = PreviousRateLimOutput + rate_limit_decr * dt;
     }
   }
   PreviousRateLimOutput = Output;
@@ -276,7 +316,18 @@ void FGActuator::Debug(int from)
           cout << "      OUTPUT: " << OutputNodes[i]->getName() << endl;
       }
       if (bias != 0.0) cout << "      Bias: " << bias << endl;
-      if (rate_limit != 0) cout << "      Rate limit: " << rate_limit << endl;
+      if (rate_limited) {
+        if (rate_limit_incr_prop != 0) {
+          cout << "      Increasing rate limit: " << rate_limit_incr_prop->GetName() << endl;
+        } else {
+          cout << "      Increasing rate limit: " << rate_limit_incr << endl;
+        }
+        if (rate_limit_decr_prop != 0) {
+          cout << "      Decreasing rate limit: " << rate_limit_decr_prop->GetName() << endl;
+        } else {
+          cout << "      Decreasing rate limit: " << rate_limit_decr << endl;
+        }
+      }
       if (lag != 0) cout << "      Actuator lag: " << lag << endl;
       if (hysteresis_width != 0) cout << "      Hysteresis width: " << hysteresis_width << endl;
       if (deadband_width != 0) cout << "      Deadband width: " << deadband_width << endl;
