@@ -46,14 +46,15 @@ INCLUDES
 #include "FGPropeller.h"
 #include "FGNozzle.h"
 #include "FGRotor.h"
-#include "input_output/FGXMLParse.h"
+#include "input_output/FGXMLFileRead.h"
+#include "input_output/FGXMLElement.h"
 #include "math/FGColumnVector3.h"
 
 using namespace std;
 
 namespace JSBSim {
 
-static const char *IdSrc = "$Id: FGEngine.cpp,v 1.50 2012/03/17 20:46:29 jentron Exp $";
+static const char *IdSrc = "$Id: FGEngine.cpp,v 1.54 2013/11/24 11:40:57 bcoconni Exp $";
 static const char *IdHdr = ID_ENGINE;
 
 /*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -83,13 +84,14 @@ FGEngine::FGEngine(FGFDMExec* exec, Element* engine_element, int engine_number,
 
   Name = engine_element->GetAttributeValue("name");
 
-  Load(engine_element, PropertyManager, ::to_string(EngineNumber)); // Call ModelFunctions loader
+  Load(engine_element, PropertyManager, to_string((int)EngineNumber)); // Call ModelFunctions loader
 
 // Find and set engine location
 
   local_element = engine_element->GetParent()->FindElement("location");
   if (local_element)  location = local_element->FindElementTripletConvertTo("IN");
-  else      cerr << "No engine location found for this engine." << endl;
+//  else      cerr << "No engine location found for this engine." << endl;
+// Jon: The engine location is not important - the nozzle location is.
 
   local_element = engine_element->GetParent()->FindElement("orient");
   if (local_element)  orientation = local_element->FindElementTripletConvertTo("RAD");
@@ -132,7 +134,7 @@ FGEngine::FGEngine(FGFDMExec* exec, Element* engine_element, int engine_number,
   property_name = base_property_name + "/fuel-used-lbs";
   PropertyManager->Tie( property_name.c_str(), this, &FGEngine::GetFuelUsedLbs);
 
-  PostLoad(engine_element, PropertyManager, ::to_string(EngineNumber));
+  PostLoad(engine_element, PropertyManager, to_string((int)EngineNumber));
 
   Debug(0);
 }
@@ -267,7 +269,8 @@ bool FGEngine::LoadThruster(Element *thruster_element)
     return false;
   }
 
-  document = LoadXMLDocument(thruster_fullpathname);
+  FGXMLFileRead XMLFileRead;
+  Element *document = XMLFileRead.LoadXMLDocument(thruster_fullpathname);
   document->SetParent(thruster_element);
 
   thrType = document->GetName();
