@@ -30,6 +30,10 @@
 #  include <config.h>
 #endif
 
+#ifdef HAVE_WINDOWS_H
+#include <windows.h>
+#endif
+
 #include <simgear/compiler.h>
 
 #include <fstream>
@@ -60,8 +64,8 @@
 #  include <shellapi.h>
 #endif
 
-#ifdef SG_MAC
-# include "FGCocoaMenuBar.hxx" // for cocoaOpenUrl
+#if defined(SG_MAC)
+# include <GUI/CocoaHelpers.h> // for cocoaOpenUrl
 #endif
 
 #include "gui.h"
@@ -476,7 +480,7 @@ namespace
 
             string dir = fgGetString("/sim/paths/screenshot-dir");
             if (dir.empty())
-                dir = fgGetString("/sim/fg-current");
+                dir = SGPath::desktop().str();
 
             _path.set(dir + '/');
             if (_path.create_dir( 0755 )) {
