@@ -69,8 +69,7 @@ TACAN::~TACAN ()
 void
 TACAN::init ()
 {
-    string branch;
-    branch = "/instrumentation/" + _name;
+    string branch = "/instrumentation/" + _name;
 
     SGPropertyNode *node = fgGetNode(branch.c_str(), _num, true );
 
@@ -115,7 +114,6 @@ TACAN::init ()
     _mp_callsign_node = mnode ? mnode->getChild("callsign", 0, false) : 0;
 
     _heading_node = fgGetNode("/orientation/heading-deg", true);
-    _yaw_node = fgGetNode("/orientation/side-slip-deg", true);
     _electrical_node = fgGetNode("/systems/electrical/outputs/tacan", true);
 }
 
@@ -141,7 +139,7 @@ TACAN::update (double delta_time_sec)
         return;
     }
 
-  SGGeod pos(globals->get_aircraft_position());
+    SGGeod pos(globals->get_aircraft_position());
                                 // On timeout, scan again
     _time_before_search_sec -= delta_time_sec;
     if ((_time_before_search_sec < 0 || _new_frequency) && _frequency_mhz >= 0)
@@ -156,18 +154,20 @@ TACAN::update (double delta_time_sec)
 
     SG_LOG( SG_INSTR, SG_DEBUG, "mobile_name " << _mobile_name);
     SG_LOG( SG_INSTR, SG_DEBUG, "mobile_valid " << _mobile_valid);
-    geo_inverse_wgs_84(pos, _mobilePos,
-                       &mobile_bearing, &mobile_az2, &mobile_distance);
 
+    if (_mobile_valid) {
+        geo_inverse_wgs_84(pos, _mobilePos,
+                       &mobile_bearing, &mobile_az2, &mobile_distance);
+    }
 
     //calculate the bearing and range of the station from the aircraft
     double az2 = 0;
     double bearing = 0;
     double distance = 0;
-
-    geo_inverse_wgs_84(pos, _transmitter_pos,
-                       &bearing, &az2, &distance);
-
+    if (_transmitter_valid) {
+        geo_inverse_wgs_84(pos, _transmitter_pos,
+                           &bearing, &az2, &distance);
+    }
 
     //select the nearer
     if ( mobile_distance <= distance && _mobile_valid) {
