@@ -25,21 +25,32 @@
 #  include <config.h>
 #endif
 
+#include <simgear/compiler.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>             // strcmp()
 
-#ifdef _WIN32
+#if defined(SG_WINDOWS)
 #  include <io.h>               // isatty()
+#  include <process.h>          // _getpid()
+#  include <Windows.h>
 #  define isatty _isatty
+#else
+// for open() and options
+#  include <sys/types.h>        
+#  include <sys/stat.h>
+#  include <fcntl.h>
 #endif
 
-#include <simgear/compiler.h>
-
 #include <string>
+
 #include <boost/algorithm/string/compare.hpp>
 #include <boost/algorithm/string/predicate.hpp>
 
+#include <osgViewer/Viewer>
+
+#include <simgear/canvas/Canvas.hxx>
 #include <simgear/constants.h>
 #include <simgear/debug/logstream.hxx>
 #include <simgear/structure/exception.hxx>
@@ -50,16 +61,19 @@
 #include <simgear/misc/sgstream.hxx>
 #include <simgear/misc/strutils.hxx>
 #include <simgear/props/props_io.hxx>
+#include <simgear/scene/tsync/terrasync.hxx>
 
-#include <simgear/misc/interpolator.hxx>
+#include <simgear/scene/model/modellib.hxx>
 #include <simgear/scene/material/matlib.hxx>
+#include <simgear/scene/material/Effect.hxx>
 #include <simgear/scene/model/particles.hxx>
+#include <simgear/scene/tsync/terrasync.hxx>
 
 #include <Aircraft/controls.hxx>
 #include <Aircraft/replay.hxx>
 #include <Aircraft/FlightHistory.hxx>
 #include <Airports/runways.hxx>
-#include <Airports/simple.hxx>
+#include <Airports/airport.hxx>
 #include <Airports/dynamics.hxx>
 
 #include <AIModel/AIManager.hxx>
@@ -75,7 +89,9 @@
 
 #include <Canvas/canvas_mgr.hxx>
 #include <Canvas/gui_mgr.hxx>
+#include <Canvas/FGCanvasSystemAdapter.hxx>
 #include <GUI/new_gui.hxx>
+#include <GUI/MessageBox.hxx>
 #include <Input/input.hxx>
 #include <Instrumentation/instrument_mgr.hxx>
 #include <Model/acmodel.hxx>
@@ -91,6 +107,8 @@
 #include <Sound/soundmanager.hxx>
 #include <Systems/system_mgr.hxx>
 #include <Time/light.hxx>
+#include <Time/TimeManager.hxx>
+
 #include <Traffic/TrafficMgr.hxx>
 #include <MultiPlayer/multiplaymgr.hxx>
 #include <FDM/fdm_shell.hxx>
@@ -98,20 +116,33 @@
 #include <Environment/environment_mgr.hxx>
 #include <Viewer/renderer.hxx>
 #include <Viewer/viewmgr.hxx>
+#include <Viewer/FGEventHandler.hxx>
 #include <Navaids/NavDataCache.hxx>
 #include <Instrumentation/HUD/HUD.hxx>
 #include <Cockpit/cockpitDisplayManager.hxx>
 #include <Network/HTTPClient.hxx>
+#include <Network/fgcom.hxx>
+
+#include <Viewer/CameraGroup.hxx>
 
 #include "fg_init.hxx"
 #include "fg_io.hxx"
 #include "fg_commands.hxx"
 #include "fg_props.hxx"
+#include "FGInterpolator.hxx"
 #include "options.hxx"
 #include "globals.hxx"
 #include "logger.hxx"
 #include "main.hxx"
 #include "positioninit.hxx"
+#include "util.hxx"
+#include "AircraftDirVisitorBase.hxx"
+
+#if defined(SG_MAC)
+#include <GUI/CocoaHelpers.h> // for Mac impl of platformDefaultDataPath()
+#endif
+
+//#define NEW_RESET 1
 
 using std::string;
 using std::endl;
@@ -119,22 +150,19 @@ using std::cerr;
 using std::cout;
 using namespace boost::algorithm;
 
+extern osg::ref_ptr<osgViewer::Viewer> viewer;
 
 // Return the current base package version
 string fgBasePackageVersion() {
     SGPath base_path( globals->get_fg_root() );
     base_path.append("version");
-
+    if (!base_path.exists()) {
+        return string();
+    }
+    
     sg_gzifstream in( base_path.str() );
-    if ( !in.is_open() ) {
-        SGPath old_path( globals->get_fg_root() );
-        old_path.append( "Thanks" );
-        sg_gzifstream old( old_path.str() );
-        if ( !old.is_open() ) {
-            return "[none]";
-        } else {
-            return "[old version]";
-        }
+    if (!in.is_open()) {
+        return string();
     }
 
     string version;
@@ -143,68 +171,7 @@ string fgBasePackageVersion() {
     return version;
 }
 
-
-template <class T>
-bool fgFindAircraftInDir(const SGPath& dirPath, T* obj, bool (T::*pred)(const SGPath& p))
-{
-  if (!dirPath.exists()) {
-    SG_LOG(SG_GENERAL, SG_WARN, "fgFindAircraftInDir: no such path:" << dirPath.str());
-    return false;
-  }
-    
-  bool recurse = true;
-  simgear::Dir dir(dirPath);
-  simgear::PathList setFiles(dir.children(simgear::Dir::TYPE_FILE, "-set.xml"));
-  simgear::PathList::iterator p;
-  for (p = setFiles.begin(); p != setFiles.end(); ++p) {
-    // check file name ends with -set.xml
-    
-    // if we found a -set.xml at this level, don't recurse any deeper
-    recurse = false;
-    
-    bool done = (obj->*pred)(*p);
-    if (done) {
-      return true;
-    }
-  } // of -set.xml iteration
-  
-  if (!recurse) {
-    return false;
-  }
-  
-  simgear::PathList subdirs(dir.children(simgear::Dir::TYPE_DIR | simgear::Dir::NO_DOT_OR_DOTDOT));
-  for (p = subdirs.begin(); p != subdirs.end(); ++p) {
-    if (p->file() == "CVS") {
-      continue;
-    }
-    
-    if (fgFindAircraftInDir(*p, obj, pred)) {
-      return true;
-    }
-  } // of subdirs iteration
-  
-  return false;
-}
-
-template <class T>
-void fgFindAircraft(T* obj, bool (T::*pred)(const SGPath& p))
-{
-  const string_list& paths(globals->get_aircraft_paths());
-  string_list::const_iterator it = paths.begin();
-  for (; it != paths.end(); ++it) {
-    bool done = fgFindAircraftInDir(SGPath(*it), obj, pred);
-    if (done) {
-      return;
-    }
-  } // of aircraft paths iteration
-  
-  // if we reach this point, search the default location (always last)
-  SGPath rootAircraft(globals->get_fg_root());
-  rootAircraft.append("Aircraft");
-  fgFindAircraftInDir(rootAircraft, obj, pred);
-}
-
-class FindAndCacheAircraft
+class FindAndCacheAircraft : public AircraftDirVistorBase
 {
 public:
   FindAndCacheAircraft(SGPropertyNode* autoSave)
@@ -216,6 +183,7 @@ public:
   {
     std::string aircraft = fgGetString( "/sim/aircraft", "");
     if (aircraft.empty()) {
+        flightgear::fatalMessageBox("No aircraft", "No aircraft was specified");
       SG_LOG(SG_GENERAL, SG_ALERT, "no aircraft specified");
       return false;
     }
@@ -233,6 +201,9 @@ public:
           readProperties(setFile.str(), globals->get_props());
         } catch ( const sg_exception &e ) {
           SG_LOG(SG_INPUT, SG_ALERT, "Error reading aircraft: " << e.getFormattedMessage());
+            flightgear::fatalMessageBox("Error reading aircraft",
+                                        "An error occured reading the requested aircraft (" + aircraft + ")",
+                                        e.getFormattedMessage());
           return false;
         }
         
@@ -240,6 +211,9 @@ public:
       } else {
         SG_LOG(SG_GENERAL, SG_ALERT, "aircraft '" << _searchAircraft << 
                "' not found in specified dir:" << aircraftDir);
+          flightgear::fatalMessageBox("Aircraft not found",
+                                      "The requested aircraft '" + aircraft + "' could not be found in the specified location.",
+                                      aircraftDir);
         return false;
       }
     }
@@ -254,11 +228,14 @@ public:
       n->setAttribute(SGPropertyNode::USERARCHIVE, true);
       _cache->removeChildren("aircraft");
   
-      fgFindAircraft(this, &FindAndCacheAircraft::checkAircraft);
+        visitAircraftPaths();
     }
     
     if (_foundPath.str().empty()) {
       SG_LOG(SG_GENERAL, SG_ALERT, "Cannot find specified aircraft: " << aircraft );
+        flightgear::fatalMessageBox("Aircraft not found",
+                                    "The requested aircraft '" + aircraft + "' could not be found in any of the search paths");
+
       return false;
     }
     
@@ -273,6 +250,9 @@ public:
       readProperties(_foundPath.str(), globals->get_props());
     } catch ( const sg_exception &e ) {
       SG_LOG(SG_INPUT, SG_ALERT, "Error reading aircraft: " << e.getFormattedMessage());
+        flightgear::fatalMessageBox("Error reading aircraft",
+                                    "An error occured reading the requested aircraft (" + aircraft + ")",
+                                    e.getFormattedMessage());
       return false;
     }
     
@@ -324,7 +304,7 @@ private:
     return false;
   }
   
-  bool checkAircraft(const SGPath& p)
+  virtual VisitResult visit(const SGPath& p)
   {
     // create cache node
     int i = 0;
@@ -346,10 +326,10 @@ private:
 
     if ( boost::equals(fileName, _searchAircraft.c_str(), is_iequal()) ) {
         _foundPath = p;
-        return true;
+        return VISIT_DONE;
     }
 
-    return false;
+    return VISIT_CONTINUE;
   }
   
   std::string _searchAircraft;
@@ -365,51 +345,98 @@ static SGPath platformDefaultDataPath()
   config.append( "flightgear.org" );
   return config;
 }
-#elif __APPLE__
 
-#include <CoreServices/CoreServices.h>
+#elif defined(SG_MAC)
+
+// platformDefaultDataPath defined in GUI/CocoaHelpers.h
 
-static SGPath platformDefaultDataPath()
-{
-  FSRef ref;
-  OSErr err = FSFindFolder(kUserDomain, kApplicationSupportFolderType, false, &ref);
-  if (err) {
-    return SGPath();
-  }
-  
-  unsigned char path[1024];
-  if (FSRefMakePath(&ref, path, 1024) != noErr) {
-    return SGPath();
-  }
-  
-  SGPath appData;
-  appData.set((const char*) path);
-  appData.append("FlightGear");
-  return appData;
-}
 #else
 static SGPath platformDefaultDataPath()
 {
-  SGPath config( homedir );
-  config.append( ".fgfs" );
-  return config;
+  return SGPath::home() / ".fgfs";
 }
 #endif
 
-// Read in configuration (file and command line)
-bool fgInitConfig ( int argc, char **argv )
+bool fgInitHome()
 {
-    SGPath dataPath = platformDefaultDataPath();
+  SGPath dataPath = SGPath::fromEnv("FG_HOME", platformDefaultDataPath());
+  globals->set_fg_home(dataPath.c_str());
     
-    const char *fg_home = getenv("FG_HOME");
-    if (fg_home)
-      dataPath = fg_home;
-      
-    globals->set_fg_home(dataPath.c_str());
+    simgear::Dir fgHome(dataPath);
+    if (!fgHome.exists()) {
+        fgHome.create(0755);
+    }
+    
+    if (!fgHome.exists()) {
+        flightgear::fatalMessageBox("Problem setting up user data",
+                                    "Unable to create the user-data storage folder at: '"
+                                    + dataPath.str() + "'");
+        return false;
+    }
+    
+    if (fgGetBool("/sim/fghome-readonly", false)) {
+        // user / config forced us into readonly mode, fine
+        SG_LOG(SG_GENERAL, SG_INFO, "Running with FG_HOME readonly");
+        return true;
+    }
+    
+// write our PID, and check writeability
+    SGPath pidPath(dataPath, "fgfs.pid");
+    if (pidPath.exists()) {
+        SG_LOG(SG_GENERAL, SG_INFO, "flightgear instance already running, switching to FG_HOME read-only.");
+        // set a marker property so terrasync/navcache don't try to write
+        // from secondary instances
+        fgSetBool("/sim/fghome-readonly", true);
+        return true;
+    }
+    
+    char buf[16];
+    bool result = false;
+#if defined(SG_WINDOWS)
+    size_t len = snprintf(buf, 16, "%d", _getpid());
+
+    HANDLE f = CreateFileA(pidPath.c_str(), GENERIC_READ | GENERIC_WRITE, 
+						   FILE_SHARE_READ, /* sharing */
+                           NULL, /* security attributes */
+                           CREATE_NEW, /* error if already exists */
+                           FILE_FLAG_DELETE_ON_CLOSE,
+						   NULL /* template */);
+    
+    result = (f != INVALID_HANDLE_VALUE);
+    if (result) {
+		DWORD written;
+        WriteFile(f, buf, len, &written, NULL /* overlapped */);
+    }
+#else
+    // POSIX, do open+unlink trick to the file is deleted on exit, even if we
+    // crash or exit(-1)
+    ssize_t len = snprintf(buf, 16, "%d", getpid());
+    int fd = ::open(pidPath.c_str(), O_WRONLY | O_CREAT | O_TRUNC | O_EXCL, 0644);
+    if (fd >= 0) {
+        result = ::write(fd, buf, len) == len;
+        if( ::unlink(pidPath.c_str()) != 0 ) // delete file when app quits
+          result = false;
+    }
+#endif
+
+    fgSetBool("/sim/fghome-readonly", false);
+
+    if (!result) {
+        flightgear::fatalMessageBox("File permissions problem",
+                                    "Can't write to user-data storage folder, check file permissions and FG_HOME.",
+                                    "User-data at:" + dataPath.str());
+    }
+    return result;
+}
+
+// Read in configuration (file and command line)
+int fgInitConfig ( int argc, char **argv, bool reinit )
+{
+    SGPath dataPath = globals->get_fg_home();
     
     simgear::Dir exportDir(simgear::Dir(dataPath).file("Export"));
     if (!exportDir.exists()) {
-      exportDir.create(0777);
+      exportDir.create(0755);
     }
     
     // Set /sim/fg-home and don't allow malign code to override it until
@@ -420,17 +447,22 @@ bool fgInitConfig ( int argc, char **argv )
     home->setStringValue(dataPath.c_str());
     home->setAttribute(SGPropertyNode::WRITE, false);
   
+    fgSetDefaults();
     flightgear::Options* options = flightgear::Options::sharedInstance();
-    options->init(argc, argv, dataPath);
-    bool loadDefaults = flightgear::Options::sharedInstance()->shouldLoadDefaultConfig();
+    if (!reinit) {
+        options->init(argc, argv, dataPath);
+    }
+    
+    bool loadDefaults = options->shouldLoadDefaultConfig();
     if (loadDefaults) {
       // Read global preferences from $FG_ROOT/preferences.xml
       SG_LOG(SG_INPUT, SG_INFO, "Reading global preferences");
       fgLoadProps("preferences.xml", globals->get_props());
       SG_LOG(SG_INPUT, SG_INFO, "Finished Reading global preferences");
-
-      // do not load user settings when reset to default is requested
-      if (flightgear::Options::sharedInstance()->isOptionSet("restore-defaults"))
+        
+      // do not load user settings when reset to default is requested, or if
+      // told to explicitly ignore
+      if (options->isOptionSet("restore-defaults") || options->isOptionSet("ignore-autosave"))
       {
           SG_LOG(SG_ALL, SG_ALERT, "Ignoring user settings. Restoring defaults.");
       }
@@ -441,29 +473,30 @@ bool fgInitConfig ( int argc, char **argv )
     } else {
       SG_LOG(SG_GENERAL, SG_INFO, "not reading default configuration files");
     }// of no-default-config selected
-  
-    // Scan user config files and command line for a specified aircraft.
-    options->initAircraft();
-
-    FindAndCacheAircraft f(globals->get_props());
-    if (!f.loadAircraft()) {
-      return false;
-    }
-
-    // parse options after loading aircraft to ensure any user
-    // overrides of defaults are honored.
-    options->processOptions();
-      
-    return true;
+    
+    return flightgear::FG_OPTIONS_OK;
 }
 
-
+int fgInitAircraft(bool reinit)
+{
+    // Scan user config files and command line for a specified aircraft.
+    if (!reinit) {
+        flightgear::Options::sharedInstance()->initAircraft();
+    }
+    
+    FindAndCacheAircraft f(globals->get_props());
+    if (!f.loadAircraft()) {
+        return flightgear::FG_OPTIONS_ERROR;
+    }
+    
+    return flightgear::FG_OPTIONS_OK;
+}
 
 /**
  * Initialize vor/ndb/ils/fix list management and query systems (as
  * well as simple airport db list)
  * This is called multiple times in the case of a cache rebuild,
- * to allow length caching to take place in the background, without
+ * to allow lengthy caching to take place in the background, without
  * blocking the main/UI thread.
  */
 bool
@@ -504,7 +537,7 @@ bool fgInitGeneral() {
         SG_LOG( SG_GENERAL, SG_ALERT,
                 "Cannot continue without a path to the base package "
                 << "being defined." );
-        exit(-1);
+        return false;
     }
     SG_LOG( SG_GENERAL, SG_INFO, "FG_ROOT = " << '"' << root << '"' << endl );
 
@@ -524,11 +557,25 @@ bool fgInitGeneral() {
     return true;
 }
 
+// Write various configuraton values out to the logs
+void fgOutputSettings()
+{    
+    SG_LOG( SG_GENERAL, SG_INFO, "Configuration State" );
+    SG_LOG( SG_GENERAL, SG_INFO, "======= ==============" );
+    
+    SG_LOG( SG_GENERAL, SG_INFO, "aircraft-dir = " << '"' << fgGetString("/sim/aircraft-dir") << '"' );
+    SG_LOG( SG_GENERAL, SG_INFO, "fghome-dir = " << '"' << globals->get_fg_home() << '"');
+    SG_LOG( SG_GENERAL, SG_INFO, "aircraft-dir = " << '"' << fgGetString("/sim/aircraft-dir") << '"');
+    
+    SG_LOG( SG_GENERAL, SG_INFO, "aircraft-search-paths = \n\t" << simgear::strutils::join(globals->get_aircraft_paths(), "\n\t") );
+    SG_LOG( SG_GENERAL, SG_INFO, "scenery-search-paths = \n\t" << simgear::strutils::join(globals->get_fg_scenery(), "\n\t") );
+}
+
 // This is the top level init routine which calls all the other
 // initialization routines.  If you are adding a subsystem to flight
 // gear, its initialization call should located in this routine.
 // Returns non-zero if a problem encountered.
-void fgCreateSubsystems() {
+void fgCreateSubsystems(bool duringReset) {
 
     SG_LOG( SG_GENERAL, SG_INFO, "Creating Subsystems");
     SG_LOG( SG_GENERAL, SG_INFO, "========== ==========");
@@ -553,8 +600,10 @@ void fgCreateSubsystems() {
     // Initialize the property interpolator subsystem. Put into the INIT
     // group because the "nasal" subsystem may need it at GENERAL take-down.
     ////////////////////////////////////////////////////////////////////
-    globals->add_subsystem("interpolator", new SGInterpolator, SGSubsystemMgr::INIT);
-
+    FGInterpolator* interp = new FGInterpolator;
+    interp->setRealtimeProperty(fgGetNode("/sim/time/delta-realtime-sec", true));
+    globals->add_subsystem("prop-interpolator", interp, SGSubsystemMgr::INIT);
+    SGPropertyNode::setInterpolationMgr(interp);
 
     ////////////////////////////////////////////////////////////////////
     // Add the FlightGear property utilities.
@@ -577,11 +626,9 @@ void fgCreateSubsystems() {
     mpath.append( fgGetString("/sim/rendering/materials-file") );
     if ( ! globals->get_matlib()->load(globals->get_fg_root(), mpath.str(),
             globals->get_props()) ) {
-        SG_LOG( SG_GENERAL, SG_ALERT,
-                "Error loading materials file " << mpath.str() );
-        exit(-1);
+       throw sg_io_exception("Error loading materials file", mpath);
     }
-
+    
     globals->add_subsystem( "http", new FGHTTPClient );
     
     ////////////////////////////////////////////////////////////////////
@@ -649,6 +696,9 @@ void fgCreateSubsystems() {
     ////////////////////////////////////////////////////////////////////
     // Initialize the canvas 2d drawing subsystem.
     ////////////////////////////////////////////////////////////////////
+    simgear::canvas::Canvas::setSystemAdapter(
+      simgear::canvas::SystemAdapterPtr(new canvas::FGCanvasSystemAdapter)
+    );
     globals->add_subsystem("Canvas", new CanvasMgr, SGSubsystemMgr::DISPLAY);
     globals->add_subsystem("CanvasGUI", new GUIMgr, SGSubsystemMgr::DISPLAY);
 
@@ -721,14 +771,22 @@ void fgCreateSubsystems() {
     globals->add_subsystem("voice", new FGVoiceMgr, SGSubsystemMgr::DISPLAY);
 #endif
 
+#ifdef ENABLE_IAX
+    ////////////////////////////////////////////////////////////////////
+    // Initialize the FGCom subsystem.
+    ////////////////////////////////////////////////////////////////////
+    globals->add_subsystem("fgcom", new FGCom);
+#endif
+
     ////////////////////////////////////////////////////////////////////
     // Initialize the lighting subsystem.
     ////////////////////////////////////////////////////////////////////
 
-    globals->add_subsystem("lighting", new FGLight, SGSubsystemMgr::DISPLAY);
-    
     // ordering here is important : Nasal (via events), then models, then views
-    globals->add_subsystem("events", globals->get_event_mgr(), SGSubsystemMgr::DISPLAY);
+    if (!duringReset) {
+        globals->add_subsystem("lighting", new FGLight, SGSubsystemMgr::DISPLAY);
+        globals->add_subsystem("events", globals->get_event_mgr(), SGSubsystemMgr::DISPLAY);
+    }
 
     globals->add_subsystem("aircraft-model", new FGAircraftModel, SGSubsystemMgr::DISPLAY);
     globals->add_subsystem("model-manager", new FGModelMgr, SGSubsystemMgr::DISPLAY);
@@ -752,6 +810,29 @@ void fgPostInitSubsystems()
     globals->add_subsystem("nasal", nasal, SGSubsystemMgr::INIT);
     nasal->init();
     SG_LOG(SG_GENERAL, SG_INFO, "Nasal init took:" << st.elapsedMSec());
+
+    // Ensure IOrules and path validation are working properly by trying to
+    // access a folder/file which should never be accessible.
+    const char* no_access_path =
+#ifdef _WIN32
+      "Z:"
+#endif
+      "/do-not-access";
+
+    if( fgValidatePath(no_access_path, true) )
+      SG_LOG
+      (
+        SG_GENERAL,
+        SG_ALERT,
+        "Check your IOrules! (write to '" << no_access_path << "' is allowed)"
+      );
+    if( fgValidatePath(no_access_path, false) )
+      SG_LOG
+      (
+        SG_GENERAL,
+        SG_ALERT,
+        "Check your IOrules! (read from '" << no_access_path << "' is allowed)"
+      );
   
     // initialize methods that depend on other subsystems.
     st.stamp();
@@ -775,6 +856,11 @@ void fgPostInitSubsystems()
 // Reset: this is what the 'reset' command (and hence, GUI) is attached to
 void fgReInitSubsystems()
 {
+#ifdef NEW_RESET
+    fgResetIdleState();
+    return;
+#endif
+    
     SGPropertyNode *master_freeze = fgGetNode("/sim/freeze/master");
 
     SG_LOG( SG_GENERAL, SG_INFO, "fgReInitSubsystems()");
@@ -802,6 +888,12 @@ void fgReInitSubsystems()
     // main-loop
     flightgear::initPosition();
     
+    simgear::SGTerraSync* terraSync =
+        static_cast<simgear::SGTerraSync*>(globals->get_subsystem("terrasync"));
+    if (terraSync) {
+        terraSync->reposition();
+    }
+    
     // Force reupdating the positions of the ai 3d models. They are used for
     // initializing ground level for the FDM.
     globals->get_subsystem("ai-model")->reinit();
@@ -845,120 +937,121 @@ void fgReInitSubsystems()
     fgSetBool("/sim/sceneryloaded",false);
 }
 
-
-///////////////////////////////////////////////////////////////////////////////
-// helper object to implement the --show-aircraft command.
-// resides here so we can share the fgFindAircraftInDir template above,
-// and hence ensure this command lists exectly the same aircraft as the normal
-// loading path.
-class ShowAircraft 
+void fgStartNewReset()
 {
-public:
-  ShowAircraft()
-  {
-    _minStatus = getNumMaturity(fgGetString("/sim/aircraft-min-status", "all"));
-  }
-  
-  
-  void show(const SGPath& path)
-  {
-    fgFindAircraftInDir(path, this, &ShowAircraft::processAircraft);
-  
-    std::sort(_aircraft.begin(), _aircraft.end(), ciLessLibC());
-    SG_LOG( SG_GENERAL, SG_ALERT, "" ); // To popup the console on Windows
-    cout << "Available aircraft:" << endl;
-    for ( unsigned int i = 0; i < _aircraft.size(); i++ ) {
-        cout << _aircraft[i] << endl;
-    }
-  }
-  
-private:
-  bool processAircraft(const SGPath& path)
-  {
-    SGPropertyNode root;
-    try {
-       readProperties(path.str(), &root);
-    } catch (sg_exception& ) {
-       return false;
-    }
-  
-    int maturity = 0;
-    string descStr("   ");
-    descStr += path.file();
-  // trim common suffix from file names
-    int nPos = descStr.rfind("-set.xml");
-    if (nPos == (int)(descStr.size() - 8)) {
-      descStr.resize(nPos);
-    }
+    SGPropertyNode_ptr preserved(new SGPropertyNode);
     
-    SGPropertyNode *node = root.getNode("sim");
-    if (node) {
-      SGPropertyNode* desc = node->getNode("description");
-      // if a status tag is found, read it in
-      if (node->hasValue("status")) {
-        maturity = getNumMaturity(node->getStringValue("status"));
-      }
-      
-      if (desc) {
-        if (descStr.size() <= 27+3) {
-          descStr.append(29+3-descStr.size(), ' ');
-        } else {
-          descStr += '\n';
-          descStr.append( 32, ' ');
+    if (!copyPropertiesWithAttribute(globals->get_props(), preserved, SGPropertyNode::PRESERVE))
+        SG_LOG(SG_GENERAL, SG_ALERT, "Error saving preserved state");
+    
+    fgSetBool("/sim/signals/reinit", true);
+    fgSetBool("/sim/freeze/master", true);
+    
+    SGSubsystemMgr* subsystemManger = globals->get_subsystem_mgr();
+    // Nasal is manually inited in fgPostInit, ensure it's already shutdown
+    // before other subsystems, so Nasal listeners don't fire during shutdonw
+    SGSubsystem* nasal = subsystemManger->get_subsystem("nasal");
+    nasal->shutdown();
+    nasal->unbind();
+    subsystemManger->remove("nasal");
+    
+    subsystemManger->shutdown();
+    subsystemManger->unbind();
+    
+    // remove most subsystems, with a few exceptions.
+    for (int g=0; g<SGSubsystemMgr::MAX_GROUPS; ++g) {
+        SGSubsystemGroup* grp = subsystemManger->get_group(static_cast<SGSubsystemMgr::GroupType>(g));
+        const string_list& names(grp->member_names());
+        string_list::const_iterator it;
+        for (it = names.begin(); it != names.end(); ++it) {
+            if ((*it == "time") || (*it == "terrasync") || (*it == "events")
+                || (*it == "lighting"))
+            {
+                continue;
+            }
+            
+            try {
+                subsystemManger->remove(it->c_str());
+            } catch (std::exception& e) {
+                SG_LOG(SG_GENERAL, SG_INFO, "caught std::exception shutting down:" << *it);
+            } catch (...) {
+                SG_LOG(SG_GENERAL, SG_INFO, "caught generic exception shutting down:" << *it);
+            }
+            
+            // don't delete here, dropping the ref should be sufficient
         }
-        descStr += desc->getStringValue();
-      }
-    } // of have 'sim' node
+    } // of top-level groups iteration
     
-    if (maturity < _minStatus) {
-      return false;
-    }
-
-    _aircraft.push_back(descStr);
-    return false;
-  }
-
-
-  int getNumMaturity(const char * str) 
-  {
-    // changes should also be reflected in $FG_ROOT/data/options.xml & 
-    // $FG_ROOT/data/Translations/string-default.xml
-    const char* levels[] = {"alpha","beta","early-production","production"}; 
-
-    if (!strcmp(str, "all")) {
-      return 0;
-    }
-
-    for (size_t i=0; i<(sizeof(levels)/sizeof(levels[0]));i++) 
-      if (strcmp(str,levels[i])==0)
-        return i;
-
-    return 0;
-  }
-
-  // recommended in Meyers, Effective STL when internationalization and embedded
-  // NULLs aren't an issue.  Much faster than the STL or Boost lex versions.
-  struct ciLessLibC : public std::binary_function<string, string, bool>
-  {
-    bool operator()(const std::string &lhs, const std::string &rhs) const
-    {
-      return strcasecmp(lhs.c_str(), rhs.c_str()) < 0 ? 1 : 0;
-    }
-  };
-
-  int _minStatus;
-  string_list _aircraft;
-};
-
-void fgShowAircraft(const SGPath &path)
-{
-    ShowAircraft s;
-    s.show(path);
+    // order is important here since tile-manager shutdown needs to
+    // access the scenery object
+    globals->set_tile_mgr(NULL);
+    globals->set_scenery(NULL);
+    flightgear::CameraGroup::setDefault(NULL);
+    
+    FGRenderer* render = globals->get_renderer();
+    // don't cancel the pager until after shutdown, since AIModels (and
+    // potentially others) can queue delete requests on the pager.
+    render->getViewer()->getDatabasePager()->cancel();
+    render->getViewer()->getDatabasePager()->clear();
+    
+    osgDB::Registry::instance()->clearObjectCache();
+    
+    // preserve the event handler; re-creating it would entail fixing the
+    // idle handler
+    osg::ref_ptr<flightgear::FGEventHandler> eventHandler = render->getEventHandler();
+    
+    globals->set_renderer(NULL);
+    globals->set_matlib(NULL);
+    globals->set_chatter_queue(NULL);
+    
+    simgear::clearEffectCache();
+    simgear::SGModelLib::resetPropertyRoot();
         
-#ifdef _MSC_VER
-    cout << "Hit a key to continue..." << endl;
-    std::cin.get();
-#endif
+    simgear::GlobalParticleCallback::setSwitch(NULL);
+    
+    globals->resetPropertyRoot();
+    fgInitConfig(0, NULL, true);
+    fgInitGeneral(); // all of this?
+    
+    if ( copyProperties(preserved, globals->get_props()) ) {
+        SG_LOG( SG_GENERAL, SG_INFO, "Preserved state restored successfully" );
+    } else {
+        SG_LOG( SG_GENERAL, SG_INFO,
+               "Some errors restoring preserved state (read-only props?)" );
+    }
+
+    fgGetNode("/sim")->removeChild("aircraft-dir");
+    fgInitAircraft(true);
+    flightgear::Options::sharedInstance()->processOptions();
+    
+    render = new FGRenderer;
+    render->setEventHandler(eventHandler);
+    globals->set_renderer(render);
+    render->init();
+    render->setViewer(viewer.get());
+
+    viewer->getDatabasePager()->setUpThreads(1, 1);
+    
+    // must do this before splashinit for Rembrandt
+    flightgear::CameraGroup::buildDefaultGroup(viewer.get());
+    render->splashinit();
+    
+    fgOSResetProperties();
+
+// init some things manually
+// which do not follow the regular init pattern
+    
+    globals->get_event_mgr()->init();
+    globals->get_event_mgr()->setRealtimeProperty(fgGetNode("/sim/time/delta-realtime-sec", true));
+    
+    globals->set_matlib( new SGMaterialLib );
+    
+// terra-sync needs the property tree root, pass it back in
+    simgear::SGTerraSync* terra_sync = static_cast<simgear::SGTerraSync*>(subsystemManger->get_subsystem("terrasync"));
+    terra_sync->setRoot(globals->get_props());
+
+    fgSetBool("/sim/signals/reinit", false);
+    fgSetBool("/sim/freeze/master", false);
+    fgSetBool("/sim/sceneryloaded",false);
 }
 
-
