@@ -27,6 +27,9 @@
 #include <boost/foreach.hpp>
 #include <algorithm>
 
+#include <osgViewer/Viewer>
+#include <osgDB/Registry>
+
 #include <simgear/structure/commands.hxx>
 #include <simgear/misc/sg_path.hxx>
 #include <simgear/misc/sg_dir.hxx>
@@ -39,6 +42,7 @@
 #include <simgear/misc/ResourceManager.hxx>
 #include <simgear/props/propertyObject.hxx>
 #include <simgear/props/props_io.hxx>
+#include <simgear/scene/model/modellib.hxx>
 
 #include <Aircraft/controls.hxx>
 #include <Airports/runways.hxx>
@@ -52,6 +56,7 @@
 #include <Navaids/navlist.hxx>
 #include <Viewer/renderer.hxx>
 #include <Viewer/viewmgr.hxx>
+#include <Sound/sample_queue.hxx>
 
 #include "globals.hxx"
 #include "locale.hxx"
@@ -85,7 +90,6 @@ public:
         }
         
         if (r.exists()) {
-          SG_LOG(SG_IO, SG_DEBUG, "found path:" << aResource << " via /sim/aircraft-dir: " << r.str());
           return r;
         }
     }
@@ -97,7 +101,6 @@ public:
     for (; it != dirs.end(); ++it) {
       SGPath p(*it, res);
       if (p.exists()) {
-        SG_LOG(SG_IO, SG_DEBUG, "found path:" << aResource << " in aircraft dir: " << *it);
         return p;
       }
     } // of aircraft path iteration
@@ -128,14 +131,12 @@ public:
 ////////////////////////////////////////////////////////////////////////
 
 // global global :-)
-FGGlobals *globals;
+FGGlobals *globals = NULL;
 
 
 // Constructor
 FGGlobals::FGGlobals() :
-    props( new SGPropertyNode ),
     initial_state( NULL ),
-    locale( new FGLocale(props) ),
     renderer( new FGRenderer ),
     subsystem_mgr( new SGSubsystemMgr ),
     event_mgr( new SGEventMgr ),
@@ -144,7 +145,6 @@ FGGlobals::FGGlobals() :
     fg_home( "" ),
     time_params( NULL ),
     ephem( NULL ),
-    matlib( NULL ),
     route_mgr( NULL ),
     ATIS_mgr( NULL ),
     controls( NULL ),
@@ -152,27 +152,36 @@ FGGlobals::FGGlobals() :
     commands( SGCommandMgr::instance() ),
     channel_options_list( NULL ),
     initial_waypoints( NULL ),
-    scenery( NULL ),
-    tile_mgr( NULL ),
     fontcache ( new FGFontCache ),
     channellist( NULL ),
-    haveUserSettings(false)
+    haveUserSettings(false),
+    _chatter_queue(NULL)
 {
-  simgear::ResourceManager::instance()->addProvider(new AircraftResourceProvider);
-  simgear::ResourceManager::instance()->addProvider(new CurrentAircraftDirProvider);
-  simgear::PropertyObjectBase::setDefaultRoot(props);
-  
-  positionLon = props->getNode("position/longitude-deg", true);
-  positionLat = props->getNode("position/latitude-deg", true);
-  positionAlt = props->getNode("position/altitude-ft", true);
-  
-  viewLon = props->getNode("sim/current-view/viewer-lon-deg", true);
-  viewLat = props->getNode("sim/current-view/viewer-lat-deg", true);
-  viewAlt = props->getNode("sim/current-view/viewer-elev-ft", true);
-  
-  orientPitch = props->getNode("orientation/pitch-deg", true);
-  orientHeading = props->getNode("orientation/heading-deg", true);
-  orientRoll = props->getNode("orientation/roll-deg", true);
+    SGPropertyNode* root = new SGPropertyNode;
+    props = SGPropertyNode_ptr(root);
+    locale = new FGLocale(props);
+    
+    simgear::ResourceManager::instance()->addProvider(new AircraftResourceProvider);
+    simgear::ResourceManager::instance()->addProvider(new CurrentAircraftDirProvider);
+    initProperties();
+}
+
+void FGGlobals::initProperties()
+{
+    simgear::PropertyObjectBase::setDefaultRoot(props);
+    
+    positionLon = props->getNode("position/longitude-deg", true);
+    positionLat = props->getNode("position/latitude-deg", true);
+    positionAlt = props->getNode("position/altitude-ft", true);
+    
+    viewLon = props->getNode("sim/current-view/viewer-lon-deg", true);
+    viewLat = props->getNode("sim/current-view/viewer-lat-deg", true);
+    viewAlt = props->getNode("sim/current-view/viewer-elev-ft", true);
+    
+    orientPitch = props->getNode("orientation/pitch-deg", true);
+    orientHeading = props->getNode("orientation/heading-deg", true);
+    orientRoll = props->getNode("orientation/roll-deg", true);
+
 }
 
 // Destructor
@@ -187,40 +196,72 @@ FGGlobals::~FGGlobals()
     // deallocation of AIModel objects. To ensure we can safely
     // shut down all subsystems, make sure we take down the 
     // AIModels system first.
-    SGSubsystem* ai = subsystem_mgr->remove("ai-model");
+    SGSubsystemRef ai = subsystem_mgr->get_subsystem("ai-model");
     if (ai) {
+        subsystem_mgr->remove("ai-model");
         ai->unbind();
-        delete ai;
+        ai.clear(); // ensure AI is deleted now, not at end of this method
     }
-    SGSubsystem* sound = subsystem_mgr->remove("sound");
-
+    
     subsystem_mgr->shutdown();
-    subsystem_mgr->unbind();
+    subsystem_mgr->unbind();    
+
+    subsystem_mgr->remove("aircraft-model");
+    subsystem_mgr->remove("tile-manager");
+    subsystem_mgr->remove("model-manager");
+    _tile_mgr.clear();
+
+    osg::ref_ptr<osgViewer::Viewer> vw(renderer->getViewer());
+    if (vw) {
+        // https://code.google.com/p/flightgear-bugs/issues/detail?id=1291
+        // explicitly stop trheading before we delete the renderer or
+        // viewMgr (which ultimately holds refs to the CameraGroup, and
+        // GraphicsContext)
+        vw->stopThreading();
+    }
+    
+    // don't cancel the pager until after shutdown, since AIModels (and
+    // potentially others) can queue delete requests on the pager.
+    if (vw && vw->getDatabasePager()) {
+        vw->getDatabasePager()->cancel();
+        vw->getDatabasePager()->clear();
+    }
+    
+    osgDB::Registry::instance()->clearObjectCache();
+    
+    // renderer touches subsystems during its destruction
+    set_renderer(NULL);
+    _scenery.clear();
+    _chatter_queue.clear();
+    
     delete subsystem_mgr;
-    
-    delete renderer;
-    renderer = NULL;
-    
+    subsystem_mgr = NULL; // important so ::get_subsystem returns NULL
+    vw = 0; // don't delete the viewer until now
+
     delete time_params;
-    delete matlib;
+    set_matlib(NULL);
     delete route_mgr;
-
     delete ATIS_mgr;
-
     delete channel_options_list;
     delete initial_waypoints;
-    delete scenery;
     delete fontcache;
-
     delete channellist;
-    delete sound;
 
+    simgear::PropertyObjectBase::setDefaultRoot(NULL);
+    simgear::SGModelLib::resetPropertyRoot();
+    
     delete locale;
     locale = NULL;
+    
+    cleanupListeners();
+    
+    props.clear();
+    
+    delete commands;
 }
 
 // set the fg_root path
-void FGGlobals::set_fg_root (const string &root) {
+void FGGlobals::set_fg_root (const std::string &root) {
     SGPath tmp(root);
     fg_root = tmp.realpath();
 
@@ -247,12 +288,60 @@ void FGGlobals::set_fg_root (const string &root) {
 }
 
 // set the fg_home path
-void FGGlobals::set_fg_home (const string &home) {
+void FGGlobals::set_fg_home (const std::string &home) {
     SGPath tmp(home);
     fg_home = tmp.realpath();
 }
 
-void FGGlobals::append_fg_scenery (const string &paths)
+PathList FGGlobals::get_data_paths() const
+{
+    PathList r(additional_data_paths);
+    r.push_back(SGPath(fg_root));
+    return r;
+}
+
+PathList FGGlobals::get_data_paths(const std::string& suffix) const
+{
+    PathList r;
+    BOOST_FOREACH(SGPath p, get_data_paths()) {
+        p.append(suffix);
+        if (p.exists()) {
+            r.push_back(p);
+        }
+    }
+
+    return r;
+}
+
+void FGGlobals::append_data_path(const SGPath& path)
+{
+    if (!path.exists()) {
+        SG_LOG(SG_GENERAL, SG_WARN, "adding non-existant data path:" << path);
+    }
+    
+    additional_data_paths.push_back(path);
+}
+
+SGPath FGGlobals::find_data_dir(const std::string& pathSuffix) const
+{
+    BOOST_FOREACH(SGPath p, additional_data_paths) {
+        p.append(pathSuffix);
+        if (p.exists()) {
+            return p;
+        }
+    }
+    
+    SGPath rootPath(fg_root);
+    rootPath.append(pathSuffix);
+    if (rootPath.exists()) {
+        return rootPath;
+    }
+    
+    SG_LOG(SG_GENERAL, SG_WARN, "dir not found in any data path:" << pathSuffix);
+    return SGPath();
+}
+
+void FGGlobals::append_fg_scenery (const std::string &paths)
 {
 //    fg_scenery.clear();
     SGPropertyNode* sim = fgGetNode("/sim", true);
@@ -311,11 +400,19 @@ void FGGlobals::append_aircraft_path(const std::string& path)
 {
   SGPath dirPath(path);
   if (!dirPath.exists()) {
-    SG_LOG(SG_GENERAL, SG_WARN, "aircraft path not found:" << path);
+    SG_LOG(SG_GENERAL, SG_ALERT, "aircraft path not found:" << path);
     return;
   }
+    
+  SGPath acSubdir(dirPath);
+  acSubdir.append("Aircraft");
+  if (acSubdir.exists()) {
+      SG_LOG(SG_GENERAL, SG_WARN, "Specified an aircraft-dir with an 'Aircraft' subdirectory:" << dirPath
+                 << ", will instead use child directory:" << acSubdir);
+      dirPath = acSubdir;
+  }
+    
   std::string abspath = dirPath.realpath();
-  
   unsigned int index = fg_aircraft_dirs.size();  
   fg_aircraft_dirs.push_back(abspath);
   
@@ -357,6 +454,16 @@ FGGlobals::get_renderer () const
    return renderer;
 }
 
+void FGGlobals::set_renderer(FGRenderer *render)
+{
+    if (render == renderer) {
+        return;
+    }
+    
+    delete renderer;
+    renderer = render;
+}
+
 SGSubsystemMgr *
 FGGlobals::get_subsystem_mgr () const
 {
@@ -366,6 +473,10 @@ FGGlobals::get_subsystem_mgr () const
 SGSubsystem *
 FGGlobals::get_subsystem (const char * name)
 {
+    if (!subsystem_mgr) {
+        return NULL;
+    }
+    
     return subsystem_mgr->get_subsystem(name);
 }
 
@@ -428,6 +539,70 @@ FGGlobals::get_view_position_cart() const
   return SGVec3d::fromGeod(get_view_position());
 }
 
+static void treeDumpRefCounts(int depth, SGPropertyNode* nd)
+{
+    for (int i=0; i<nd->nChildren(); ++i) {
+        SGPropertyNode* cp = nd->getChild(i);
+        if (SGReferenced::count(cp) > 1) {
+            SG_LOG(SG_GENERAL, SG_INFO, "\t" << cp->getPath() << " refcount:" << SGReferenced::count(cp));
+        }
+        
+        treeDumpRefCounts(depth + 1, cp);
+    }
+}
+
+static void treeClearAliases(SGPropertyNode* nd)
+{
+    if (nd->isAlias()) {
+        nd->unalias();
+    }
+    
+    for (int i=0; i<nd->nChildren(); ++i) {
+        SGPropertyNode* cp = nd->getChild(i);
+        treeClearAliases(cp);
+    }
+}
+
+void
+FGGlobals::resetPropertyRoot()
+{
+    delete locale;
+    
+    cleanupListeners();
+    
+    // we don't strictly need to clear these (they will be reset when we
+    // initProperties again), but trying to reduce false-positives when dumping
+    // ref-counts.
+    positionLon.clear();
+    positionLat.clear();
+    positionAlt.clear();
+    viewLon.clear();
+    viewLat.clear();
+    viewAlt.clear();
+    orientPitch.clear();
+    orientHeading.clear();
+    orientRoll.clear();
+    
+    // clear aliases so ref-counts are accurate when dumped
+    treeClearAliases(props);
+    
+    SG_LOG(SG_GENERAL, SG_INFO, "root props refcount:" << props.getNumRefs());
+    treeDumpRefCounts(0, props);
+
+    //BaseStackSnapshot::dumpAll(std::cout);
+    
+    props = new SGPropertyNode;
+    initProperties();
+    locale = new FGLocale(props);
+    
+    // remove /sim/fg-root before writing to prevent hijacking
+    SGPropertyNode *n = props->getNode("/sim", true);
+    n->removeChild("fg-root", 0, false);
+    n = n->getChild("fg-root", 0, true);
+    n->setStringValue(fg_root.c_str());
+    n->setAttribute(SGPropertyNode::WRITE, false);
+}
+
 // Save the current state as the initial state.
 void
 FGGlobals::saveInitialState ()
@@ -453,6 +628,17 @@ FGGlobals::saveInitialState ()
   }
 }
 
+static std::string autosaveName()
+{
+    std::ostringstream os;
+    string_list versionParts = simgear::strutils::split(VERSION, ".");
+    if (versionParts.size() < 2) {
+        return "autosave.xml";
+    }
+    
+    os << "autosave_" << versionParts[0] << "_" << versionParts[1] << ".xml";
+    return os.str();
+}
 
 // Restore the saved initial state, if any
 void
@@ -483,7 +669,7 @@ FGGlobals::loadUserSettings(const SGPath& dataPath)
     // remember that we have (tried) to load any existing autsave.xml
     haveUserSettings = true;
 
-    SGPath autosaveFile = simgear::Dir(dataPath).file("autosave.xml");
+    SGPath autosaveFile = simgear::Dir(dataPath).file(autosaveName());
     SGPropertyNode autosave;
     if (autosaveFile.exists()) {
       SG_LOG(SG_INPUT, SG_INFO, "Reading user settings from " << autosaveFile.str());
@@ -511,13 +697,13 @@ FGGlobals::saveUserSettings()
       haveUserSettings = false;
 
       SGPath autosaveFile(globals->get_fg_home());
-      autosaveFile.append( "autosave.xml" );
+      autosaveFile.append(autosaveName());
       autosaveFile.create_dir( 0700 );
       SG_LOG(SG_IO, SG_INFO, "Saving user settings to " << autosaveFile.str());
       try {
         writeProperties(autosaveFile.str(), globals->get_props(), false, SGPropertyNode::USERARCHIVE);
       } catch (const sg_exception &e) {
-        guiErrorMessage("Error writing autosave.xml: ", e);
+        guiErrorMessage("Error writing autosave:", e);
       }
       SG_LOG(SG_INPUT, SG_DEBUG, "Finished Saving user settings");
     }
@@ -549,4 +735,53 @@ void FGGlobals::set_warp_delta( long int d )
   fgSetInt("/sim/time/warp-delta", d);
 }
 
+FGScenery* FGGlobals::get_scenery () const
+{
+    return _scenery.get();
+}
+
+void FGGlobals::set_scenery ( FGScenery *s )
+{
+    _scenery = s;
+}
+
+FGTileMgr* FGGlobals::get_tile_mgr () const
+{
+    return _tile_mgr.get();
+}
+
+void FGGlobals::set_tile_mgr ( FGTileMgr *t )
+{
+    _tile_mgr = t;
+}
+
+void FGGlobals::set_matlib( SGMaterialLib *m )
+{
+    matlib = m;
+}
+
+FGSampleQueue* FGGlobals::get_chatter_queue() const
+{
+    return _chatter_queue;
+}
+
+void FGGlobals::set_chatter_queue(FGSampleQueue* queue)
+{
+    _chatter_queue = queue;
+}
+
+void FGGlobals::addListenerToCleanup(SGPropertyChangeListener* l)
+{
+    _listeners_to_cleanup.push_back(l);
+}
+
+void FGGlobals::cleanupListeners()
+{
+    SGPropertyChangeListenerVec::iterator i = _listeners_to_cleanup.begin();
+    for (; i != _listeners_to_cleanup.end(); ++i) {
+        delete *i;
+    }
+    _listeners_to_cleanup.clear();
+}
+
 // end of globals.cxx
