@@ -33,10 +33,19 @@
 #include <osg/GraphicsContext>
 #include <osgDB/Registry>
 
+#if defined(HAVE_CRASHRPT)
+	#include <CrashRpt.h>
+
+// defined in bootstrap.cxx
+extern bool global_crashRptEnabled;
+
+#endif
+
 // Class references
 #include <simgear/canvas/VGInitOperation.hxx>
 #include <simgear/scene/model/modellib.hxx>
 #include <simgear/scene/material/matlib.hxx>
+#include <simgear/scene/material/Effect.hxx>
 #include <simgear/props/AtomicChangeListener.hxx>
 #include <simgear/props/props.hxx>
 #include <simgear/timing/sg_time.hxx>
@@ -44,6 +53,7 @@
 #include <simgear/scene/tsync/terrasync.hxx>
 #include <simgear/math/SGMath.hxx>
 #include <simgear/math/sg_random.h>
+#include <simgear/misc/strutils.hxx>
 
 #include <Model/panelnode.hxx>
 #include <Scenery/scenery.hxx>
@@ -51,10 +61,12 @@
 #include <Sound/soundmanager.hxx>
 #include <Time/TimeManager.hxx>
 #include <GUI/gui.h>
+#include <GUI/MessageBox.hxx>
 #include <Viewer/splash.hxx>
 #include <Viewer/renderer.hxx>
 #include <Viewer/WindowSystemAdapter.hxx>
 #include <Navaids/NavDataCache.hxx>
+#include <Include/version.h>
 
 #include "fg_commands.hxx"
 #include "fg_io.hxx"
@@ -64,7 +76,10 @@
 #include "fg_os.hxx"
 #include "fg_props.hxx"
 #include "positioninit.hxx"
+#include "screensaver_control.hxx"
 #include "subsystemFactory.hxx"
+#include "options.hxx"
+
 
 using namespace flightgear;
 
@@ -84,9 +99,6 @@ static void fgMainLoop( void )
 {
     frame_signal->fireValueChanged();
 
-    SG_LOG( SG_GENERAL, SG_DEBUG, "Running Main Loop");
-    SG_LOG( SG_GENERAL, SG_DEBUG, "======= ==== ====");
-
     // compute simulated time (allowing for pause, warp, etc) and
     // real elapsed time
     double sim_dt, real_dt;
@@ -96,10 +108,73 @@ static void fgMainLoop( void )
     globals->get_subsystem_mgr()->update(sim_dt);
 
     simgear::AtomicChangeListener::fireChangeListeners();
-
-    SG_LOG( SG_GENERAL, SG_DEBUG, "" );
 }
 
+static void initTerrasync()
+{
+    // add the terrasync root as a data path so data can be retrieved from it
+    // (even if we are in read-only mode)
+    std::string terraSyncDir(fgGetString("/sim/terrasync/scenery-dir"));
+    globals->append_data_path(terraSyncDir);
+    
+    if (fgGetBool("/sim/fghome-readonly", false)) {
+        return;
+    }
+    
+    // start TerraSync up now, so it can be synchronizing shared models
+    // and airports data in parallel with a nav-cache rebuild.
+    SGPath tsyncCache(globals->get_fg_home());
+    tsyncCache.append("terrasync-cache.xml");
+    
+    // wipe the cache file if requested
+    if (flightgear::Options::sharedInstance()->isOptionSet("restore-defaults")) {
+        SG_LOG(SG_GENERAL, SG_INFO, "restore-defaults requested, wiping terrasync update cache at " <<
+               tsyncCache);
+        if (tsyncCache.exists()) {
+            tsyncCache.remove();
+        }
+    }
+    
+    fgSetString("/sim/terrasync/cache-path", tsyncCache.c_str());
+    
+    simgear::SGTerraSync* terra_sync = new simgear::SGTerraSync();
+    terra_sync->setRoot(globals->get_props());
+    globals->add_subsystem("terrasync", terra_sync);
+    
+    terra_sync->bind();
+    terra_sync->init();
+}
+
+static void checkOpenGLVersion()
+{
+#if defined(SG_MAC)
+    // Mac users can't upgrade their drivers, so complaining about
+    // versions doesn't help them much
+    return;
+#endif
+    
+    // format of these strings is not standardised, so be careful about
+    // parsing them.
+    std::string versionString(fgGetString("/sim/rendering/gl-version"));
+    string_list parts = simgear::strutils::split(versionString);
+    if (parts.size() == 3) {
+        if (parts[1].find("NVIDIA") != std::string::npos) {
+            // driver version number, dot-seperared
+            string_list driverVersion = simgear::strutils::split(parts[2], ".");
+            if (!driverVersion.empty()) {
+                int majorDriverVersion = simgear::strutils::to_int(driverVersion[0]);
+                if (majorDriverVersion < 300) {
+                    std::ostringstream ss;
+                    ss << "Please upgrade to at least version 300 of the nVidia drivers (installed version is " << parts[2] << ")";
+
+                    flightgear::modalMessageBox("Outdated graphics drivers",
+                                                "FlightGear has detected outdated drivers for your graphics card.",
+                                                ss.str());
+                }
+            }
+        } // of NVIDIA-style version string
+    } // of three parts
+}
 
 static void registerMainLoop()
 {
@@ -116,20 +191,23 @@ static void registerMainLoop()
 // per pass) and once everything has been initialized fgMainLoop from
 // then on.
 
+static int idle_state = 0;
+
 static void fgIdleFunction ( void ) {
     // Specify our current idle function state.  This is used to run all
     // our initializations out of the idle callback so that we can get a
     // splash screen up and running right away.
-    static int idle_state = 0;
-  
+    
     if ( idle_state == 0 ) {
         if (guiInit())
         {
+            checkOpenGLVersion();
             idle_state+=2;
             fgSplashProgress("loading-aircraft-list");
         }
 
     } else if ( idle_state == 2 ) {
+        initTerrasync();
         idle_state++;
         fgSplashProgress("loading-nav-dat");
 
@@ -144,21 +222,14 @@ static void fgIdleFunction ( void ) {
         }
       
     } else if ( idle_state == 4 ) {
-        idle_state+=2;
-        // based on the requested presets, calculate the true starting
-        // lon, lat
-        flightgear::initPosition();
-        flightgear::initTowerLocationListener();
+        idle_state++;
 
         TimeManager* t = new TimeManager;
         globals->add_subsystem("time", t, SGSubsystemMgr::INIT);
-        t->init(); // need to init now, not during initSubsystems
         
         // Do some quick general initializations
         if( !fgInitGeneral()) {
-            SG_LOG( SG_GENERAL, SG_ALERT,
-                "General initialization failed ..." );
-            exit(-1);
+            throw sg_exception("General initialization failed");
         }
 
         ////////////////////////////////////////////////////////////////////
@@ -172,30 +243,34 @@ static void fgIdleFunction ( void ) {
         // Initialize the material manager
         ////////////////////////////////////////////////////////////////////
         globals->set_matlib( new SGMaterialLib );
-        simgear::SGModelLib::init(globals->get_fg_root(), globals->get_props());
         simgear::SGModelLib::setPanelFunc(FGPanelNode::load);
-
+ 
+    } else if (( idle_state == 5 ) || (idle_state == 2005)) {
+        idle_state+=2;
+        flightgear::initPosition();
+        flightgear::initTowerLocationListener();
+        
+        simgear::SGModelLib::init(globals->get_fg_root(), globals->get_props());
+        
+        TimeManager* timeManager = (TimeManager*) globals->get_subsystem("time");
+        timeManager->init();
+        
         ////////////////////////////////////////////////////////////////////
         // Initialize the TG scenery subsystem.
         ////////////////////////////////////////////////////////////////////
-        simgear::SGTerraSync* terra_sync = new simgear::SGTerraSync(globals->get_props());
-        globals->add_subsystem("terrasync", terra_sync);
+        
         globals->set_scenery( new FGScenery );
         globals->get_scenery()->init();
         globals->get_scenery()->bind();
         globals->set_tile_mgr( new FGTileMgr );
-
-        fgSplashProgress("loading-aircraft");
-
-    } else if ( idle_state == 6 ) {
-        idle_state++;
+        
         fgSplashProgress("creating-subsystems");
-
-    } else if ( idle_state == 7 ) {
-        idle_state++;
+    } else if (( idle_state == 7 ) || (idle_state == 2007)) {
+        bool isReset = (idle_state == 2007);
+        idle_state = 8; // from the next state on, reset & startup are identical
         SGTimeStamp st;
         st.stamp();
-        fgCreateSubsystems();
+        fgCreateSubsystems(isReset);
         SG_LOG(SG_GENERAL, SG_INFO, "Creating subsystems took:" << st.elapsedMSec());
         fgSplashProgress("binding-subsystems");
       
@@ -243,8 +318,20 @@ static void fgIdleFunction ( void ) {
         fgSetBool("sim/sceneryloaded", false);
         registerMainLoop();
     }
+    
+    if ( idle_state == 2000 ) {
+        fgStartNewReset();
+        idle_state = 2005;
+    }
 }
 
+void fgResetIdleState()
+{
+    idle_state = 2000;
+    fgRegisterIdleHandler( &fgIdleFunction );
+}
+
+
 static void upper_case_property(const char *name)
 {
     using namespace simgear;
@@ -259,7 +346,9 @@ static void upper_case_property(const char *name)
         else
             assert(t == props::STRING);
     }
-    p->addChangeListener(new FGMakeUpperCase);
+    SGPropertyChangeListener* muc = new FGMakeUpperCase;
+    globals->addListenerToCleanup(muc);
+    p->addChangeListener(muc);
 }
 
 // see http://code.google.com/p/flightgear-bugs/issues/detail?id=385
@@ -273,13 +362,54 @@ static void ATIScreenSizeHack()
     globals->get_renderer()->addCamera(hackCam, false);
 }
 
+// Propose NVIDIA Optimus to use high-end GPU
+#if defined(SG_WINDOWS)
+extern "C" {
+    _declspec(dllexport) DWORD NvOptimusEnablement = 0x00000001;
+}
+#endif
+
+static void logToFile()
+{
+    SGPath logPath = globals->get_fg_home();
+    logPath.append("fgfs.log");
+    if (logPath.exists()) {
+        SGPath prevLogPath = globals->get_fg_home();
+        prevLogPath.append("fgfs_0.log");
+        logPath.rename(prevLogPath);
+        // bit strange, we need to restore the correct value of logPath now
+        logPath = globals->get_fg_home();
+        logPath.append("fgfs.log");
+    }
+    sglog().logToFile(logPath, SG_ALL, SG_INFO);
+
+#if defined(HAVE_CRASHRPT)
+	if (global_crashRptEnabled) {
+		crAddFile2(logPath.c_str(), NULL, "FlightGear Log File", CR_AF_MAKE_FILE_COPY);
+		SG_LOG( SG_GENERAL, SG_INFO, "CrashRpt enabled");
+	} else {
+		SG_LOG(SG_GENERAL, SG_WARN, "CrashRpt enabled at compile time but failed to install");
+	}
+#endif
+}
+
 // Main top level initialization
 int fgMainInit( int argc, char **argv ) {
 
     // set default log levels
     sglog().setLogLevels( SG_ALL, SG_ALERT );
 
-    string version;
+    globals = new FGGlobals;
+    if (!fgInitHome()) {
+        return EXIT_FAILURE;
+    }
+    
+    if (!fgGetBool("/sim/fghome-readonly")) {
+        // now home is initialised, we can log to a file inside it
+        logToFile();
+    }
+    
+    std::string version;
 #ifdef FLIGHTGEAR_VERSION
     version = FLIGHTGEAR_VERSION;
 #else
@@ -287,13 +417,15 @@ int fgMainInit( int argc, char **argv ) {
 #endif
     SG_LOG( SG_GENERAL, SG_INFO, "FlightGear:  Version "
             << version );
-    SG_LOG( SG_GENERAL, SG_INFO, "Built with " << SG_COMPILER_STR << std::endl );
+    SG_LOG( SG_GENERAL, SG_INFO, "Built with " << SG_COMPILER_STR);
+	SG_LOG( SG_GENERAL, SG_INFO, "Jenkins number/ID " << HUDSON_BUILD_NUMBER << ":"
+			<< HUDSON_BUILD_ID);
 
     // Allocate global data structures.  This needs to happen before
     // we parse command line options
 
-    globals = new FGGlobals;
-
+    
+    
     // seed the random number generator
     sg_srandom_time();
 
@@ -309,11 +441,27 @@ int fgMainInit( int argc, char **argv ) {
     // Load the configuration parameters.  (Command line options
     // override config file options.  Config file options override
     // defaults.)
-    if ( !fgInitConfig(argc, argv) ) {
-      SG_LOG( SG_GENERAL, SG_ALERT, "Config option parsing failed ..." );
-      exit(-1);
+    int configResult = fgInitConfig(argc, argv, false);
+    if (configResult == flightgear::FG_OPTIONS_ERROR) {
+        return EXIT_FAILURE;
+    } else if (configResult == flightgear::FG_OPTIONS_EXIT) {
+        return EXIT_SUCCESS;
     }
-
+    
+    configResult = fgInitAircraft(false);
+    if (configResult == flightgear::FG_OPTIONS_ERROR) {
+        return EXIT_FAILURE;
+    } else if (configResult == flightgear::FG_OPTIONS_EXIT) {
+        return EXIT_SUCCESS;
+    }
+    
+    configResult = flightgear::Options::sharedInstance()->processOptions();
+    if (configResult == flightgear::FG_OPTIONS_ERROR) {
+        return EXIT_FAILURE;
+    } else if (configResult == flightgear::FG_OPTIONS_EXIT) {
+        return EXIT_SUCCESS;
+    }
+    
     // Initialize the Window/Graphics environment.
     fgOSInit(&argc, argv);
     _bootstrap_OSInit++;
@@ -325,18 +473,28 @@ int fgMainInit( int argc, char **argv ) {
 
     // Clouds3D requires an alpha channel
     fgOSOpenWindow(true /* request stencil buffer */);
-
+    fgOSResetProperties();
+    
     // Initialize the splash screen right away
     fntInit();
     fgSplashInit();
 
-    if (fgGetBool("/sim/ati-viewport-hack", false)) {
+    if (fgGetBool("/sim/ati-viewport-hack", true)) {
         SG_LOG(SG_GENERAL, SG_ALERT, "Enabling ATI viewport hack");
         ATIScreenSizeHack();
     }
     
+    fgOutputSettings();
+    
+    //try to disable the screensaver
+    fgOSDisableScreensaver();
+    
     // pass control off to the master event handler
     int result = fgOSMainLoop();
+    frame_signal.clear();
+    fgOSCloseWindow();
+    
+    simgear::clearEffectCache();
     
     // clean up here; ensure we null globals to avoid
     // confusing the atexit() handler
