@@ -173,6 +173,11 @@ static const IdPropertyList sIdPropertyList[] = {
 
   {1400, "scenery/events", simgear::props::STRING},
 
+  {1500, "instrumentation/transponder/transmitted-id", simgear::props::INT},
+  {1501, "instrumentation/transponder/altitude", simgear::props::INT},
+  {1502, "instrumentation/transponder/ident", simgear::props::BOOL},
+  {1503, "instrumentation/transponder/inputs/mode", simgear::props::INT},
+
   {10001, "sim/multiplay/transmission-freq-hz",  simgear::props::STRING},
   {10002, "sim/multiplay/chat",  simgear::props::STRING},
 
@@ -424,6 +429,8 @@ FGMultiplayMgr::init (void)
   mTimeUntilSend = 0.0;
   
   mCallsign = fgGetString("/sim/multiplay/callsign");
+  fgGetNode("/sim/multiplay/callsign", true)->setAttribute(SGPropertyNode::PRESERVE, true);
+    
   if ((!txAddress.empty()) && (txAddress!="0")) {
     mServer.set(txAddress.c_str(), txPort);
     if (strncmp (mServer.getHost(), "0.0.0.0", 8) == 0) {
