@@ -37,7 +37,7 @@
 #include <simgear/bucket/newbucket.hxx>
 
 #include <Airports/runways.hxx>
-#include <Airports/simple.hxx>
+#include <Airports/airport.hxx>
 #include <Airports/dynamics.hxx>
 #include <Airports/parking.hxx>
 #include <Scripting/NasalSys.hxx>
@@ -73,6 +73,7 @@ naGhostType NavaidGhostType = { positionedGhostDestroy, "navaid", navaidGhostGet
 
 static const char* runwayGhostGetMember(naContext c, void* g, naRef field, naRef* out);
 naGhostType RunwayGhostType = { positionedGhostDestroy, "runway", runwayGhostGetMember, 0 };
+naGhostType HelipadGhostType = { positionedGhostDestroy, "helipad", runwayGhostGetMember, 0 };
 naGhostType TaxiwayGhostType = { positionedGhostDestroy, "taxiway", runwayGhostGetMember, 0 };
 
 static const char* fixGhostGetMember(naContext c, void* g, naRef field, naRef* out);
@@ -123,6 +124,17 @@ static naRef stringToNasal(naContext c, const std::string& s)
                    s.length());
 }
 
+static bool convertToNum(naRef v, double& result)
+{
+    naRef n = naNumValue(v);
+    if (naIsNil(n)) {
+        return false; // couldn't convert
+    }
+    
+    result = n.num;
+    return true;
+}
+
 static WayptFlag wayptFlagFromString(const char* s)
 {
   if (!strcmp(s, "sid")) return WPT_DEPARTURE;
@@ -289,6 +301,16 @@ naRef ghostForRunway(naContext c, const FGRunway* r)
   return naNewGhost2(c, &RunwayGhostType, (void*) r);
 }
 
+naRef ghostForHelipad(naContext c, const FGHelipad* r)
+{
+  if (!r) {
+    return naNil();
+  }
+
+  FGPositioned::get(r); // take a ref
+  return naNewGhost2(c, &HelipadGhostType, (void*) r);
+}
+
 naRef ghostForTaxiway(naContext c, const FGTaxiway* r)
 {
   if (!r) {
@@ -368,16 +390,24 @@ static const char* airportGhostGetMember(naContext c, void* g, naRef field, naRe
     double minLengthFt = fgGetDouble("/sim/navdb/min-runway-length-ft");
     for(unsigned int r=0; r<apt->numRunways(); ++r) {
       FGRunway* rwy(apt->getRunwayByIndex(r));
-      
-    // ignore unusably short runways
+      // ignore unusably short runways
       if (rwy->lengthFt() < minLengthFt) {
         continue;
       }
-      
       naRef rwyid = stringToNasal(c, rwy->ident());
       naRef rwydata = ghostForRunway(c, rwy);
       naHash_set(*out, rwyid, rwydata);
     }
+  } else if (!strcmp(fieldName, "helipads")) {
+    *out = naNewHash(c);
+
+    for(unsigned int r=0; r<apt->numHelipads(); ++r) {
+      FGHelipad* hp(apt->getHelipadByIndex(r));
+
+      naRef rwyid = stringToNasal(c, hp->ident());
+      naRef rwydata = ghostForHelipad(c, hp);
+      naHash_set(*out, rwyid, rwydata);
+    }
 
   } else if (!strcmp(fieldName, "taxiways")) {
     *out = naNewVector(c);
@@ -396,11 +426,11 @@ static const char* airportGhostGetMember(naContext c, void* g, naRef field, naRe
 
 static const char* waypointCommonGetMember(naContext c, Waypt* wpt, const char* fieldName, naRef* out)
 {
-  if (!strcmp(fieldName, "wp_name")) *out = stringToNasal(c, wpt->ident());
+  if (!strcmp(fieldName, "wp_name") || !strcmp(fieldName, "id")) *out = stringToNasal(c, wpt->ident());
   else if (!strcmp(fieldName, "wp_type")) *out = stringToNasal(c, wpt->type());
   else if (!strcmp(fieldName, "wp_role")) *out = wayptFlagToNasal(c, wpt->flags());
-  else if (!strcmp(fieldName, "wp_lat")) *out = naNum(wpt->position().getLatitudeDeg());
-  else if (!strcmp(fieldName, "wp_lon")) *out = naNum(wpt->position().getLongitudeDeg());
+  else if (!strcmp(fieldName, "wp_lat") || !strcmp(fieldName, "lat")) *out = naNum(wpt->position().getLatitudeDeg());
+  else if (!strcmp(fieldName, "wp_lon") || !strcmp(fieldName, "lon")) *out = naNum(wpt->position().getLongitudeDeg());
   else if (!strcmp(fieldName, "wp_parent_name")) {
     Procedure* proc = dynamic_cast<Procedure*>(wpt->owner());
     *out = proc ? stringToNasal(c, proc->ident()) : naNil();
@@ -443,7 +473,7 @@ static const char* wayptGhostGetMember(naContext c, void* g, naRef field, naRef*
 
 static RouteRestriction routeRestrictionFromString(const char* s)
 {
-  string u(s);
+  std::string u(s);
   boost::to_lower(u);
   if (u == "computed") return RESTRICT_COMPUTED;
   if (u == "at") return RESTRICT_AT;
@@ -478,6 +508,7 @@ static const char* legGhostGetMember(naContext c, void* g, naRef field, naRef* o
   if (!strcmp(fieldName, "parents")) {
     *out = naNewVector(c);
     naVec_append(*out, fpLegPrototype);
+    naVec_append(*out, waypointPrototype);
   } else if (!strcmp(fieldName, "index")) {
     *out = naNum(leg->index());
   } else if (!strcmp(fieldName, "alt_cstr")) {
@@ -570,6 +601,11 @@ static void flightplanGhostSetMember(naContext c, void* g, naRef field, naRef va
       return;
     }
     
+    if (naIsNil(value)) {
+      fp->setDeparture(static_cast<FGAirport*>(NULL));
+      return;
+    }
+      
     naRuntimeError(c, "bad argument type setting departure");
   } else if (!strcmp(fieldName, "destination")) {
     FGAirport* apt = airportGhost(value);
@@ -592,7 +628,7 @@ static void flightplanGhostSetMember(naContext c, void* g, naRef field, naRef va
       return;
     }
     
-    naRuntimeError(c, "bad argument type setting departure");
+    naRuntimeError(c, "bad argument type setting departure runway");
   } else if (!strcmp(fieldName, "destination_runway")) {
     FGRunway* rwy = runwayGhost(value);
     if (rwy){
@@ -600,7 +636,7 @@ static void flightplanGhostSetMember(naContext c, void* g, naRef field, naRef va
       return;
     }
     
-    naRuntimeError(c, "bad argument type setting departure");
+    naRuntimeError(c, "bad argument type setting destination runway");
   } else if (!strcmp(fieldName, "sid")) {
     Procedure* proc = procedureGhost(value);
     if (proc && (proc->type() == PROCEDURE_SID)) {
@@ -709,7 +745,7 @@ static const char* procedureGhostGetMember(naContext c, void* g, naRef field, na
         
     ArrivalDeparture* ad = static_cast<ArrivalDeparture*>(proc);
     *out = naNewVector(c);
-    BOOST_FOREACH(string id, ad->transitionIdents()) {
+    BOOST_FOREACH(std::string id, ad->transitionIdents()) {
       naVec_append(*out, stringToNasal(c, id));
     }
   } else {
@@ -735,7 +771,9 @@ static const char* runwayGhostGetMember(naContext c, void* g, naRef field, naRef
     FGRunway* rwy = (FGRunway*) g;
     if (!strcmp(fieldName, "threshold")) *out = naNum(rwy->displacedThresholdM());
     else if (!strcmp(fieldName, "stopway")) *out = naNum(rwy->stopwayM());
-    else if (!strcmp(fieldName, "ils_frequency_mhz")) {
+    else if (!strcmp(fieldName, "reciprocal")) {
+      *out = ghostForRunway(c, rwy->reciprocalRunway());
+    } else if (!strcmp(fieldName, "ils_frequency_mhz")) {
       *out = rwy->ILS() ? naNum(rwy->ILS()->get_freq() / 100.0) : naNil();
     } else if (!strcmp(fieldName, "ils")) {
       *out = ghostForNavaid(c, rwy->ILS());
@@ -962,49 +1000,6 @@ static naRef f_geodinfo(naContext c, naRef me, int argc, naRef* args)
 }
 
 
-class AirportInfoFilter : public FGAirport::AirportFilter
-{
-public:
-  AirportInfoFilter() : type(FGPositioned::AIRPORT) {
-    minRunwayLengthFt = fgGetDouble("/sim/navdb/min-runway-length-ft", 0.0);
-  }
-  
-  bool fromArg(naRef arg)
-  {
-    const char *s = naStr_data(arg);
-    if(!strcmp(s, "airport")) type = FGPositioned::AIRPORT;
-    else if(!strcmp(s, "seaport")) type = FGPositioned::SEAPORT;
-    else if(!strcmp(s, "heliport")) type = FGPositioned::HELIPORT;
-    else
-      return false;
-    
-    return true;
-  }
-  
-  virtual FGPositioned::Type minType() const {
-    return type;
-  }
-  
-  virtual FGPositioned::Type maxType() const {
-    return type;
-  }
-    
-  virtual bool pass(FGPositioned* aPos) const
-  {
-    FGAirport* apt = (FGAirport*) aPos;
-    if ((apt->type() == FGPositioned::AIRPORT) && 
-        !apt->hasHardRunwayOfLengthFt(minRunwayLengthFt)) 
-    {
-      return false;
-    }
-
-    return true;
-  }
-  
-  FGPositioned::Type type;
-  double minRunwayLengthFt;
-};
-
 // Returns data hash for particular or nearest airport of a <type>, or nil
 // on error.
 //
@@ -1025,12 +1020,12 @@ static naRef f_airportinfo(naContext c, naRef me, int argc, naRef* args)
   
   double maxRange = 10000.0; // expose this? or pick a smaller value?
   
-  AirportInfoFilter filter; // defaults to airports only
+  FGAirport::TypeRunwayFilter filter; // defaults to airports only
   
   if(argc == 0) {
     // fall through and use AIRPORT
   } else if(argc == 1 && naIsString(args[0])) {
-    if (filter.fromArg(args[0])) {
+    if (filter.fromTypeString(naStr_data(args[0]))) {
       // done!
     } else {
       // user provided an <id>, hopefully
@@ -1065,15 +1060,15 @@ static naRef f_findAirportsWithinRange(naContext c, naRef me, int argc, naRef* a
     naRuntimeError(c, "findAirportsWithinRange expected range (in nm) as arg %d", argOffset);
   }
   
-  AirportInfoFilter filter; // defaults to airports only
+  FGAirport::TypeRunwayFilter filter; // defaults to airports only
   double rangeNm = args[argOffset++].num;
   if (argOffset < argc) {
-    filter.fromArg(args[argOffset++]);
+    filter.fromTypeString(naStr_data(args[argOffset++]));
   }
   
   naRef r = naNewVector(c);
   
-  FGPositioned::List apts = FGPositioned::findWithinRange(pos, rangeNm, &filter);
+  FGPositionedList apts = FGPositioned::findWithinRange(pos, rangeNm, &filter);
   FGPositioned::sortByRange(apts, pos);
   
   BOOST_FOREACH(FGPositionedRef a, apts) {
@@ -1091,15 +1086,15 @@ static naRef f_findAirportsByICAO(naContext c, naRef me, int argc, naRef* args)
   }
   
   int argOffset = 0;
-  string prefix(naStr_data(args[argOffset++]));
-  AirportInfoFilter filter; // defaults to airports only
+  std::string prefix(naStr_data(args[argOffset++]));
+  FGAirport::TypeRunwayFilter filter; // defaults to airports only
   if (argOffset < argc) {
-    filter.fromArg(args[argOffset++]);
+    filter.fromTypeString(naStr_data(args[argOffset++]));
   }
   
   naRef r = naNewVector(c);
   
-  FGPositioned::List apts = FGPositioned::findAllWithIdent(prefix, &filter, false);
+  FGPositionedList apts = FGPositioned::findAllWithIdent(prefix, &filter, false);
   
   BOOST_FOREACH(FGPositionedRef a, apts) {
     FGAirport* apt = (FGAirport*) a.get();
@@ -1134,7 +1129,11 @@ static naRef f_airport_comms(naContext c, naRef me, int argc, naRef* args)
     naRef comms = naNewVector(c);
     
 // if we have an explicit type, return a simple vector of frequencies
-    if (argc > 0 && naIsScalar(args[0])) {
+    if (argc > 0 && !naIsString(args[0])) {
+        naRuntimeError(c, "airport.comms argument must be a frequency type name");
+    }
+    
+    if (argc > 0) {
         std::string commName = naStr_data(args[0]);
         FGPositioned::Type commType = FGPositioned::typeFromName(commName);
         
@@ -1160,38 +1159,36 @@ static naRef f_airport_runway(naContext c, naRef me, int argc, naRef* args)
   if (!apt) {
     naRuntimeError(c, "airport.runway called on non-airport object");
   }
-  
+
   if ((argc < 1) || !naIsString(args[0])) {
     naRuntimeError(c, "airport.runway expects a runway ident argument");
   }
-  
+
   std::string ident(naStr_data(args[0]));
   boost::to_upper(ident);
-  if (!apt->hasRunwayWithIdent(ident)) {
-    return naNil();
+
+  if (apt->hasRunwayWithIdent(ident)) {
+    return ghostForRunway(c, apt->getRunwayByIdent(ident));
+  } else if (apt->hasHelipadWithIdent(ident)) {
+    return ghostForHelipad(c, apt->getHelipadByIdent(ident));
   }
-  
-  return ghostForRunway(c, apt->getRunwayByIdent(ident));
+  return naNil();
 }
 
-static naRef f_airport_taxiway(naContext c, naRef me, int argc, naRef* args)
+static naRef f_airport_runwaysWithoutReciprocals(naContext c, naRef me, int argc, naRef* args)
 {
   FGAirport* apt = airportGhost(me);
   if (!apt) {
-    naRuntimeError(c, "airport.taxiway called on non-airport object");
+    naRuntimeError(c, "airport.runwaysWithoutReciprocals called on non-airport object");
   }
-  
-  if ((argc < 1) || !naIsString(args[0])) {
-    naRuntimeError(c, "airport.taxiway expects a taxiway ident argument");
+
+  FGRunwayList rwylist(apt->getRunwaysWithoutReciprocals());
+  naRef runways = naNewVector(c);
+  for (unsigned int r=0; r<rwylist.size(); ++r) {
+    FGRunway* rwy(rwylist[r]);
+    naVec_append(runways, ghostForRunway(c, apt->getRunwayByIdent(rwy->ident())));
   }
-  
-  naRef taxiways = naNewVector(c);
-  
-  for (unsigned int i = 0; i < apt->numTaxiways(); i++) {
-    naVec_append(taxiways, ghostForTaxiway(c, apt->getTaxiwayByIndex(i)));
-  }
-  
-  return taxiways;  
+  return runways;
 }
 
 static naRef f_airport_sids(naContext c, naRef me, int argc, naRef* args)
@@ -1378,7 +1375,7 @@ static naRef f_airport_getSid(naContext c, naRef me, int argc, naRef* args)
     naRuntimeError(c, "airport.getSid passed invalid argument");
   }
   
-  string ident = naStr_data(args[0]);
+  std::string ident = naStr_data(args[0]);
   return ghostForProcedure(c, apt->findSIDWithIdent(ident));
 }
 
@@ -1393,7 +1390,7 @@ static naRef f_airport_getStar(naContext c, naRef me, int argc, naRef* args)
     naRuntimeError(c, "airport.getStar passed invalid argument");
   }
   
-  string ident = naStr_data(args[0]);
+  std::string ident = naStr_data(args[0]);
   return ghostForProcedure(c, apt->findSTARWithIdent(ident));
 }
 
@@ -1408,10 +1405,25 @@ static naRef f_airport_getApproach(naContext c, naRef me, int argc, naRef* args)
     naRuntimeError(c, "airport.getIAP passed invalid argument");
   }
   
-  string ident = naStr_data(args[0]);
+  std::string ident = naStr_data(args[0]);
   return ghostForProcedure(c, apt->findApproachWithIdent(ident));
 }
 
+static naRef f_airport_findBestRunway(naContext c, naRef me, int argc, naRef* args)
+{
+    FGAirport* apt = airportGhost(me);
+    if (!apt) {
+        naRuntimeError(c, "findBestRunway called on non-airport object");
+    }
+    
+    SGGeod pos;
+    if (!geodFromArgs(args, 0, argc, pos)) {
+        naRuntimeError(c, "findBestRunway must be passed a position");
+    }
+    
+    return ghostForRunway(c,  apt->findBestRunwayForPos(pos));
+}
+
 static naRef f_airport_toString(naContext c, naRef me, int argc, naRef* args)
 {
   FGAirport* apt = airportGhost(me);
@@ -1507,7 +1519,7 @@ static naRef f_findNavaidsWithinRange(naContext c, naRef me, int argc, naRef* ar
   
   naRef r = naNewVector(c);
   FGNavList::TypeFilter filter(type);
-  FGPositioned::List navs = FGPositioned::findWithinRange(pos, rangeNm, &filter);
+  FGPositionedList navs = FGPositioned::findWithinRange(pos, rangeNm, &filter);
   FGPositioned::sortByRange(navs, pos);
   
   BOOST_FOREACH(FGPositionedRef a, navs) {
@@ -1582,7 +1594,7 @@ static naRef f_findNavaidsByIdent(naContext c, naRef me, int argc, naRef* args)
   }
   
   FGPositioned::Type type = FGPositioned::INVALID;
-  string ident = naStr_data(args[argOffset++]);
+  std::string ident = naStr_data(args[argOffset++]);
   if (argOffset < argc) {
     type = FGPositioned::typeFromName(naStr_data(args[argOffset]));
   }
@@ -1608,11 +1620,11 @@ static naRef f_findFixesByIdent(naContext c, naRef me, int argc, naRef* args)
     naRuntimeError(c, "findFixesByIdent expectes ident string as arg %d", argOffset);
   }
   
-  string ident(naStr_data(args[argOffset]));
+  std::string ident(naStr_data(args[argOffset]));
   naRef r = naNewVector(c);
   
   FGPositioned::TypeFilter filter(FGPositioned::FIX);
-  FGPositioned::List fixes = FGPositioned::findAllWithIdent(ident, &filter);
+  FGPositionedList fixes = FGPositioned::findAllWithIdent(ident, &filter);
   FGPositioned::sortByRange(fixes, pos);
   
   BOOST_FOREACH(FGPositionedRef f, fixes) {
@@ -1744,13 +1756,11 @@ public:
     _plan(fp),
     _instance(ins)
   {
-    SG_LOG(SG_NASAL, SG_INFO, "created Nasal delegate for " << fp);
     _gcSaveKey = _nasal->gcSave(ins);
   }
   
   virtual ~NasalFPDelegate()
   {
-    SG_LOG(SG_NASAL, SG_INFO, "destroying Nasal delegate for " << _plan);
     _nasal->gcRelease(_gcSaveKey);
   }
   
@@ -1778,19 +1788,25 @@ public:
   {
     callDelegateMethod("cleared");
   }
+    
+  virtual void endOfFlightPlan()
+  {
+    callDelegateMethod("endOfFlightPlan");
+  }
 private:
   
   void callDelegateMethod(const char* method)
   {
     naRef f;
-    naMember_cget(_nasal->context(), _instance, method, &f);
-    if (naIsNil(f)) {
-      return; // no method on the delegate
+    naContext ctx = naNewContext();
+      
+    if (naMember_cget(ctx, _instance, method, &f) != 0) {
+        naRef arg[1];
+        arg[0] = ghostForFlightPlan(ctx, _plan);
+        _nasal->callMethod(f, _instance, 1, arg, naNil());
     }
     
-    naRef arg[1];
-    arg[0] = ghostForFlightPlan(_nasal->context(), _plan);
-    _nasal->callMethod(f, _instance, 1, arg, naNil());
+    naFreeContext(ctx);
   }
   
   FGNasalSys* _nasal;
@@ -1799,6 +1815,7 @@ private:
   int _gcSaveKey;
 };
 
+
 class NasalFPDelegateFactory : public FlightPlan::DelegateFactory
 {
 public:
@@ -1809,7 +1826,7 @@ public:
     _gcSaveKey = _nasal->gcSave(_func);
   }
   
-  ~NasalFPDelegateFactory()
+  virtual ~NasalFPDelegateFactory()
   {
     _nasal->gcRelease(_gcSaveKey);
   }
@@ -1817,13 +1834,18 @@ public:
   virtual FlightPlan::Delegate* createFlightPlanDelegate(FlightPlan* fp)
   {
     naRef args[1];
-    args[0] = ghostForFlightPlan(_nasal->context(), fp);
+    naContext ctx = naNewContext();
+    args[0] = ghostForFlightPlan(ctx, fp);
     naRef instance = _nasal->call(_func, 1, args, naNil());
-    if (naIsNil(instance)) {
-      return NULL;
-    }
     
-    return new NasalFPDelegate(fp, _nasal, instance);
+      FlightPlan::Delegate* result = NULL;
+      if (!naIsNil(instance)) {
+          // will GC-save instance
+          result = new NasalFPDelegate(fp, _nasal, instance);
+      }
+      
+      naFreeContext(ctx);
+      return result;
   }
 private:
   FGNasalSys* _nasal;
@@ -1831,6 +1853,20 @@ private:
   int _gcSaveKey;
 };
 
+static std::vector<NasalFPDelegateFactory*> static_nasalDelegateFactories;
+
+void shutdownNasalPositioned()
+{
+    std::vector<NasalFPDelegateFactory*>::iterator it;
+    for (it = static_nasalDelegateFactories.begin();
+         it != static_nasalDelegateFactories.end(); ++it)
+    {
+        FlightPlan::unregisterDelegateFactory(*it);
+        delete (*it);
+    }
+    static_nasalDelegateFactories.clear();
+}
+
 static naRef f_registerFPDelegate(naContext c, naRef me, int argc, naRef* args)
 {
   if ((argc < 1) || !naIsFunc(args[0])) {
@@ -1839,7 +1875,7 @@ static naRef f_registerFPDelegate(naContext c, naRef me, int argc, naRef* args)
   
   NasalFPDelegateFactory* factory = new NasalFPDelegateFactory(args[0]);
   FlightPlan::registerDelegateFactory(factory);
-  
+    static_nasalDelegateFactories.push_back(factory);
   return naNil();
 }
 
@@ -1915,12 +1951,16 @@ static naRef f_createWP(naContext c, naRef me, int argc, naRef* args)
     naRuntimeError(c, "createWP: no identifier supplied");
   }
     
-  string ident = naStr_data(args[argOffset++]);
+  std::string ident = naStr_data(args[argOffset++]);
   WayptRef wpt = new BasicWaypt(pos, ident, NULL);
   
 // set waypt flags - approach, departure, pseudo, etc
   if (argc > argOffset) {
     WayptFlag f = wayptFlagFromString(naStr_data(args[argOffset++]));
+    if (f == 0) {
+      naRuntimeError(c, "createWP: bad waypoint role");
+    }
+      
     wpt->setFlag(f);
   }
   
@@ -1948,6 +1988,9 @@ static naRef f_createWPFrom(naContext c, naRef me, int argc, naRef* args)
   // set waypt flags - approach, departure, pseudo, etc
   if (argc > 1) {
     WayptFlag f = wayptFlagFromString(naStr_data(args[1]));
+    if (f == 0) {
+      naRuntimeError(c, "createWPFrom: bad waypoint role");
+    }
     wpt->setFlag(f);
   }
   
@@ -2117,6 +2160,10 @@ static naRef f_flightplan_clearWPType(naContext c, naRef me, int argc, naRef* ar
   }
   
   WayptFlag flag = wayptFlagFromString(naStr_data(args[0]));
+  if (flag == 0) {
+    naRuntimeError(c, "clearWPType: bad waypoint role");
+  }
+    
   fp->clearWayptsWithFlag(flag);
   return naNil();
 }
@@ -2155,6 +2202,16 @@ static naRef f_flightplan_pathGeod(naContext c, naRef me, int argc, naRef* args)
   return result;
 }
 
+static naRef f_flightplan_finish(naContext c, naRef me, int argc, naRef* args)
+{
+    FlightPlan* fp = flightplanGhost(me);
+    if (!fp) {
+        naRuntimeError(c, "flightplan.finish called on non-flightplan object");
+    }
+    
+    fp->finish();
+    return naNil();
+}
 
 static naRef f_leg_setSpeed(naContext c, naRef me, int argc, naRef* args)
 {
@@ -2162,13 +2219,14 @@ static naRef f_leg_setSpeed(naContext c, naRef me, int argc, naRef* args)
   if (!leg) {
     naRuntimeError(c, "leg.setSpeed called on non-flightplan-leg object");
   }
-  
-  if (argc < 2) {
-    naRuntimeError(c, "bad arguments to leg.setSpeed");
-  }
-  
+
+    double speed = 0.0;
+    if ((argc < 2) || !convertToNum(args[0], speed))
+        naRuntimeError(c, "bad arguments to setSpeed");
+    
   RouteRestriction rr = routeRestrictionFromString(naStr_data(args[1]));
-  leg->setSpeed(rr, args[0].num);
+  leg->setSpeed(rr, speed);
+  
   return naNil();
 }
 
@@ -2179,12 +2237,13 @@ static naRef f_leg_setAltitude(naContext c, naRef me, int argc, naRef* args)
     naRuntimeError(c, "leg.setAltitude called on non-flightplan-leg object");
   }
   
-  if (argc < 2) {
-    naRuntimeError(c, "bad arguments to leg.setAltitude");
-  }
-  
+    double alt = 0.0;
+    if ((argc < 2) || !convertToNum(args[0], alt))
+        naRuntimeError(c, "bad arguments to leg.setAltitude");
+    
   RouteRestriction rr = routeRestrictionFromString(naStr_data(args[1]));
-  leg->setAltitude(rr, args[0].num);
+    leg->setAltitude(rr, alt);
+
   return naNil();
 }
 
@@ -2383,13 +2442,14 @@ static struct { const char* name; naCFunction func; } funcs[] = {
 };
 
 
-naRef initNasalPositioned(naRef globals, naContext c, naRef gcSave)
+naRef initNasalPositioned(naRef globals, naContext c)
 {
     airportPrototype = naNewHash(c);
-    hashset(c, gcSave, "airportProto", airportPrototype);
+    naSave(c, airportPrototype);
   
     hashset(c, airportPrototype, "runway", naNewFunc(c, naNewCCode(c, f_airport_runway)));
-    hashset(c, airportPrototype, "taxiway", naNewFunc(c, naNewCCode(c, f_airport_taxiway)));
+    hashset(c, airportPrototype, "runwaysWithoutReciprocals", naNewFunc(c, naNewCCode(c, f_airport_runwaysWithoutReciprocals)));
+    hashset(c, airportPrototype, "helipad", naNewFunc(c, naNewCCode(c, f_airport_runway)));
     hashset(c, airportPrototype, "tower", naNewFunc(c, naNewCCode(c, f_airport_tower)));
     hashset(c, airportPrototype, "comms", naNewFunc(c, naNewCCode(c, f_airport_comms)));
     hashset(c, airportPrototype, "sids", naNewFunc(c, naNewCCode(c, f_airport_sids)));
@@ -2399,10 +2459,11 @@ naRef initNasalPositioned(naRef globals, naContext c, naRef gcSave)
     hashset(c, airportPrototype, "getSid", naNewFunc(c, naNewCCode(c, f_airport_getSid)));
     hashset(c, airportPrototype, "getStar", naNewFunc(c, naNewCCode(c, f_airport_getStar)));
     hashset(c, airportPrototype, "getIAP", naNewFunc(c, naNewCCode(c, f_airport_getApproach)));
+    hashset(c, airportPrototype, "findBestRunwayForPos", naNewFunc(c, naNewCCode(c, f_airport_findBestRunway)));
     hashset(c, airportPrototype, "tostring", naNewFunc(c, naNewCCode(c, f_airport_toString)));
   
     flightplanPrototype = naNewHash(c);
-    hashset(c, gcSave, "flightplanProto", flightplanPrototype);
+    naSave(c, flightplanPrototype);
       
     hashset(c, flightplanPrototype, "getWP", naNewFunc(c, naNewCCode(c, f_flightplan_getWP)));
     hashset(c, flightplanPrototype, "currentWP", naNewFunc(c, naNewCCode(c, f_flightplan_currentWP))); 
@@ -2417,21 +2478,22 @@ naRef initNasalPositioned(naRef globals, naContext c, naRef gcSave)
     hashset(c, flightplanPrototype, "clearWPType", naNewFunc(c, naNewCCode(c, f_flightplan_clearWPType))); 
     hashset(c, flightplanPrototype, "clone", naNewFunc(c, naNewCCode(c, f_flightplan_clone))); 
     hashset(c, flightplanPrototype, "pathGeod", naNewFunc(c, naNewCCode(c, f_flightplan_pathGeod)));
+    hashset(c, flightplanPrototype, "finish", naNewFunc(c, naNewCCode(c, f_flightplan_finish)));
     
     waypointPrototype = naNewHash(c);
-    hashset(c, gcSave, "wayptProto", waypointPrototype);
+    naSave(c, waypointPrototype);
     
     hashset(c, waypointPrototype, "navaid", naNewFunc(c, naNewCCode(c, f_waypoint_navaid)));
     hashset(c, waypointPrototype, "runway", naNewFunc(c, naNewCCode(c, f_waypoint_runway)));
     hashset(c, waypointPrototype, "airport", naNewFunc(c, naNewCCode(c, f_waypoint_airport)));
   
     procedurePrototype = naNewHash(c);
-    hashset(c, gcSave, "procedureProto", procedurePrototype);
+    naSave(c, procedurePrototype);
     hashset(c, procedurePrototype, "transition", naNewFunc(c, naNewCCode(c, f_procedure_transition)));
     hashset(c, procedurePrototype, "route", naNewFunc(c, naNewCCode(c, f_procedure_route)));
   
     fpLegPrototype = naNewHash(c);
-    hashset(c, gcSave, "fpLegProto", fpLegPrototype);
+    naSave(c, fpLegPrototype);
     hashset(c, fpLegPrototype, "setSpeed", naNewFunc(c, naNewCCode(c, f_leg_setSpeed)));
     hashset(c, fpLegPrototype, "setAltitude", naNewFunc(c, naNewCCode(c, f_leg_setAltitude)));
     hashset(c, fpLegPrototype, "path", naNewFunc(c, naNewCCode(c, f_leg_path)));
