@@ -3,6 +3,10 @@
 #  include "config.h"
 #endif
 
+#ifdef HAVE_WINDOWS_H
+#include <windows.h>
+#endif
+
 #ifdef HAVE_SYS_TIME_H
 #  include <sys/time.h>  // gettimeofday
 #endif
@@ -19,23 +23,36 @@
 #include <simgear/math/sg_random.h>
 #include <simgear/misc/sg_path.hxx>
 #include <simgear/misc/sg_dir.hxx>
-#include <simgear/misc/interpolator.hxx>
 #include <simgear/structure/commands.hxx>
 #include <simgear/math/sg_geodesy.hxx>
 #include <simgear/structure/event_mgr.hxx>
+#include <simgear/debug/BufferedLogCallback.hxx>
 
+#include <simgear/nasal/cppbind/from_nasal.hxx>
+#include <simgear/nasal/cppbind/to_nasal.hxx>
+#include <simgear/nasal/cppbind/Ghost.hxx>
+#include <simgear/nasal/cppbind/NasalHash.hxx>
+
+#include "NasalSGPath.hxx"
 #include "NasalSys.hxx"
+#include "NasalSys_private.hxx"
+#include "NasalModelData.hxx"
 #include "NasalPositioned.hxx"
 #include "NasalCanvas.hxx"
 #include "NasalClipboard.hxx"
 #include "NasalCondition.hxx"
+#include "NasalHTTP.hxx"
+#include "NasalString.hxx"
 
 #include <Main/globals.hxx>
 #include <Main/util.hxx>
 #include <Main/fg_props.hxx>
 
-
 using std::map;
+using std::string;
+using std::vector;
+
+void postinitNasalGUI(naRef globals, naContext c);
 
 static FGNasalSys* nasalSys = 0;
 
@@ -64,6 +81,97 @@ void FGNasalModuleListener::valueChanged(SGPropertyNode*)
     }
 }
 
+//////////////////////////////////////////////////////////////////////////
+
+
+class TimerObj : public SGReferenced
+{
+public:
+  TimerObj(FGNasalSys* sys, naRef f, naRef self, double interval) :
+    _sys(sys),
+    _func(f),
+    _self(self),
+    _isRunning(false),
+    _interval(interval),
+    _singleShot(false)
+  {
+    char nm[128];
+    snprintf(nm, 128, "nasal-timer-%p", this);
+    _name = nm;
+    _gcRoot =  sys->gcSave(f);
+    _gcSelf = sys->gcSave(self);
+  }
+  
+  virtual ~TimerObj()
+  {
+    stop();
+    _sys->gcRelease(_gcRoot);
+    _sys->gcRelease(_gcSelf);
+  }
+  
+  bool isRunning() const { return _isRunning; }
+    
+  void stop()
+  {
+    if (_isRunning) {
+      globals->get_event_mgr()->removeTask(_name);
+      _isRunning = false;
+    }
+  }
+  
+  void start()
+  {
+    if (_isRunning) {
+      return;
+    }
+    
+    _isRunning = true;
+    if (_singleShot) {
+      globals->get_event_mgr()->addEvent(_name, this, &TimerObj::invoke, _interval);
+    } else {
+      globals->get_event_mgr()->addTask(_name, this, &TimerObj::invoke,
+                                        _interval, _interval /* delay */);
+    }
+  }
+  
+  // stop and then start -
+  void restart(double newInterval)
+  {
+    _interval = newInterval;
+    stop();
+    start();
+  }
+  
+  void invoke()
+  {
+    naRef *args = NULL;
+    _sys->callMethod(_func, _self, 0, args, naNil() /* locals */);
+    if (_singleShot) {
+      _isRunning = false;
+    }
+  }
+  
+  void setSingleShot(bool aSingleShot)
+  {
+    _singleShot = aSingleShot;
+  }
+  
+  bool isSingleShot() const
+  { return _singleShot; }
+private:
+  std::string _name;
+  FGNasalSys* _sys;
+  naRef _func, _self;
+  int _gcRoot, _gcSelf;
+  bool _isRunning;
+  double _interval;
+  bool _singleShot;
+};
+
+typedef SGSharedPtr<TimerObj> TimerObjRef;
+typedef nasal::Ghost<TimerObjRef> NasalTimerObj;
+
+///////////////////////////////////////////////////////////////////////////
 
 // Read and return file contents in a single buffer.  Note use of
 // stat() to get the file size.  This is a win32 function, believe it
@@ -98,9 +206,14 @@ FGNasalSys::FGNasalSys()
     nasalSys = this;
     _context = 0;
     _globals = naNil();
-    _gcHash = naNil();
-    _nextGCKey = 0; // Any value will do
-    _callCount = 0;
+    _string = naNil();
+    _wrappedNodeFunc = naNil();
+    
+    _log = new simgear::BufferedLogCallback(SG_NASAL, SG_INFO);
+    _log->truncateAt(255);
+    sglog().addCallback(_log);
+
+    naSetErrorHandler(&logError);
 }
 
 // Utility.  Sets a named key in a hash by C string, rather than nasal
@@ -131,27 +244,12 @@ naRef FGNasalSys::call(naRef code, int argc, naRef* args, naRef locals)
 
 naRef FGNasalSys::callMethod(naRef code, naRef self, int argc, naRef* args, naRef locals)
 {
-    naContext ctx = naNewContext();
-    if(_callCount) naModUnlock();
-    _callCount++;
-    naRef result = naCall(ctx, code, argc, args, self, locals);
-    if(naGetError(ctx))
-        logError(ctx);
-    _callCount--;
-    if(_callCount) naModLock();
-    naFreeContext(ctx);
-    return result;
+  return naCallMethod(code, self, argc, args, locals);
 }
 
 FGNasalSys::~FGNasalSys()
 {
     nasalSys = 0;
-    map<int, FGNasalListener *>::iterator it, end = _listener.end();
-    for(it = _listener.begin(); it != end; ++it)
-        delete it->second;
-
-    naFreeContext(_context);
-    _globals = naNil();
 }
 
 bool FGNasalSys::parseAndRun(const char* sourceCode)
@@ -164,6 +262,7 @@ bool FGNasalSys::parseAndRun(const char* sourceCode)
     return true;
 }
 
+#if 0
 FGNasalScript* FGNasalSys::parseScript(const char* src, const char* name)
 {
     FGNasalScript* script = new FGNasalScript();
@@ -185,6 +284,7 @@ FGNasalScript* FGNasalSys::parseScript(const char* src, const char* name)
     script->_gcKey = gcSave(script->_code);
     return script;
 }
+#endif
 
 // The get/setprop functions accept a *list* of strings and walk
 // through the property tree with them to find the appropriate node.
@@ -309,6 +409,31 @@ static naRef f_print(naContext c, naRef me, int argc, naRef* args)
     return naNum(buf.length());
 }
 
+// logprint() extension function.  Same as above, all arguments after the
+// first argument are concatenated. Argument 0 is the log-level, matching
+// sgDebugPriority.
+static naRef f_logprint(naContext c, naRef me, int argc, naRef* args)
+{
+  if (argc < 1)
+    naRuntimeError(c, "no prioirty argument to logprint()");
+  
+  naRef priority = args[0];
+  string buf;
+  int n = argc;
+  for(int i=1; i<n; i++) {
+    naRef s = naStringValue(c, args[i]);
+    if(naIsNil(s)) continue;
+    buf += naStr_data(s);
+  }
+// use the nasal source file and line for the message location, since
+// that's more useful than the location here!
+  sglog().log(SG_NASAL, (sgDebugPriority)(int) priority.num,
+               naStr_data(naGetSourceFile(c, 0)),
+               naGetLine(c, 0), buf);
+  return naNum(buf.length());
+}
+
+
 // fgcommand() extension function.  Executes a named command via the
 // FlightGear command manager.  Takes a single property node name as
 // an argument.
@@ -336,6 +461,24 @@ static naRef f_settimer(naContext c, naRef me, int argc, naRef* args)
     return naNil();
 }
 
+static naRef f_makeTimer(naContext c, naRef me, int argc, naRef* args)
+{
+  if (!naIsNum(args[0])) {
+    naRuntimeError(c, "bad interval argument to maketimer");
+  }
+    
+  naRef func, self = naNil();
+  if (naIsFunc(args[1])) {
+    func = args[1];
+  } else if ((argc == 3) && naIsFunc(args[2])) {
+    self = args[1];
+    func = args[2];
+  }
+  
+  TimerObj* timerObj = new TimerObj(nasalSys, func, self, args[0].num);
+  return NasalTimerObj::create(c, timerObj);
+}
+
 // setlistener(func, property, bool) extension function.  Falls through to
 // FGNasalSys::setListener().  See there for docs.
 static naRef f_setlistener(naContext c, naRef me, int argc, naRef* args)
@@ -363,29 +506,31 @@ static naRef f_cmdarg(naContext c, naRef me, int argc, naRef* args)
 // value/delta numbers.
 static naRef f_interpolate(naContext c, naRef me, int argc, naRef* args)
 {
-    SGPropertyNode* node;
-    naRef prop = argc > 0 ? args[0] : naNil();
-    if(naIsString(prop)) node = fgGetNode(naStr_data(prop), true);
-    else if(naIsGhost(prop)) node = *(SGPropertyNode_ptr*)naGhost_ptr(prop);
-    else return naNil();
+  SGPropertyNode* node;
+  naRef prop = argc > 0 ? args[0] : naNil();
+  if(naIsString(prop)) node = fgGetNode(naStr_data(prop), true);
+  else if(naIsGhost(prop)) node = *(SGPropertyNode_ptr*)naGhost_ptr(prop);
+  else return naNil();
 
-    naRef curve = argc > 1 ? args[1] : naNil();
-    if(!naIsVector(curve)) return naNil();
-    int nPoints = naVec_size(curve) / 2;
-    double* values = new double[nPoints];
-    double* deltas = new double[nPoints];
-    for(int i=0; i<nPoints; i++) {
-        values[i] = naNumValue(naVec_get(curve, 2*i)).num;
-        deltas[i] = naNumValue(naVec_get(curve, 2*i+1)).num;
-    }
+  naRef curve = argc > 1 ? args[1] : naNil();
+  if(!naIsVector(curve)) return naNil();
+  int nPoints = naVec_size(curve) / 2;
 
-    ((SGInterpolator*)globals->get_subsystem_mgr()
-        ->get_group(SGSubsystemMgr::INIT)->get_subsystem("interpolator"))
-        ->interpolate(node, nPoints, values, deltas);
+  simgear::PropertyList value_nodes;
+  value_nodes.reserve(nPoints);
+  double_list deltas;
+  deltas.reserve(nPoints);
 
-    delete[] values;
-    delete[] deltas;
-    return naNil();
+  for( int i = 0; i < nPoints; ++i )
+  {
+    SGPropertyNode* val = new SGPropertyNode;
+    val->setDoubleValue(naNumValue(naVec_get(curve, 2*i)).num);
+    value_nodes.push_back(val);
+    deltas.push_back(naNumValue(naVec_get(curve, 2*i+1)).num);
+  }
+
+  node->interpolate("numeric", value_nodes, deltas, "linear");
+  return naNil();
 }
 
 // This is a better RNG than the one in the default Nasal distribution
@@ -440,6 +585,69 @@ static naRef f_resolveDataPath(naContext c, naRef me, int argc, naRef* args)
     return naStr_fromdata(naNewString(c), const_cast<char*>(pdata), strlen(pdata));
 }
 
+static naRef f_findDataDir(naContext c, naRef me, int argc, naRef* args)
+{
+    if(argc != 1 || !naIsString(args[0]))
+        naRuntimeError(c, "bad arguments to findDataDir()");
+    
+    SGPath p = globals->find_data_dir(naStr_data(args[0]));
+    const char* pdata = p.c_str();
+    return naStr_fromdata(naNewString(c), const_cast<char*>(pdata), strlen(pdata));
+}
+
+class NasalCommand : public SGCommandMgr::Command
+{
+public:
+    NasalCommand(FGNasalSys* sys, naRef f, const std::string& name) :
+        _sys(sys),
+        _func(f),
+        _name(name)
+    {
+        globals->get_commands()->addCommandObject(_name, this);
+        _gcRoot =  sys->gcSave(f);
+    }
+    
+    virtual ~NasalCommand()
+    {
+        _sys->gcRelease(_gcRoot);
+    }
+    
+    virtual bool operator()(const SGPropertyNode* aNode)
+    {
+        _sys->setCmdArg(const_cast<SGPropertyNode*>(aNode));
+        naRef args[1];
+        args[0] = _sys->wrappedPropsNode(const_cast<SGPropertyNode*>(aNode));
+    
+        _sys->callMethod(_func, naNil(), 1, args, naNil() /* locals */);
+
+        return true;
+    }
+    
+private:
+    FGNasalSys* _sys;
+    naRef _func;
+    int _gcRoot;
+    std::string _name;
+};
+
+static naRef f_addCommand(naContext c, naRef me, int argc, naRef* args)
+{
+    if(argc != 2 || !naIsString(args[0]) || !naIsFunc(args[1]))
+        naRuntimeError(c, "bad arguments to addcommand()");
+    
+    nasalSys->addCommand(args[1], naStr_data(args[0]));
+    return naNil();
+}
+
+static naRef f_removeCommand(naContext c, naRef me, int argc, naRef* args)
+{
+    if ((argc < 1) || !naIsString(args[0]))
+        naRuntimeError(c, "bad argument to removecommand()");
+    
+    globals->get_commands()->removeCommand(naStr_data(args[0]));
+    return naNil();
+}
+
 // Parse XML file.
 //     parsexml(<path> [, <start-tag> [, <end-tag> [, <data> [, <pi>]]]]);
 //
@@ -499,10 +707,14 @@ static struct { const char* name; naCFunction func; } funcs[] = {
     { "getprop",   f_getprop },
     { "setprop",   f_setprop },
     { "print",     f_print },
+    { "logprint",  f_logprint },
     { "_fgcommand", f_fgcommand },
     { "settimer",  f_settimer },
+    { "maketimer", f_makeTimer },
     { "_setlistener", f_setlistener },
     { "removelistener", f_removelistener },
+    { "addcommand", f_addCommand },
+    { "removecommand", f_removeCommand },
     { "_cmdarg",  f_cmdarg },
     { "_interpolate",  f_interpolate },
     { "rand",  f_rand },
@@ -510,6 +722,7 @@ static struct { const char* name; naCFunction func; } funcs[] = {
     { "abort", f_abort },
     { "directory", f_directory },
     { "resolvepath", f_resolveDataPath },
+    { "finddata", f_findDataDir },
     { "parsexml", f_parsexml },
     { "systime", f_systime },
     { 0, 0 }
@@ -520,6 +733,11 @@ naRef FGNasalSys::cmdArgGhost()
     return propNodeGhost(_cmdArg);
 }
 
+void FGNasalSys::setCmdArg(SGPropertyNode* aNode)
+{
+    _cmdArg = aNode;
+}
+
 void FGNasalSys::init()
 {
     int i;
@@ -545,22 +763,29 @@ void FGNasalSys::init()
         hashset(_globals, funcs[i].name,
                 naNewFunc(_context, naNewCCode(_context, funcs[i].func)));
 
-
-  
     // And our SGPropertyNode wrapper
     hashset(_globals, "props", genPropsModule());
 
-    // Make a "__gcsave" hash to hold the naRef objects which get
-    // passed to handles outside the interpreter (to protect them from
-    // begin garbage-collected).
-    _gcHash = naNewHash(_context);
-    hashset(_globals, "__gcsave", _gcHash);
+    // Add string methods
+    _string = naInit_string(_context);
+    naSave(_context, _string);
+    initNasalString(_globals, _string, _context);
 
-    initNasalPositioned(_globals, _context, _gcHash);
+    initNasalPositioned(_globals, _context);
+    initNasalPositioned_cppbind(_globals, _context);
     NasalClipboard::init(this);
-    initNasalCanvas(_globals, _context, _gcHash);
-    initNasalCondition(_globals, _context, _gcHash);
+    initNasalCanvas(_globals, _context);
+    initNasalCondition(_globals, _context);
+    initNasalHTTP(_globals, _context);
+    initNasalSGPath(_globals, _context);
   
+    NasalTimerObj::init("Timer")
+      .method("start", &TimerObj::start)
+      .method("stop", &TimerObj::stop)
+      .method("restart", &TimerObj::restart)
+      .member("singleShot", &TimerObj::isSingleShot, &TimerObj::setSingleShot)
+      .member("isRunning", &TimerObj::isRunning);
+
     // Now load the various source files in the Nasal directory
     simgear::Dir nasalDir(SGPath(globals->get_fg_root(), "Nasal"));
     loadScriptDirectory(nasalDir);
@@ -585,6 +810,57 @@ void FGNasalSys::init()
   
     // now Nasal modules are loaded, we can do some delayed work
     postinitNasalPositioned(_globals, _context);
+    postinitNasalGUI(_globals, _context);
+}
+
+void FGNasalSys::shutdown()
+{
+    shutdownNasalPositioned();
+    
+    map<int, FGNasalListener *>::iterator it, end = _listener.end();
+    for(it = _listener.begin(); it != end; ++it)
+        delete it->second;
+    _listener.clear();
+    
+    NasalCommandDict::iterator j = _commands.begin();
+    for (; j != _commands.end(); ++j) {
+        globals->get_commands()->removeCommand(j->first);
+    }
+    _commands.clear();
+    
+    std::vector<FGNasalModuleListener*>::iterator k = _moduleListeners.begin();
+    for(; k!= _moduleListeners.end(); ++k)
+        delete *k;
+    _moduleListeners.clear();
+    
+    naClearSaved();
+    
+    _string = naNil(); // will be freed by _context
+    naFreeContext(_context);
+   
+    //setWatchedRef(_globals);
+    
+    // remove the recursive reference in globals
+    hashset(_globals, "globals", naNil());
+    _globals = naNil();    
+    
+    naGC();
+    
+}
+
+naRef FGNasalSys::wrappedPropsNode(SGPropertyNode* aProps)
+{
+    if (naIsNil(_wrappedNodeFunc)) {
+        nasal::Hash props = getGlobals().get<nasal::Hash>("props");
+        _wrappedNodeFunc = props.get("wrapNode");
+    }
+    
+    naRef args[1];
+    args[0] = propNodeGhost(aProps);
+    naContext ctx = naNewContext();
+    naRef wrapped = naCall(ctx, _wrappedNodeFunc, 1, args, naNil(), naNil());
+    naFreeContext(ctx);
+    return wrapped;
 }
 
 void FGNasalSys::update(double)
@@ -600,8 +876,11 @@ void FGNasalSys::update(double)
 
     if (!_loadList.empty())
     {
-        // process Nasal load hook (only one per update loop to avoid excessive lags)
-        _loadList.pop()->load();
+        if( _delay_load )
+          _delay_load = false;
+        else
+          // process Nasal load hook (only one per update loop to avoid excessive lags)
+          _loadList.pop()->load();
     }
     else
     if (!_unloadList.empty())
@@ -646,7 +925,7 @@ void FGNasalSys::loadScriptDirectory(simgear::Dir nasalDir)
 // Create module with list of scripts
 void FGNasalSys::addModule(string moduleName, simgear::PathList scripts)
 {
-    if (scripts.size()>0)
+    if (! scripts.empty())
     {
         SGPropertyNode* nasal = globals->get_props()->getNode("nasal");
         SGPropertyNode* module_node = nasal->getChild(moduleName,0,true);
@@ -735,6 +1014,7 @@ void FGNasalSys::loadPropertyScripts(SGPropertyNode* n)
         if (enable)
         {
             FGNasalModuleListener* listener = new FGNasalModuleListener(n);
+            _moduleListeners.push_back(listener);
             enable->addChangeListener(listener, false);
         }
     }
@@ -746,12 +1026,14 @@ void FGNasalSys::loadPropertyScripts(SGPropertyNode* n)
 // Logs a runtime error, with stack trace, to the FlightGear log stream
 void FGNasalSys::logError(naContext context)
 {
-    SG_LOG(SG_NASAL, SG_ALERT,
-           "Nasal runtime error: " << naGetError(context));
+    SG_LOG(SG_NASAL, SG_ALERT, "Nasal runtime error: " << naGetError(context));
+    int stack_depth = naStackDepth(context);
+    if( stack_depth < 1 )
+      return;
     SG_LOG(SG_NASAL, SG_ALERT,
            "  at " << naStr_data(naGetSourceFile(context, 0)) <<
            ", line " << naGetLine(context, 0));
-    for(int i=1; i<naStackDepth(context); i++)
+    for(int i=1; i<stack_depth; i++)
         SG_LOG(SG_NASAL, SG_ALERT,
                "  called from: " << naStr_data(naGetSourceFile(context, i)) <<
                ", line " << naGetLine(context, i));
@@ -789,44 +1071,54 @@ bool FGNasalSys::createModule(const char* moduleName, const char* fileName,
     if(naIsNil(code))
         return false;
 
+    naContext ctx = naNewContext();
+    
     // See if we already have a module hash to use.  This allows the
     // user to, for example, add functions to the built-in math
     // module.  Make a new one if necessary.
     naRef locals;
-    naRef modname = naNewString(_context);
+    naRef modname = naNewString(ctx);
     naStr_fromdata(modname, (char*)moduleName, strlen(moduleName));
     if(!naHash_get(_globals, modname, &locals))
-        locals = naNewHash(_context);
+        locals = naNewHash(ctx);
 
     _cmdArg = (SGPropertyNode*)cmdarg;
 
     call(code, argc, args, locals);
     hashset(_globals, moduleName, locals);
+    
+    naFreeContext(ctx);
     return true;
 }
 
 void FGNasalSys::deleteModule(const char* moduleName)
 {
-    naRef modname = naNewString(_context);
+    naContext ctx = naNewContext();
+    naRef modname = naNewString(ctx);
     naStr_fromdata(modname, (char*)moduleName, strlen(moduleName));
     naHash_delete(_globals, modname);
+    naFreeContext(ctx);
 }
 
 naRef FGNasalSys::parse(const char* filename, const char* buf, int len)
 {
     int errLine = -1;
-    naRef srcfile = naNewString(_context);
+    naContext ctx = naNewContext();
+    naRef srcfile = naNewString(ctx);
     naStr_fromdata(srcfile, (char*)filename, strlen(filename));
-    naRef code = naParseCode(_context, srcfile, 1, (char*)buf, len, &errLine);
+    naRef code = naParseCode(ctx, srcfile, 1, (char*)buf, len, &errLine);
     if(naIsNil(code)) {
         SG_LOG(SG_NASAL, SG_ALERT,
-               "Nasal parse error: " << naGetError(_context) <<
+               "Nasal parse error: " << naGetError(ctx) <<
                " in "<< filename <<", line " << errLine);
+        naFreeContext(ctx);
         return naNil();
     }
 
     // Bind to the global namespace before returning
-    return naBindFunction(_context, code, _globals);
+    naRef bound = naBindFunction(ctx, code, _globals);
+    naFreeContext(ctx);
+    return bound;
 }
 
 bool FGNasalSys::handleCommand( const char* moduleName,
@@ -842,12 +1134,14 @@ bool FGNasalSys::handleCommand( const char* moduleName,
     // command.
     naRef locals = naNil();
     if(moduleName[0]) {
-        naRef modname = naNewString(_context);
+        naContext ctx = naNewContext();
+        naRef modname = naNewString(ctx);
         naStr_fromdata(modname, (char*)moduleName, strlen(moduleName));
         if(!naHash_get(_globals, modname, &locals)) {
-            locals = naNewHash(_context);
+            locals = naNewHash(ctx);
             naHash_set(_globals, modname, locals);
         }
+        naFreeContext(ctx);
     }
 
     // Cache this command's argument for inspection via cmdarg().  For
@@ -916,14 +1210,12 @@ void FGNasalSys::handleTimer(NasalTimer* t)
 
 int FGNasalSys::gcSave(naRef r)
 {
-    int key = _nextGCKey++;
-    naHash_set(_gcHash, naNum(key), r);
-    return key;
+    return naGCSave(r);
 }
 
 void FGNasalSys::gcRelease(int key)
 {
-    naHash_delete(_gcHash, naNum(key));
+    naGCRelease(key);
 }
 
 void FGNasalSys::NasalTimer::timerExpired()
@@ -994,8 +1286,43 @@ naRef FGNasalSys::removeListener(naContext c, int argc, naRef* args)
     return naNum(_listener.size());
 }
 
+void FGNasalSys::registerToLoad(FGNasalModelData *data)
+{
+  if( _loadList.empty() )
+    _delay_load = true;
+  _loadList.push(data);
+}
 
+void FGNasalSys::registerToUnload(FGNasalModelData *data)
+{
+    _unloadList.push(data);
+}
 
+void FGNasalSys::addCommand(naRef func, const std::string& name)
+{
+    if (_commands.find(name) != _commands.end()) {
+        SG_LOG(SG_NASAL, SG_WARN, "duplicate add of command:" << name);
+        return;
+    }
+    
+    NasalCommand* cmd = new NasalCommand(this, func, name);
+    _commands[name] = cmd;
+}
+
+void FGNasalSys::removeCommand(const std::string& name)
+{
+    NasalCommandDict::iterator it = _commands.find(name);
+    if (it == _commands.end()) {
+        SG_LOG(SG_NASAL, SG_WARN, "remove of unknwon command:" << name);
+        return;
+    }
+
+    // will delete the NasalCommand instance
+    globals->get_commands()->removeCommand(name);
+    _commands.erase(it);
+}
+
+//////////////////////////////////////////////////////////////////////////
 // FGNasalListener class.
 
 FGNasalListener::FGNasalListener(SGPropertyNode *node, naRef code,
@@ -1026,7 +1353,6 @@ FGNasalListener::~FGNasalListener()
 void FGNasalListener::call(SGPropertyNode* which, naRef mode)
 {
     if(_active || _dead) return;
-    SG_LOG(SG_NASAL, SG_DEBUG, "trigger listener #" << _id);
     _active++;
     naRef arg[4];
     arg[0] = _nas->propNodeGhost(which);
@@ -1092,90 +1418,6 @@ bool FGNasalListener::changed(SGPropertyNode* node)
     }
 }
 
-
-
-// FGNasalModelData class.  If sgLoad3DModel() is called with a pointer to
-// such a class, then it lets modelLoaded() run the <load> script, and the
-// destructor the <unload> script. The latter happens when the model branch
-// is removed from the scene graph.
-
-unsigned int FGNasalModelData::_module_id = 0;
-
-void FGNasalModelData::load()
-{
-    std::stringstream m;
-    m << "__model" << _module_id++;
-    _module = m.str();
-
-    SG_LOG(SG_NASAL, SG_DEBUG, "Loading nasal module " << _module.c_str());
-
-    const char *s = _load ? _load->getStringValue() : "";
-
-    naRef arg[2];
-    arg[0] = nasalSys->propNodeGhost(_root);
-    arg[1] = nasalSys->propNodeGhost(_prop);
-    nasalSys->createModule(_module.c_str(), _path.c_str(), s, strlen(s),
-                           _root, 2, arg);
-}
-
-void FGNasalModelData::unload()
-{
-    if (_module.empty())
-        return;
-
-    if(!nasalSys) {
-        SG_LOG(SG_NASAL, SG_WARN, "Trying to run an <unload> script "
-                "without Nasal subsystem present.");
-        return;
-    }
-
-    SG_LOG(SG_NASAL, SG_DEBUG, "Unloading nasal module " << _module.c_str());
-
-    if (_unload)
-    {
-        const char *s = _unload->getStringValue();
-        nasalSys->createModule(_module.c_str(), _module.c_str(), s, strlen(s), _root);
-    }
-
-    nasalSys->deleteModule(_module.c_str());
-}
-
-void FGNasalModelDataProxy::modelLoaded(const string& path, SGPropertyNode *prop,
-                                   osg::Node *)
-{
-    if(!nasalSys) {
-        SG_LOG(SG_NASAL, SG_WARN, "Trying to run a <load> script "
-                "without Nasal subsystem present.");
-        return;
-    }
-
-    if(!prop)
-        return;
-
-    SGPropertyNode *nasal = prop->getNode("nasal");
-    if(!nasal)
-        return;
-
-    SGPropertyNode* load   = nasal->getNode("load");
-    SGPropertyNode* unload = nasal->getNode("unload");
-
-    if ((!load) && (!unload))
-        return;
-
-    _data = new FGNasalModelData(_root, path, prop, load, unload);
-
-    // register Nasal module to be created and loaded in the main thread.
-    nasalSys->registerToLoad(_data);
-}
-
-FGNasalModelDataProxy::~FGNasalModelDataProxy()
-{
-    // when necessary, register Nasal module to be destroyed/unloaded
-    // in the main thread.
-    if ((_data.valid())&&(nasalSys))
-        nasalSys->registerToUnload(_data);
-}
-
 // NasalXMLVisitor class: handles EasyXML visitor callback for parsexml()
 //
 NasalXMLVisitor::NasalXMLVisitor(naContext c, int argc, naRef* args) :
